// ==============================================================
//
// f9h_stg1_centre_expendable
// Control module for f9h_stg1_centre_expendable vessel class
//
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"
#include <cstdio>
#include <cmath>
#include <string>
#include <stdio.h>

// ==============================================================
// Some vessel parameters
// ==============================================================

const DWORD COMM_SEND_STR = 2300;
const DWORD COMM_SEND_AFS = 2301;
const DWORD COMM_SEND_EDLO = 2304;
const double ISP0 = 3062;//2767;
const double ISPV = 3311;//3051;
const double MAINTH0 = 845222;//756222;
const double MAINTH1 = 914111;//825111;
const double RCSTH = 38000;
const double RCSTH2 = 1000;
const double FUELMASS = 402500;//401500;//396500;//384000
const double DRYMASS = 22100;//19000;

const VECTOR3 INT_OFF = _V(0,0,0);
const double RCS2MASS = 400;
const double RCS2ISPV = 800;
const double RCS2ISP0 = 800;
#define GREEN RGB(125, 255, 125)

OBJHANDLE hGref;

static const DWORD ntdvtx_gearup = 4;
static TOUCHDOWNVTX tdvtx_gearup[ntdvtx_gearup] = {
	{_V(0, -5,-20), 1e6, 1e5, 1, 1},
	{_V(-4.33, 2.5,-20), 1e6, 1e5, 1, 1},
	{_V( 4.33, 2.5,-20), 1e6, 1e5, 1, 1},
	{_V(0 ,0 , 20), 1e6, 1e5, 1}
};

void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*22.5;
	static const double istep = 1.0/step;
	static const int nabsc = 17;
	static const double CL[nabsc] = {0, 0.1, 0.2, 0.1, 0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0, -0.1, -0.2, -0.1, 0};


	aoa += PI;
	int idx = max (0, min (15, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = 0.0;
	*cd = 0.55*(0.11 + oapiGetInducedDrag (*cl, 0.16, 0.2));
	
	
}

// 2. horizontal lift component (vertical stabiliser and body)

void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*22.5;
	static const double istep = 1.0/step;
	static const int nabsc = 17;
	static const double CL[nabsc] = {0, 0.1, 0.2, 0.1, 0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0, -0.1, -0.2, -0.1, 0};

	beta += PI;
	int idx = max (0, min (15, (int)(beta*istep)));
	double d = beta*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = 0.0;
	*cd = 0.55*(0.11 + oapiGetInducedDrag (*cl, 0.16, 0.2));
	
}

// ==============================================================
// Shuttle-PB class interface
// ==============================================================

class f9h_stg1_centre_expendable: public VESSEL3 {
public:
	f9h_stg1_centre_expendable (OBJHANDLE hVessel, int flightmodel);
	~f9h_stg1_centre_expendable ();
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPostCreation();
	void clbkPreStep(double SimT, double SimDT, double MJD);
	void clbkPostStep (double simt, double simdt, double mjd);
	void clbkSaveState (FILEHANDLE scn);
	void clbkLoadStateEx (FILEHANDLE scn, void *status);
	int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);
	bool clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp);
	bool clbkLoadGenericCockpit ();
	
	
private:

	
	EXHAUSTSPEC main_exhaust[9];
	EXHAUSTSPEC main_exhaust1[4];
	EXHAUSTSPEC main_exhaust2[2];
	THRUSTER_HANDLE th_main[3],th_rcs[12], th_group[9], th_rcs2[12], th_boom[2];
	THGROUP_HANDLE htgrpmain, htgrppup,htgrppdown,htgrpbright, htgrpbleft, htgrpyright, htgrpyleft;
	PROPELLANT_HANDLE hProp, hRCSProp;
	PSTREAM_HANDLE at_trail;
	CTRLSURFHANDLE hElevator, hRudder;
	SURFHANDLE bs1, bs2, bs3, bs4;
	MESHHANDLE hMhull, hMintstg;
	//PointLight *pnt_light;


	void ConfigMain(void);
	void ConfigRCS(void);
	void RevertRCS(void);

	void SetCamDir();
	void KillAllRCS();

	void ApplySkin();
	void DoThrustVector();


	double skin_sw;//pitch_angle,
	int eng_status, landed_sw;
	int cam_status, rot_sw, rcs_sw, rc_sw;

	double  vrot_lim;
	
	VECTOR3 torq_vect;
	double pu_level, pd_level, yr_level, yl_level, mth_level;
};

f9h_stg1_centre_expendable::f9h_stg1_centre_expendable (OBJHANDLE hVessel, int flightmodel)
: VESSEL3 (hVessel, flightmodel)
{
    

}

f9h_stg1_centre_expendable::~f9h_stg1_centre_expendable ()
{
	


}

// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void f9h_stg1_centre_expendable::clbkSetClassCaps (FILEHANDLE cfg)
{
	cam_status = 0;
	//pitch_angle = 90*RAD;

	//intfair_status = 0;
	eng_status = 0;
	//calc_sw = 0;
	rot_sw = 0;
	rcs_sw = 0;
	rc_sw = 0;
	vrot_lim = 5*RAD;
	
	landed_sw  = 0;
	skin_sw = 0;


	torq_vect = _V(0,0,0);

	pu_level = 0;
	pd_level = 0;
	yr_level = 0;
	yl_level = 0;
	mth_level = 0;

	// physical vessel parameters
	SetSize (20);
	SetEmptyMass (DRYMASS);
	SetPMI(_V(129,129,1.9));
	SetCrossSections (_V(159,159,16));
	SetCW(0.5,4,1,1);
	SetRotDrag (_V(0.25,0.25,0.1));
	SetTouchdownPoints ( tdvtx_gearup, ntdvtx_gearup);

	// camera parameters
	SetCameraOffset (_V(0,3,7.5));
	SetCameraRotationRange(PI,PI,(0.4*PI),(0.4*PI));

	hMhull = oapiLoadMesh("FalconHeavy/f9h_stg1_centre_nolegs");
	SetMeshVisibilityMode(AddMesh(hMhull),MESHVIS_ALWAYS);

	hMintstg = oapiLoadMesh("FalconHeavy/f9h_interstg_stowed_nolegs");
	SetMeshVisibilityMode(AddMesh(hMintstg, &INT_OFF),MESHVIS_ALWAYS);

	bs2 = oapiLoadTexture ("FalconHeavy/f9_hull_logo_char.dds", true);
	bs4 = oapiLoadTexture ("FalconHeavy/f9logo_char.dds", true);
	// propellant resources
	
	hProp = CreatePropellantResource (FUELMASS,FUELMASS);
	hRCSProp = CreatePropellantResource (RCS2MASS,RCS2MASS);

	// ***************** thruster definitions *******************

	th_rcs2[0] = CreateThruster (_V( 1.95, 0, 18.8), _V( -1,0,0), RCSTH2, hRCSProp, RCS2ISPV, RCS2ISP0);
	th_rcs2[1] = CreateThruster (_V( 0, 1.95, 18.8), _V( 0,-1,0), RCSTH2, hRCSProp, RCS2ISPV, RCS2ISP0);
	th_rcs2[2] = CreateThruster (_V( -1.95, 0, 18.8), _V( 1,0,0), RCSTH2, hRCSProp, RCS2ISPV, RCS2ISP0);
	th_rcs2[3] = CreateThruster (_V( 0, -1.95, 18.8), _V( 0,1,0), RCSTH2, hRCSProp, RCS2ISPV, RCS2ISP0);

	th_rcs2[4] = CreateThruster (_V( 1.95, 0, -18.12), _V( -1,0,0), RCSTH2, hRCSProp, RCS2ISPV, RCS2ISP0);
	th_rcs2[5] = CreateThruster (_V( 0, 1.95, -18.12), _V( 0,-1,0), RCSTH2, hRCSProp, RCS2ISPV, RCS2ISP0);
	th_rcs2[6] = CreateThruster (_V( -1.95, 0, -18.12), _V( 1,0,0), RCSTH2, hRCSProp, RCS2ISPV, RCS2ISP0);
	th_rcs2[7] = CreateThruster (_V( 0, -1.95, -18.12), _V( 0,1,0), RCSTH2, hRCSProp, RCS2ISPV, RCS2ISP0);

	th_rcs2[8] = CreateThruster (_V( 1.884, 0.103, 18.963), _V( 0,-1,0), RCSTH2, hRCSProp, RCS2ISPV, RCS2ISP0);
	th_rcs2[9] = CreateThruster (_V( 1.884, -0.103, 18.963), _V( 0,1,0), RCSTH2, hRCSProp, RCS2ISPV, RCS2ISP0);
	th_rcs2[10] = CreateThruster (_V( -1.884, 0.103, 18.963), _V( 0,-1,0), RCSTH2, hRCSProp, RCS2ISPV, RCS2ISP0);
	th_rcs2[11] = CreateThruster (_V( -1.884, -0.103, 18.963), _V( 0,1,0), RCSTH2, hRCSProp, RCS2ISPV, RCS2ISP0);
	//th_rcs2[11] = CreateThruster (_V( -1.884, -0.103, 18.963), _V( 0,1,0), RCSTH2, hProp, ISPV, ISP0);
	
	////////////////////////////

	th_rcs[0] = CreateThruster (_V( 0, 2, -21), _V( 1,0,0), RCSTH, hProp, ISPV, ISP0);
	th_rcs[1] = CreateThruster (_V( 0, 2, -21), _V(-1,0,0), RCSTH, hProp, ISPV, ISP0);
	th_rcs[2] = CreateThruster (_V( 0,-2, -21), _V( 1,0,0), RCSTH, hProp, ISPV, ISP0);
	th_rcs[3] = CreateThruster (_V( 0,-2, -21), _V(-1,0,0), RCSTH, hProp, ISPV, ISP0);

	th_rcs[4] = CreateThruster (_V( 0, 4, -21), _V( 1,0,0), RCSTH, hProp, ISPV, ISP0);
	th_rcs[5] = CreateThruster (_V( 0, 4, -21), _V(-1,0,0), RCSTH, hProp, ISPV, ISP0);
	th_rcs[6] = CreateThruster (_V( 0,-4, -21), _V( 1,0,0), RCSTH, hProp, ISPV, ISP0);
	th_rcs[7] = CreateThruster (_V( 0,-4, -21), _V(-1,0,0), RCSTH, hProp, ISPV, ISP0);

	th_rcs[8] = CreateThruster (_V( 2, 0, -21), _V(0, 1,0), RCSTH, hProp, ISPV, ISP0);
	th_rcs[9] = CreateThruster (_V( 2, 0, -21), _V(0,-1,0), RCSTH, hProp, ISPV, ISP0);
	th_rcs[10] = CreateThruster (_V(-2,0, -21), _V(0, 1,0), RCSTH, hProp, ISPV, ISP0);
	th_rcs[11] = CreateThruster (_V(-2,0, -21), _V(0,-1,0), RCSTH, hProp, ISPV, ISP0);

	th_group[0] = th_rcs[9];
	th_group[1] = th_rcs[11];
	htgrppup = CreateThrusterGroup (th_group, 2, THGROUP_ATT_PITCHUP);

	th_group[0] = th_rcs[8];
	th_group[1] = th_rcs[10];
	htgrppdown = CreateThrusterGroup (th_group, 2, THGROUP_ATT_PITCHDOWN);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[3];
	htgrpyright = CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWRIGHT);

	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[2];
	htgrpyleft = CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWLEFT);

	th_group[0] = th_rcs[4];
	th_group[1] = th_rcs[7];
	htgrpbright = CreateThrusterGroup (th_group, 2, THGROUP_ATT_BANKRIGHT);

	th_group[0] = th_rcs[5];
	th_group[1] = th_rcs[6];
	htgrpbleft = CreateThrusterGroup (th_group, 2, THGROUP_ATT_BANKLEFT);


	th_main[0] = CreateThruster (_V( 0, 0, -21), _V( 0,0,1), 9*MAINTH1, hProp, ISPV, ISP0);
	th_main[1] = CreateThruster (_V( 0, 0, -21), _V( 0,0,1), 3*MAINTH1, hProp, ISPV, ISP0);
	th_main[2] = CreateThruster (_V( 0, 0, -21), _V( 0,0,1), MAINTH1, hProp, ISPV, ISP0);

	th_group[0] = th_main[0];
	htgrpmain = CreateThrusterGroup (th_group, 1, THGROUP_MAIN);

	SURFHANDLE exhaust = oapiRegisterExhaustTexture ("FalconHeavy/KeroExhaust3");
//	SURFHANDLE exhaust = oapiRegisterExhaustTexture ("Exhaust2");
	SURFHANDLE exhaust_rcs = oapiRegisterExhaustTexture ("exhaust_atrcs");

	double exlenrcs = 6;
	double exwidrcs = 0.6;

	AddExhaust(th_rcs2[0],exlenrcs,exwidrcs,exhaust_rcs);
	AddExhaust(th_rcs2[1],exlenrcs,exwidrcs,_V(1.884,  0.103, 18.963), _V( 0,1,0),exhaust_rcs);
	AddExhaust(th_rcs2[2],exlenrcs,exwidrcs,exhaust_rcs);
	AddExhaust(th_rcs2[3],exlenrcs,exwidrcs,_V(1.884,-0.103, 18.963), _V( 0,-1,0),exhaust_rcs);
	AddExhaust(th_rcs2[4],exlenrcs,exwidrcs,exhaust_rcs);
	AddExhaust(th_rcs2[5],exlenrcs,exwidrcs,_V(-1.884,-0.103, 18.963), _V( 0,-1,0),exhaust_rcs);
	AddExhaust(th_rcs2[6],exlenrcs,exwidrcs,exhaust_rcs);
	AddExhaust(th_rcs2[7],exlenrcs,exwidrcs,_V(-1.884, 0.103, 18.963), _V( 0,1,0),exhaust_rcs);
	AddExhaust(th_rcs2[8],exlenrcs,exwidrcs,exhaust_rcs);
	AddExhaust(th_rcs2[9],exlenrcs,exwidrcs,exhaust_rcs);
	AddExhaust(th_rcs2[10],exlenrcs,exwidrcs,exhaust_rcs);
	AddExhaust(th_rcs2[11],exlenrcs,exwidrcs,exhaust_rcs);




	double exlen = 30;
	double exwid = 3.7;

	MATRIX3 rm;
	VECTOR3 exhaust_pos[9];
	VECTOR3 exhaust_pos1[4];
	VECTOR3 exhaust_pos2[2];

	for (int i = 0; i < 8; i++) // core
	{
	rm = rotm(_V(0,0,1),(45*RAD*i));
	exhaust_pos[i] = mul(rm,_V(0,1.32,-20.5));

	main_exhaust[i].th = th_main[0];
	main_exhaust[i].level = NULL;
	main_exhaust[i].lpos = &exhaust_pos[i];
	main_exhaust[i].ldir = NULL; //&main2_exhaust_dir;
	main_exhaust[i].lsize = exlen;
	main_exhaust[i].wsize = exwid;
	main_exhaust[i].lofs = 0;
	main_exhaust[i].modulate = 0.8;
	main_exhaust[i].tex = exhaust;
	main_exhaust[i].flags = EXHAUST_CONSTANTPOS;
	main_exhaust[i].id = 0;
	AddExhaust (&main_exhaust[i]);
	}
	{
	int i = 8;
	exhaust_pos[i] = _V(0,0,-20.5);

	main_exhaust[i].th = th_main[0];
	main_exhaust[i].level = NULL;
	main_exhaust[i].lpos = &exhaust_pos[i];
	main_exhaust[i].ldir = NULL; //&main2_exhaust_dir;
	main_exhaust[i].lsize = exlen;
	main_exhaust[i].wsize = exwid;
	main_exhaust[i].lofs = 0;
	main_exhaust[i].modulate = 0.8;
	main_exhaust[i].tex = exhaust;
	main_exhaust[i].flags = EXHAUST_CONSTANTPOS;
	main_exhaust[i].id = 0;
	AddExhaust (&main_exhaust[i]);
	}
	///////////////////////////// 3  engines
	for (int i = 0; i < 2; i++) // core
	{
	rm = rotm(_V(0,0,1),(180*RAD*i));
	exhaust_pos[i] = mul(rm,_V(1.32, 0,-20.5));

	main_exhaust1[i].th = th_main[1];
	main_exhaust1[i].level = NULL;
	main_exhaust1[i].lpos = &exhaust_pos[i];
	main_exhaust1[i].ldir = NULL; //&main2_exhaust_dir;
	main_exhaust1[i].lsize = exlen;
	main_exhaust1[i].wsize = exwid;
	main_exhaust1[i].lofs = 0;
	main_exhaust1[i].modulate = 0.8;
	main_exhaust1[i].tex = exhaust;
	main_exhaust1[i].flags = EXHAUST_CONSTANTPOS;
	main_exhaust1[i].id = 0;
	AddExhaust (&main_exhaust1[i]);
	}
	for (int i = 2; i < 4; i++) // core
	{
	exhaust_pos1[i] = _V(0, 0,-20.5);

	main_exhaust1[i].th = th_main[1];
	main_exhaust1[i].level = NULL;
	main_exhaust1[i].lpos = &exhaust_pos1[i];
	main_exhaust1[i].ldir = NULL; //&main2_exhaust_dir;
	main_exhaust1[i].lsize = exlen;
	main_exhaust1[i].wsize = exwid;
	main_exhaust1[i].lofs = 0;
	main_exhaust1[i].modulate = 0.8;
	main_exhaust1[i].tex = exhaust;
	main_exhaust1[i].flags = EXHAUST_CONSTANTPOS;
	main_exhaust1[i].id = 0;
	AddExhaust (&main_exhaust1[i]);
	}
	///// 1 engine ////////////////////////
	for (int i = 0; i < 2; i++)
	{
	exhaust_pos2[i] = mul(rm,_V(0, 0,-20.5));

	main_exhaust2[i].th = th_main[2];
	main_exhaust2[i].level = NULL;
	main_exhaust2[i].lpos = &exhaust_pos2[i];
	main_exhaust2[i].ldir = NULL; //&main2_exhaust_dir;
	main_exhaust2[i].lsize = exlen;
	main_exhaust2[i].wsize = exwid;
	main_exhaust2[i].lofs = 0;
	main_exhaust2[i].modulate = 0.8;
	main_exhaust2[i].tex = exhaust;
	main_exhaust2[i].flags = EXHAUST_CONSTANTPOS;
	main_exhaust2[i].id = 0;
	AddExhaust (&main_exhaust2[i]);
	}

	/*AddExhaust(th_main[0],exlen,exwid,_V(0,0,-20.5),_V(0,0,-1),exhaust);
	AddExhaust(th_main[0],exlen,exwid,_V(1.32,0,-20.5),_V(0,0,-1),exhaust);
	AddExhaust(th_main[0],exlen,exwid,_V(-1.32,0,-20.5),_V(0,0,-1),exhaust);
	AddExhaust(th_main[0],exlen,exwid,_V(0,1.32,-20.5),_V(0,0,-1),exhaust);
	AddExhaust(th_main[0],exlen,exwid,_V(0,-1.32,-20.5),_V(0,0,-1),exhaust);
	AddExhaust(th_main[0],exlen,exwid,_V(0.931,0.931,-20.5),_V(0,0,-1),exhaust);
	AddExhaust(th_main[0],exlen,exwid,_V(-0.931,-0.931,-20.5),_V(0,0,-1),exhaust);
	AddExhaust(th_main[0],exlen,exwid,_V(-0.931,0.931,-20.5),_V(0,0,-1),exhaust);
	AddExhaust(th_main[0],exlen,exwid,_V(0.931,-0.931,-20.5),_V(0,0,-1),exhaust);

	AddExhaust(th_main[1],exlen,exwid,_V(0,0,-20.5),_V(0,0,-1),exhaust);
	AddExhaust(th_main[1],exlen,exwid,_V(0,0,-20.5),_V(0,0,-1),exhaust);
	AddExhaust(th_main[1],exlen,exwid,_V(1.32,0,-20.5),_V(0,0,-1),exhaust);
	AddExhaust(th_main[1],exlen,exwid,_V(-1.32,0,-20.5),_V(0,0,-1),exhaust);

	AddExhaust(th_main[2],exlen,exwid,_V(0,0,-20.5),_V(0,0,-1),exhaust);
	AddExhaust(th_main[2],exlen,exwid,_V(0,0,-20.5),_V(0,0,-1),exhaust);*/

	SURFHANDLE exhaust2 = oapiRegisterParticleTexture ("FalconHeavy/Contrail4x");
	SURFHANDLE exhaust2x = oapiRegisterParticleTexture ("FalconHeavy/Contrail3");
	SURFHANDLE exhaustrcs = oapiRegisterParticleTexture ("Contrail1");

	PARTICLESTREAMSPEC exhaust_main1 = {
		//0, 1, 20, 120, 0.044, 0.11, 2.6, 0.3, PARTICLESTREAMSPEC::EMISSIVE,
		0, 1.0, 20, 120, 0.01, 0.08, 2.6, 1, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_FLAT, 1, 0,
		PARTICLESTREAMSPEC::ATM_FLAT, 1, 1,
		exhaust2
	};

	PARTICLESTREAMSPEC exhaust_main2 = {
		//0, 1, 20, 110, 0.041, 0.09, 2.5, 0.3, PARTICLESTREAMSPEC::EMISSIVE,
		0, 1.0, 20, 120, 0.01, 0.08, 2.5, 1, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_FLAT, 1, 0,
		PARTICLESTREAMSPEC::ATM_FLAT, 1, 1,
		exhaust2x
	};

	PARTICLESTREAMSPEC exhaust_rcs2 = {
		0, 0.1, 10, 100, 0.1, 0.05, 60, 10, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0.5, 1,
		PARTICLESTREAMSPEC::ATM_FLAT, 0.1, 0.5,
		exhaustrcs
	};

	PARTICLESTREAMSPEC atmostrail = {
		0, 3, 20, 2, 3, 1, -0.25, 10, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLIN, 0.5, 1,
		exhaustrcs
	};


	AddExhaustStream(th_main[0],_V(0,0,-21),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(1.32,0,-21),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(-1.32,0,-21),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(0,1.32,-21),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(0,-1.32,-21),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(0.931,0.931,-21),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(-0.931,-0.931,-21),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(-0.931,0.931,-21),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(0.931,-0.931,-21),&exhaust_main1);

	AddExhaustStream(th_main[1],_V(0,0,-21),&exhaust_main1);
	AddExhaustStream(th_main[1],_V(1.32,0,-21),&exhaust_main1);
	AddExhaustStream(th_main[1],_V(-1.32,0,-21),&exhaust_main1);

	AddExhaustStream(th_main[2],_V(0,0,-21),&exhaust_main1);



	AddExhaustStream(th_main[0],_V(0,0,-21),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(1.32,0,-21),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(-1.32,0,-21),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(0,1.32,-21),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(0,-1.32,-21),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(0.931,0.931,-21),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(-0.931,-0.931,-21),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(-0.931,0.931,-21),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(0.931,-0.931,-21),&exhaust_main2);

	AddExhaustStream(th_main[1],_V(0,0,-21),&exhaust_main2);
	AddExhaustStream(th_main[1],_V(1.32,0,-21),&exhaust_main2);
	AddExhaustStream(th_main[1],_V(-1.32,0,-21),&exhaust_main2);

	AddExhaustStream(th_main[2],_V(0,0,-21),&exhaust_main2);
	at_trail = AddExhaustStream(th_main[2],_V(0,0,-19),&atmostrail);

	//// rcs exhaust
	AddExhaustStream(th_rcs2[0], &exhaust_rcs2);
	AddExhaustStream(th_rcs2[1],_V( 1.884, 0.103, 18.963), &exhaust_rcs2);
	AddExhaustStream(th_rcs2[2], &exhaust_rcs2);
	AddExhaustStream(th_rcs2[3],_V( 1.884,-0.103, 18.963),&exhaust_rcs2);
	AddExhaustStream(th_rcs2[4], &exhaust_rcs2);
	AddExhaustStream(th_rcs2[3],_V(-1.884,-0.103, 18.963),&exhaust_rcs2);
	AddExhaustStream(th_rcs2[6], &exhaust_rcs2);
	AddExhaustStream(th_rcs2[1],_V(-1.884, 0.103, 18.963),&exhaust_rcs2);
	AddExhaustStream(th_rcs2[8], &exhaust_rcs2);
	AddExhaustStream(th_rcs2[9], &exhaust_rcs2);
	AddExhaustStream(th_rcs2[10], &exhaust_rcs2);
	AddExhaustStream(th_rcs2[11], &exhaust_rcs2);
	///////////////////////////// boom p streams

	th_boom[0] = CreateThruster (_V( 0, 0, 0), _V( 0,0,1), 0.001, hProp, 2, 1);
	th_boom[1] = CreateThruster (_V( 0, 0, 0), _V( 0,0,1), 0.001, hProp, 2, 1);


	///////////////////////////////////////////////////////
	/////AirFoils

	CreateAirfoil (LIFT_VERTICAL, _V(0,0,0.1), VLiftCoeff, 25, 80, 0.16);
	CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,0.1), HLiftCoeff, 25, 80, 0.16); //90
	////////

	

}

void f9h_stg1_centre_expendable::clbkPostCreation ()
{
 hGref = GetGravityRef();

  if (eng_status > 0) ConfigMain();

if (skin_sw == 2) ApplySkin();

}


// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new f9h_stg1_centre_expendable (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (f9h_stg1_centre_expendable*)vessel;
}


void f9h_stg1_centre_expendable::clbkSaveState (FILEHANDLE scn)
{
	char cbuf[256];


	// default vessel parameters
	VESSEL3::clbkSaveState(scn);

	sprintf (cbuf, "%0.4f", skin_sw);
	oapiWriteScenario_string (scn, "LEGS", cbuf);

	//sprintf (cbuf, "%d %d %d %d %d %d", eng_status, intfair_status, dp_status, boostback_sw, rat_sw, afs_status);
	sprintf (cbuf, "%d", eng_status);
	oapiWriteScenario_string (scn, "APST", cbuf);



}
void f9h_stg1_centre_expendable::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
    char *line;
	while (oapiReadScenario_nextline (scn, line)) {
		if (!strnicmp (line, "LEGS", 4)) {
			sscanf (line+4, "%d", &skin_sw);
		} else if (!strnicmp (line, "APST", 4)) {
		sscanf (line+4, "%d", &eng_status);
		}else {
            ParseScenarioLineEx (line, vs);
			// unrecognised option - pass to Orbiter's generic parser
        }
   }

}


int f9h_stg1_centre_expendable::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate) {
	/////////////////////////////////

	if (key == COMM_SEND_STR) {
			return 1;
       }

	if (key == COMM_SEND_AFS) {
			return 1;
       }

	if (key == COMM_SEND_EDLO) {
			return 1;
       }

	//////////////////////////////////
if (!down) return 0; // only process keydown events

  if (KEYMOD_SHIFT (kstate)) 
  {	 
	switch (key)
	{
	

	} 
  }

  else if (KEYMOD_CONTROL (kstate)) 
  { 
	switch (key)
	{
		
	} 
  }

else { // unmodified keys
    switch (key) {


		case OAPI_KEY_P: // configure main eng
		if (eng_status == 0) eng_status = 1;
		else if (eng_status == 1) eng_status = 2;
		else if (eng_status == 2) eng_status = 0;
		ConfigMain();
		return 1;

		//case OAPI_KEY_K:
		//hGref = GetGravityRef();
		//oapiOpenInputBox("Enter Flyback Target Name",SetBBTgt, 0, 50, 0);
		//return 1;


		case OAPI_KEY_E:
			if (cam_status == 0)cam_status = 1;
			else if (cam_status == 1)cam_status = 0;
			SetCamDir();
		return 1;

		//case OAPI_KEY_V:
		//if (boostback_sw == 0)
		//	{
		//	rat_sw = 0;
		//	dp_status = 0;
		//	boostback_sw = 1;
		//	KillAllRCS();
		//	}
		//else if (boostback_sw > 0)
		//{
		//	boostback_sw = 0;
		//	KillAllRCS();
		//}
		//return 1;

		//case OAPI_KEY_N:
		//if (rat_sw == 0)
		//{
		//	rat_sw = 1;
		//	dp_status = 0;
		//	boostback_sw = 0;
		//	KillAllRCS();
		//}
		//else if (rat_sw > 0)
		//{
		//	rat_sw = 0;
		//	KillAllRCS();
		//}
		//return 1;
	}
  }
  return 0;
}


void f9h_stg1_centre_expendable::clbkPreStep(double SimT, double SimDT, double MJD)
{
SetThrusterDir(th_main[eng_status], _V(0,0,1));
double getalt = GetAltitude(ALTMODE_GROUND);
if(getalt < 30)oapiDeleteVessel(GetHandle());


////// char skin

	if (GetDynPressure() > 100000)
	{
					skin_sw = 2; // finished
					ApplySkin();
	}


double simdt = SimDT;

double mth_lev = GetThrusterLevel(th_main[eng_status]);
if (mth_lev == 0 && rcs_sw == 0) ConfigRCS();
else if (mth_lev > 0 && rcs_sw == 1) RevertRCS();

DoThrustVector ();

}

void f9h_stg1_centre_expendable::clbkPostStep (double simt, double simdt, double mjd)
{
	if (GroundContact() == true)landed_sw = 1;
}




bool f9h_stg1_centre_expendable::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp)
{
	// draw the default HUD
	VESSEL3::clbkDrawHUD (mode, hps, skp);

	int s = hps->H;
	double d = (s*0.00130208);
	int sw = ((hps->W));
	int lw =  (int)(16*sw/1024);
	int lwoffset = sw - (18*lw);
	int hlw = (int) (lw/2);

	//int chw = 16;

	
	char test[256];
	strcpy(test,"abcdefghijklmnopqrstuvwxyz");
	int it = skp->GetTextWidth(test, 26);
	it = (int)(it/26);
	int chw = it;

	int roxl = 0;
	int royl = 0;

	double ds = s;
	double dsw = sw;
	double sc_ratio = ds/dsw;


	if (sc_ratio < 0.7284)
	{
	roxl = (lw*10);
	royl = (int) (-88*d);
	}



	int wd = (int)(152*d);
	int wc = (int)(168*d);
	int w0 = (int)(184*d);
	int w1 = (int) (200*d);
	int w2 = (int)(216*d);
	/*int w3 = (int)(232*d);
	int w4 = (int)(248*d);
	int w5 = (int)(264*d);
	int w6 = (int)(280*d);
	int w7 = (int)(296*d);
	int w8 = (int)(312*d);
	int w9 = (int)(328*d);
	int w10 = (int)(344*d);
	int w11 = (int)(360*d);
	int w12 = (int)(376*d);
	int w13 = (int)(392*d);
	int w14 = (int)(408*d);
	int w15 = (int)(424*d);*/



	{
	skp->SetTextColor(0x00FFFFFF);
	char abuf[256];
	double m1 = GetEmptyMass()+GetPropellantMass(hRCSProp);
	double m2 = GetPropellantMass(hProp);
	double test = ((m1+m2)/m1);
	double isp_curr = GetThrusterIsp(th_main[2]);
	double zdv = (isp_curr*(log(test)));

	sprintf(abuf,"Remaining dV %0.1fm/s : RCS(N)Prop. %0.1fkg", zdv, GetPropellantMass(hRCSProp) );
	skp->Text ((10+roxl), (wc+royl), abuf,strlen(abuf));
	}


	

	if (eng_status == 0)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"P = Engine Selection : 9");
	skp->Text ((10+roxl), (w0+royl), abuf,strlen(abuf));
	}

	else if (eng_status == 1)
	{
	skp->SetTextColor(0x0066FFFF);
	char abuf[256];
	sprintf(abuf,"P = Engine Selection : 3");
	skp->Text ((10+roxl), (w0+royl), abuf,strlen(abuf));
	}

	else if (eng_status == 2)
	{
	skp->SetTextColor(0x006666FF);
	char abuf[256];
	sprintf(abuf,"P = Engine Selection : 1");
	skp->Text ((10+roxl), (w0+royl), abuf,strlen(abuf));
	}


	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"E = Set View +Z/-Z");
	skp->Text ((10+roxl), (w1+royl), abuf,strlen(abuf));
	}

	return true;
}



void f9h_stg1_centre_expendable::RevertRCS (void)
{
	SetThrusterGroupLevel(htgrppup,0);
	SetThrusterGroupLevel(htgrppdown,0);
	SetThrusterGroupLevel(htgrpbright,0);
	SetThrusterGroupLevel(htgrpbleft,0);
	SetThrusterGroupLevel(htgrpyright,0);
	SetThrusterGroupLevel(htgrpyleft,0);

	DelThrusterGroup(htgrppup,THGROUP_ATT_PITCHUP, false);
	DelThrusterGroup(htgrppdown,THGROUP_ATT_PITCHDOWN, false);
	DelThrusterGroup(htgrpbright,THGROUP_ATT_BANKRIGHT, false);
	DelThrusterGroup(htgrpbleft,THGROUP_ATT_BANKLEFT, false);
	DelThrusterGroup(htgrpyright,THGROUP_ATT_YAWRIGHT, false);
	DelThrusterGroup(htgrpyleft,THGROUP_ATT_YAWLEFT, false);

	th_group[0] = th_rcs[9];
	th_group[1] = th_rcs[11];
	htgrppup = CreateThrusterGroup (th_group, 2, THGROUP_ATT_PITCHUP);

	th_group[0] = th_rcs[8];
	th_group[1] = th_rcs[10];
	htgrppdown = CreateThrusterGroup (th_group, 2, THGROUP_ATT_PITCHDOWN);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[3];
	htgrpyright = CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWRIGHT);

	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[2];
	htgrpyleft = CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWLEFT);

	th_group[0] = th_rcs[4];
	th_group[1] = th_rcs[7];
	htgrpbright = CreateThrusterGroup (th_group, 2, THGROUP_ATT_BANKRIGHT);

	th_group[0] = th_rcs[5];
	th_group[1] = th_rcs[6];
	htgrpbleft = CreateThrusterGroup (th_group, 2, THGROUP_ATT_BANKLEFT);

	rcs_sw = 0;
	vrot_lim = 5*RAD;
}

void f9h_stg1_centre_expendable::ConfigRCS (void)
{
	SetThrusterGroupLevel(htgrppup,0);
	SetThrusterGroupLevel(htgrppdown,0);
	SetThrusterGroupLevel(htgrpbright,0);
	SetThrusterGroupLevel(htgrpbleft,0);
	SetThrusterGroupLevel(htgrpyright,0);
	SetThrusterGroupLevel(htgrpyleft,0);

	DelThrusterGroup(htgrppup,THGROUP_ATT_PITCHUP, false);
	DelThrusterGroup(htgrppdown,THGROUP_ATT_PITCHDOWN, false);
	DelThrusterGroup(htgrpbright,THGROUP_ATT_BANKRIGHT, false);
	DelThrusterGroup(htgrpbleft,THGROUP_ATT_BANKLEFT, false);
	DelThrusterGroup(htgrpyright,THGROUP_ATT_YAWRIGHT, false);
	DelThrusterGroup(htgrpyleft,THGROUP_ATT_YAWLEFT, false);

	th_group[0] = th_rcs2[3];
	th_group[1] = th_rcs2[5];
	htgrppup = CreateThrusterGroup (th_group, 2, THGROUP_ATT_PITCHUP);

	th_group[0] = th_rcs2[1];
	th_group[1] = th_rcs2[7];
	htgrppdown = CreateThrusterGroup (th_group, 2, THGROUP_ATT_PITCHDOWN);

	th_group[0] = th_rcs2[8];
	th_group[1] = th_rcs2[11];
	htgrpbright = CreateThrusterGroup (th_group, 2, THGROUP_ATT_BANKRIGHT);

	th_group[0] = th_rcs2[9];
	th_group[1] = th_rcs2[10];
	htgrpbleft = CreateThrusterGroup (th_group, 2, THGROUP_ATT_BANKLEFT);

	th_group[0] = th_rcs2[2];
	th_group[1] = th_rcs2[4];
	htgrpyright = CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWRIGHT);

	th_group[0] = th_rcs2[0];
	th_group[1] = th_rcs2[6];
	htgrpyleft = CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWLEFT);

	rcs_sw = 1;
	vrot_lim = 3*RAD;
}

void f9h_stg1_centre_expendable::ConfigMain (void)
{
	double old_tg_lev = GetThrusterGroupLevel(htgrpmain);
	SetThrusterGroupLevel(htgrpmain,0);



	DelThrusterGroup(htgrpmain,false);
	DelThrusterGroup(THGROUP_MAIN, false);

	if (eng_status == 0) 
	{
		th_group[0] = th_main[0];
		SetThrusterLevel(th_main[1],0);
		SetThrusterLevel(th_main[2],0);
		int i;
		for (i = 0; i < 12; i++)
		{
		SetThrusterMax0(th_rcs[i],RCSTH*1);
		}
	}
	else if (eng_status == 1)
	{
		th_group[0] = th_main[1];
		SetThrusterLevel(th_main[0],0);
		SetThrusterLevel(th_main[2],0);
		int i;
		for (i = 0; i < 12; i++)
		{
		SetThrusterMax0(th_rcs[i],RCSTH*0.25);
		}
	}
	else if (eng_status == 2)
	{
		th_group[0] = th_main[2];
		SetThrusterLevel(th_main[0],0);
		SetThrusterLevel(th_main[1],0);
		int i;
		for (i = 0; i < 12; i++)
		{
		SetThrusterMax0(th_rcs[i],RCSTH*0.15);
		}
	}
	htgrpmain = CreateThrusterGroup (th_group, 1, THGROUP_MAIN);

	SetThrusterGroupLevel(htgrpmain,old_tg_lev);
}


void f9h_stg1_centre_expendable::KillAllRCS()
{
SetThrusterGroupLevel(THGROUP_ATT_PITCHUP, 0);
SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN, 0);
SetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT, 0);
SetThrusterGroupLevel(THGROUP_ATT_YAWLEFT, 0);
SetThrusterGroupLevel(THGROUP_ATT_BANKRIGHT, 0);
SetThrusterGroupLevel(THGROUP_ATT_BANKLEFT, 0);
}



bool f9h_stg1_centre_expendable::clbkLoadGenericCockpit ()
{
	SetCamDir();
return true;
}


void f9h_stg1_centre_expendable::SetCamDir()
{
	VECTOR3 camdefdir = _V(0,0,1);
	if(cam_status == 1)camdefdir = _V(0,0,-1);
	SetCameraDefaultDirection(camdefdir, 0);
	oapiCameraAttach(GetHandle(), 2);
}


void f9h_stg1_centre_expendable::ApplySkin ()
{	
	bs1 = oapiGetTextureHandle (hMhull, 1);
	oapiBlt(bs1, bs2, 0, 0, 0, 0, 256, 1024);

	bs3 = oapiGetTextureHandle (hMintstg, 1);
	oapiBlt(bs3, bs4, 0, 0, 0, 0, 256, 512);
}


void f9h_stg1_centre_expendable::DoThrustVector ()
{
if (mth_level == 0) return;

double abs_pitch_level = pu_level - pd_level;
double abs_yaw_level = yr_level - yl_level;

//sprintf(oapiDebugString(), "apl %0.5f ayl %0.5f", abs_pitch_level, abs_yaw_level);

double main_max_th = GetThrusterMax(th_main[eng_status]);
double rcs_max_th = GetThrusterMax(th_rcs[0]) * 2;
double main_level = mth_level;

double abs_main_th = main_max_th * main_level;
double abs_pitch_th = rcs_max_th * abs_pitch_level;
double abs_yaw_th = rcs_max_th * abs_yaw_level;

VECTOR3 main_vector = _V(-abs_yaw_th,-abs_pitch_th,abs_main_th);
double ratio_main = length(main_vector) / length(_V(0,0,abs_main_th));
normalise(main_vector);

double new_th_main_level = ratio_main * main_level;
if (new_th_main_level > 1)new_th_main_level = 1;
else if (new_th_main_level < 0)new_th_main_level = 0;

SetThrusterDir(th_main[eng_status], main_vector);

double ga = acos(dotp(main_vector, _V(0,0,1)));
//sprintf(oapiDebugString(), "gimbal angle %0.6f", ga*DEG);
SetThrusterLevel(th_main[eng_status], new_th_main_level);

SetThrusterGroupLevel(THGROUP_ATT_PITCHUP, 0);
SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN, 0);
SetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT, 0);
SetThrusterGroupLevel(THGROUP_ATT_YAWLEFT, 0);


}
