// ==============================================================
//
// star48bv.cpp
// Control module for star48bv vessel class
//
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"

// ==============================================================
// Some vessel parameters
// ==============================================================



const double ISP = 2865.5;
const double MAINTH = 68720;
const double FUELMASS = 2011.8;
const double RCS_FUELMASS = 20;
const double RCSTH = 50;
const double DRY_MASS = 144.7;
//const double PYLD_MASS = 610;
char pldname[256];
ATTACHMENTHANDLE hpld_attach;
OBJHANDLE hme;

//const VECTOR3 PYLD_OFF = _V(0,0,2.544);

#define GREEN RGB(125, 255, 125)

static const DWORD ntdvtx_geardown = 3;
static TOUCHDOWNVTX tdvtx_geardown[ntdvtx_geardown] = {
	{_V( 0,-1, 5), 1e6, 1e6, 0.7, 0.7},
	{_V(-2,-1,-5), 1e6, 1e6, 0.7, 0.7},
	{_V( 2,-1,-5), 1e6, 1e6, 0.7, 0.7}
};

// ==============================================================
// Shuttle-PB class interface
// ==============================================================

class star48bv: public VESSEL3 {
public:
	star48bv (OBJHANDLE hVessel, int flightmodel);
	~star48bv ();
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPreStep(double SimT, double SimDT, double MJD);
	//////void clbkPostStep (double simt, double simdt, double mjd);
	void clbkSaveState (FILEHANDLE scn);
	void clbkLoadStateEx (FILEHANDLE scn, void *status);
	int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);
	bool clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp);
	void clbkPostCreation();
	

	THRUSTER_HANDLE th_main, th_rcs[6], th_group[2];
	PROPELLANT_HANDLE hProp, hProp_rcs;
	

private:
	
	//double step_time;
	int eng_status;
	double pld_mass;
	int calc_sw;
	//double drot;

	
};


star48bv::star48bv (OBJHANDLE hVessel, int flightmodel)
: VESSEL3 (hVessel, flightmodel)
{
    

}

star48bv::~star48bv ()
{
}




// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void star48bv::clbkSetClassCaps (FILEHANDLE cfg)
{
	//step_time = 0;
	//drot = 0;
	//lad_status = 0;
	eng_status = 0;
	strcpy(pldname, "NULL");
	pld_mass = 0.0;
	calc_sw = 1;
	// physical vessel parameters
	SetSize (5);
	SetEmptyMass (DRY_MASS);
	SetPMI(_V(0.28,0.28,0.19));
	SetCrossSections (_V(30,30,7));
	SetCW(0.5,0.5,0.1,0.3);
	SetRotDrag (_V(2,2,1));
	//SetTouchdownPoints ( _V(0,-1, 5), _V(-2,-1,-5), _V( 2,-1,-5));
	SetTouchdownPoints(tdvtx_geardown,ntdvtx_geardown);

	// camera parameters
	SetCameraOffset (_V(0,2,-4));

	SetMeshVisibilityMode(AddMesh("FalconHeavy/star48bv"),MESHVIS_ALWAYS);
	


	

	// propellant resources
	
	hProp = CreatePropellantResource (FUELMASS,FUELMASS);
	hProp_rcs = CreatePropellantResource (RCS_FUELMASS,RCS_FUELMASS);
	// ***************** thruster definitions *******************
	
	// main
	th_main = CreateThruster (_V( 0,0, -1.4), _V(0, 0, 1), MAINTH, hProp, ISP);
	
	// bank right / yaw2
	th_rcs[0] = CreateThruster (_V( 0.06,-0.667,-0.655), _V(-1,0,0), RCSTH, hProp_rcs, ISP);
	th_rcs[1] = CreateThruster (_V( -0.06,0.667,-0.655), _V(1,0,0), RCSTH, hProp_rcs, ISP);
	
	// bank left / yaw2
	th_rcs[2] = CreateThruster (_V( -0.06,-0.667,-0.655), _V(1,0,0), RCSTH, hProp_rcs, ISP);
	th_rcs[3] = CreateThruster (_V( 0.06,0.667,-0.655), _V(-1,0,0), RCSTH, hProp_rcs, ISP);
	
	// pitch up 2
	th_rcs[4] = CreateThruster (_V( 0,0.73,-0.655), _V(0,-1,0), RCSTH, hProp_rcs, ISP);
	
	// pitch down 2
	th_rcs[5] = CreateThruster (_V( 0,-0.73,-0.655), _V(0,1,0), RCSTH, hProp_rcs, ISP);
	
	

	
	// create rcs thrusters
	th_group[0] = th_rcs[4];
	CreateThrusterGroup (th_group, 1, THGROUP_ATT_PITCHUP);

	th_group[0] = th_rcs[5];
	CreateThrusterGroup (th_group, 1, THGROUP_ATT_PITCHDOWN);

	th_group[0] = th_rcs[2];
	th_group[1] = th_rcs[3];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_BANKLEFT);

	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[1];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_BANKRIGHT);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[2];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWLEFT);

	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[3];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWRIGHT);

	CreateThrusterGroup (&th_main, 1, THGROUP_MAIN);

	SURFHANDLE tex_main = oapiRegisterExhaustTexture ("Exhaust2");
	AddExhaust(th_main,5,0.6,tex_main);

	double exlen = 0.6;
	double exwid = 0.03;
	SURFHANDLE exhaust2 = oapiRegisterExhaustTexture ("exhaust_atrcs");
	AddExhaust(th_rcs[0],exlen,exwid,exhaust2);
	AddExhaust(th_rcs[1],exlen,exwid,exhaust2);
	AddExhaust(th_rcs[2],exlen,exwid,exhaust2);
	AddExhaust(th_rcs[3],exlen,exwid,exhaust2);
	AddExhaust(th_rcs[4],exlen,exwid,exhaust2);
	AddExhaust(th_rcs[5],exlen,exwid,exhaust2);
	
	
	//// attachments
	hpld_attach = CreateAttachment (false, _V( 0, 0, 0.956), _V( 0,0,1), _V(0,1,0), "PL");
}



// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new star48bv (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (star48bv*)vessel;
}




void star48bv::clbkSaveState (FILEHANDLE scn)
{
	char cbuf[256];

	// default vessel parameters
	VESSEL3::clbkSaveState(scn);

	// custom parameters
	//sprintf (cbuf, "%d",lad_status);
	//oapiWriteScenario_string (scn, "PDST", cbuf);
	
	sprintf (cbuf, "%d",eng_status);
	oapiWriteScenario_string (scn, "ENST", cbuf);


}
void star48bv::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
    char *line;
	while (oapiReadScenario_nextline (scn, line)) {
		if (!strnicmp (line, "ENST", 4)) {
			sscanf (line+4, "%d",&eng_status);
		}else {
            ParseScenarioLineEx (line, vs);
			// unrecognised option - pass to Orbiter's generic parser
        }
   }

	


}


bool SetPLD(void *id, char *str, void *data)
{
	char cbuf[256];
	sscanf(str, "%s", &cbuf);
	OBJHANDLE hPld = oapiGetObjectByName(cbuf);
	if (hPld == NULL)return false;
	else
	{
	VESSEL2* v = (VESSEL2*)oapiGetVesselInterface(hPld);
	DWORD ac = v->AttachmentCount(true);
	if (ac == 0) return false;
	for (DWORD j = 0; j < ac; j++)
		{
		ATTACHMENTHANDLE hAtt = v->GetAttachmentHandle(true, j);
		OBJHANDLE hOp = v->GetAttachmentStatus(hAtt);
		if (hOp == NULL)
			{
			VESSEL2* vme = (VESSEL2*)oapiGetVesselInterface(hme);
			if (vme->AttachChild(hPld, hpld_attach, hAtt) == true)
				{
					return true;
				}
			else return false;
			}
		else if (j == ac-1) return false;
		}
	
	}
	
}


int star48bv::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate) {



  if (!down) return 0; // only process keydown events

 if (KEYMOD_SHIFT (kstate)) 
  {	 
	switch (key)
	{
		

	

	} 
  }


 else if (KEYMOD_CONTROL (kstate)) 
  { 
	switch (key)
	{
		
	} 
  }

else { // unmodified keys
    switch (key) {


	case OAPI_KEY_J: // jettison
		DetachChild(hpld_attach,0.5);
		SetEmptyMass (DRY_MASS);
		return 1;

	case OAPI_KEY_P: // add payload
		hme = GetHandle();
		if(GetAttachmentStatus(hpld_attach) == NULL )
		{
			oapiOpenInputBox("Enter Payload Name", SetPLD, 0, 50, 0);
		}
		calc_sw = 1;
		return 1;

	}
  }
  return 0;
}




void star48bv::clbkPostCreation()
{

	//else ShiftCG(_V(0,0,1));

}



void star48bv::clbkPreStep(double SimT, double SimDT, double MJD)
{
	if (calc_sw == 1)
	{
	OBJHANDLE hpd = GetAttachmentStatus(hpld_attach);
	if (hpd == NULL) SetEmptyMass (DRY_MASS);
	else 
		{
		pld_mass = oapiGetMass(hpd);
		oapiGetObjectName(hpd, pldname, 40);
		SetEmptyMass (DRY_MASS+pld_mass);
		}
		calc_sw = 0;
	}
	
	if (GetThrusterLevel(th_main) > 0 && eng_status == 0)
	{
		eng_status = 1;
	}

	if (eng_status == 1)
	{
	SetThrusterLevel(th_main, 1);
	if (GetPropellantMass(hProp) == 0)
		{
		eng_status = 2;
		SetThrusterLevel(th_main, 0);
		}
	}


//sprintf(oapiDebugString(), "%d %0.4f %0.4f", (sa1_stbd_elb_sts + sa1_stbd_scale_sts + sa1_port_elb_sts + sa1_port_scale_sts),sa1_port_rot_proc,sa1_stbd_rot_proc);
//sprintf(oapiDebugString(), "IT %0.1f", ign_timer);

////////////////// ANIMATIONS ///////////////

}




bool star48bv::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp)
{
	// draw the default HUD
	VESSEL3::clbkDrawHUD (mode, hps, skp);

	int s = hps->H;
	double d = (s*0.00130208);
	int sw = ((hps->W));
	int lw =  (int)(16*sw/1024);
	int lwoffset = sw - (18*lw);
	int hlw = (int) (lw/2);


	int roxl = 0;
	int royl = 0;

	double ds = s;
	double dsw = sw;
	double sc_ratio = ds/dsw;


	if (sc_ratio < 0.7284)
	{
	roxl = (lw*10);
	royl = (int) (-88*d);
	}



	int wd = (int)(152*d);
	int wc = (int)(168*d);
	int w0 = (int)(184*d);
	int w1 = (int) (200*d);
	int w2 = (int)(216*d);
	int w3 = (int)(232*d);
	//int w4 = (int)(248*d);
	//int w5 = (int)(264*d);
	//int w6 = (int)(280*d);
	//int w7 = (int)(296*d);
	//int w8 = (int)(312*d);
	//int w9 = (int)(328*d);
	//int w10 = (int)(344*d);
	//int w11 = (int)(360*d);
	//int w12 = (int)(376*d);
	//int w13 = (int)(392*d);
	//int w14 = (int)(408*d);
	//int w15 = (int)(424*d);



	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	double m1 = GetEmptyMass();
	double m2 = GetPropellantMass(hProp);
	double m3 = GetPropellantMass(hProp_rcs);
	double test = ((m1+m2+m3)/(m1+m3));
	double dv = (ISP*(log(test)));


	sprintf(abuf,"Remaining dV = %0.2f m/s", dv);
	skp->Text ((10+roxl), (wc+royl), abuf,strlen(abuf));
	}

	if (GetAttachmentStatus(hpld_attach) == NULL)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"P = Attach Payload");
	skp->Text ((10+roxl), (w0+royl), abuf,strlen(abuf));
	}
	else
	{
	skp->SetTextColor(0x00FFFFFF);
	char abuf[256];
	sprintf(abuf,"Payload: %s %dkg", pldname, int(pld_mass));
	skp->Text ((10+roxl), (w0+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"Jettison Payload = J");
	skp->Text ((10+roxl), (w1+royl), abuf,strlen(abuf));
	}

	

	return true;
}
