// ==============================================================
//                 ORBITER MODULE: weather_balloon
//
// weather_balloon.cpp
// Control module for weather_balloon vessel class
//
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"

// ==============================================================
// Some vessel parameters
// ==============================================================


static const DWORD ntdvtx_geardown = 3;
static TOUCHDOWNVTX tdvtx_geardown[ntdvtx_geardown] = {
	{_V(    0,-0.5,1), 1e6, 1e5, 1, 1},
	{_V(-0.866,-0.5,-0.5), 1e6, 1e5, 1, 1},
	{_V( 0.866,-0.5,-0.5), 1e6, 1e5, 1, 1}
};

class weather_balloon: public VESSEL3 {
public:
	weather_balloon (OBJHANDLE hVessel, int flightmodel)
		: VESSEL3 (hVessel, flightmodel) {}

	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPreStep(double SimT, double SimDT, double MJD);
	//void clbkPostStep (double simt, double simdt, double mjd);
	//void clbkPostCreation();
	//void clbkSaveState (FILEHANDLE scn);
	//void clbkLoadStateEx (FILEHANDLE scn, void *status);
	//int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);
	//int clbkConsumeDirectKey(char *kstate);
	//bool clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp);



private:
	//int light_status;

};
// Define animation sequences for moving parts
// --------------------------------------------------------------



// ==============================================================
// weather_balloon class interface
// ==============================================================

// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void weather_balloon::clbkSetClassCaps (FILEHANDLE cfg)
{

	SetEmptyMass (1);
	SetCW (0.2, 0.2, 0.2, 0.2);
	SetSize (3);
	SetCrossSections (_V(1,1,1));
	SetPMI (_V(4,2.1,4));
	//SetTouchdownPoints (_V(0,-0.4,100), _V(-86.6,-0.4,-50), _V( 86.6,-0.4,-50));
	SetTouchdownPoints(tdvtx_geardown,ntdvtx_geardown);
	SetRotDrag (_V(2,1,2));

	SetMeshVisibilityMode ((AddMesh ("FalconHeavy/weather_balloon")), MESHVIS_ALWAYS);
	SetCameraOffset (_V(0,2,-4));

}

// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new weather_balloon (hvessel, flightmodel);
}
// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (weather_balloon*)vessel;
}
// ==============================================================
// Overloaded clbkPostCreation callback functions
// ==============================================================

//void weather_balloon::clbkPostCreation ()
//{


//}
//void weather_balloon::clbkSaveState (FILEHANDLE scn)
//{

//}
//void weather_balloon::clbkLoadStateEx (FILEHANDLE scn, void *vs)
//{
 
	
//}

//int weather_balloon::clbkConsumeDirectKey(char *kstate)
//{

//}


//int weather_balloon::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate) 
//{

//}


void weather_balloon::clbkPreStep(double SimT, double SimDT, double MJD) 
{
	VECTOR3 aspdv;
	GetAirspeedVector(FRAME_HORIZON, aspdv);

	double speed_factor = 10 - aspdv.y;
	if (speed_factor < 0)speed_factor = 0;
	else if (speed_factor > 10)speed_factor = 10;
	double force = (9.81 + (speed_factor*0.8));
	VECTOR3 force_vector_hrz = _V(0,force,0);
	VECTOR3 force_vector_loc;
	HorizonInvRot(force_vector_hrz, force_vector_loc);
	AddForce(force_vector_loc, _V(0,2,0));
	

	if(GetAltitude(ALTMODE_MEANRAD) > 6550)oapiDeleteVessel(GetHandle());
}


//void weather_balloon::clbkPostStep (double simt, double simdt, double mjd)
//{
	
//}



//bool weather_balloon::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp)
//{
	// draw the default HUD

//}