// ==============================================================
//
// f9h_stg2
// Control module for f9h_stg2 vessel class
//
// ==============================================================
//
#define STRICT 1
#define ORBITER_MODULE

#include "orbitersdk.h"
#include "OrbiterSoundSDK40.h"
#include <cstdio>
#include <cmath>
#include <string>
#include <stdio.h>
#include "DlgCtrl.h"
#include "f9h_stg2_resource.h"
#include "gcAPI.h"

// ==============================================================
// Some vessel parameters
// ==============================================================

#define CTDN	1
#define SEP		2
#define SEPST	3
#define SEPFR	4

const DWORD COMM_SEND_STR = 2300;
const DWORD COMM_SEND_AFS = 2301;
const DWORD COMM_CALC_MASS = 2302;
const DWORD COMM_CALC_FRE = 2303;
const DWORD COMM_SEND_EDLO = 2304;
const DWORD COMM_CALC_ASDS = 2305;
const double STG0DRYMASS = 45000;//2*22500;//38700;// 2 * 19350;
const VECTOR3 STG0_STBD_OFF = _V(4.242,0,-10);
const VECTOR3 STG0_PORT_OFF = _V(-4.242,0,-10);
//const double STG0FUELMASS = 1152000;// 3*384000;//372000;
const double STG0FUELMASS = 1233000;//3*411000//3*409500//1207500;//3*402500;//1152000;// 3*384000 + 6?;//372000;
const double STG0RCSTH = 114000; // 3*38000;

const double STG1ISP0 = 2767;//3062;
const double STG1ISPV = 3050;//3311;
const double STG1MAINTH0 = 845148;
const double STG1MAINTH1 = 914111;//716000;
const double STG1RCSTH = 38000;
const double STG1DRYMASS = 25600;//22500;//19350;//18000;
const double STG1FUELMASS = 411000;//409500;//402500;//384000;//372000;

const double STG2ISP0 = 2767;//3218;
const double STG2ISPV = 3580;//3730;//3580;//3385;//3556;
const double STG2MAINTH0 = 845148;
const double STG2MAINTH1 = 934000;//801000;//716000;
const double STG2RCSTH1 = 500;
const double STG2RCSTH2 = 13000;
const double STG2FUELMASS = 107500;//104275;//93100;//372000;
const double STG2DRYMASS = 4000;//4300;//5800;//18000;
const double STG2RCSFUELMASS = 100;
const double STG2RCSISP = 800;

const VECTOR3 STG1_OFF = _V(0,0,-10);
const VECTOR3 STG1INT_OFF = _V(0,0,-10);
const VECTOR3 STG1FAIR_OFF = _V(0,0,31.384);
const VECTOR3 STG1FAIR_LARGE_OFF = _V(0,0,35.384);
const VECTOR3 STG1_FAIR1_OFF = _V(0,-1,31.384);
const VECTOR3 STG1_FAIR1_LARGE_OFF = _V(0,-1,35.384);
const VECTOR3 STG1_FAIR2_OFF = _V(0, 1,31.384);
const VECTOR3 STG1_FAIR2_LARGE_OFF = _V(0, 1,35.384);
const VECTOR3 STG1STG2_OFF = _V(0,0,19.596);
const VECTOR3 STG2FAIR_OFF = _V(0,0,12.288);
const VECTOR3 STG2FAIR_LARGE_OFF = _V(0,0,16.288);
const VECTOR3 STG2_FAIR1_OFF = _V(0,-0.95,11.788);
const VECTOR3 STG2_FAIR1_LARGE_OFF = _V(0,-0.95,15.788);
const VECTOR3 STG2_FAIR2_OFF = _V(0, 0.95,11.788);
const VECTOR3 STG2_FAIR2_LARGE_OFF = _V(0, 0.95,15.788);



const VECTOR3 ADAPTOR_OFF = _V(0,0,26.646);

const double SS_SPEED = 0.5;

//const VECTOR3 STG1_OFF = _V(0,0,-10);
//const VECTOR3 STG1INT_OFF = _V(0,0,-10);
//const VECTOR3 STG1FAIR_OFF = _V(0,0,29.384);
//const VECTOR3 STG1STG2_OFF = _V(0,0,19.096);
//const VECTOR3 STG2FAIR_OFF = _V(0,0,10.288);
//const double  FAIR_MASS = 1750;
//
//const VECTOR3 ADAPTOR_OFF = _V(0,0,24.646);

char refname[256];
char refnamep[256];
char refnames[256];
char pldname[256][100];
int afs_status = 0;
int bafs_status = 0;
double tgt_inc = 0;
double min_inc = 0;
double fuel_reserve = 0;
double pld_mass = 0;
//double pld_mass2 = 0;
double booster_fuel_reserve = 300;
//double pmq_throttle = 100;
double accel_limit = 39.24;
//int dlg_status = 0;
//int dlgl_status = 0;
//int dlgb_status = 0;
int dlgf_status = 0;
//int dlgh_status = 0;
int pld_att_status = 0;
int calc_sw = 0;
int intfair_status = 0;
int eng_status = 0;
int apop_status = 0;
double ap_tgt = 175000; // 1st stage apogee
const double mu = 398600439968871.2;
const double rt = 6371010;
double op_meco_alt = 200000; // perigee
double op_insertion = 200000; // insertion
double op_tgt = 200000; // apogee
double fuel_res_est = 0;
double booster_fuel_res_est = 0;
int edlo_sw_core = 0;
int edlo_sw_boosters = 0;
ATTACHMENTHANDLE pld_attach[100];
ATTACHMENTHANDLE lpd_attach;
NOTEHANDLE hNote;
OBJHANDLE hMe;
OBJHANDLE hGref;
OBJHANDLE bchk;
int lot_hh = 0;
int lot_mm = 0;
int lot_ss = 0;
int lot_status = 0;
double lot_timer = 0;
double asds_core_lat = 0;
double asds_core_lng = 0;
double asds_bstr_lat = 0;
double asds_bstr_lng = 0;
double frv_core_lat = 0;
double frv_core_lng = 0;
const double M_E = 2.71828182845904523536;

static TOUCHDOWNVTX tdvtx_geardown[4] = {
	{_V(    0,-5, -43), 1e6, 1e5, 0.8, 0.8},
	{_V(-5, 3, -43), 1e6, 1e5, 0.8, 0.8},
	{_V( 5, 3,-43), 1e6, 1e5, 0.8, 0.8},
	{_V( 0, 0 , 40), 1e6, 1e5, 1.5}
};

static TOUCHDOWNVTX tdvtx_gearup[4] = {
	{_V(    0,-5, -10), 1e6, 1e5, 0.8, 0.8},
	{_V(-5, 3, -10), 1e6, 1e5, 0.8, 0.8},
	{_V( 5, 3,-10), 1e6, 1e5, 0.8, 0.8},
	{_V( 0, 0 , 10), 1e6, 1e5, 1.5}
};

#define GREEN RGB(125, 255, 125)

VECTOR3 bspos = _V(1.27, 1.27, 25.4);
VECTOR3 bscol = _V(1,1,1);
VECTOR3 new_beac_pos = _V(1.27, 1.27, 5.804);

typedef struct {
	HINSTANCE hDLL;
} GDIParams;
GDIParams g_Param;

void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*22.5;
	static const double istep = 1.0/step;
	static const int nabsc = 17;
	static const double CL[nabsc] = {0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0};


	aoa += PI;
	int idx = max (0, min (15, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = 0.0;
	*cd = 0.11 + oapiGetInducedDrag (*cl, 0.16, 0.2);
	
	
}

// 2. horizontal lift component (vertical stabiliser and body)

void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*22.5;
	static const double istep = 1.0/step;
	static const int nabsc = 17;
	static const double CL[nabsc] = {0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0};

	beta += PI;
	int idx = max (0, min (15, (int)(beta*istep)));
	double d = beta*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = 0.0;
	*cd = 0.11 + oapiGetInducedDrag (*cl, 0.16, 0.2);
	
}

// ==============================================================
// Shuttle-PB class interface
// ==============================================================

class f9h_stg2: public VESSEL3 {
public:
	f9h_stg2 (OBJHANDLE hVessel, int flightmodel);
	~f9h_stg2 ();
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPostCreation();
	void clbkPreStep(double SimT, double SimDT, double MJD);
	void clbkPostStep (double simt, double simdt, double mjd);
	void clbkSaveState (FILEHANDLE scn);
	void clbkLoadStateEx (FILEHANDLE scn, void *status);
	int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);
	int MyID;	// this is your unique Id, it will identify your ship in OrbiterSound.
	bool clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp);
	bool clbkLoadGenericCockpit ();
	void clbkVisualCreated (VISHANDLE vis, int refcount);
	void clbkVisualDestroyed (VISHANDLE vis, int refcount);
	
	
private:

	char adname [256];

	THRUSTER_HANDLE th_main[3],th_rcs[24], th_group[9], th_dump;
	THRUSTER_HANDLE th_rcs0[12];
	THGROUP_HANDLE htgrpmain;
	PROPELLANT_HANDLE hProp0, hProp1,hProp2, hRCSProp2;
	THGROUP_HANDLE htgrppup2,htgrppdown2,htgrpbright2, htgrpbleft2, htgrpyright2, htgrpyleft2;
	UINT anim_ss;
	ANIMATIONCOMPONENT_HANDLE Section0, Section1;
	

	ELEMENTS el;
	ORBITPARAM prm;

	MGROUP_TRANSFORM *ss1trans[4];

	OBJHANDLE hpd;
	OBJHANDLE bbchk;
	PointLight *spt_light;

	EXHAUSTSPEC main_exhaust[27];

	void DefineAnimations (void);
	void DefineAnimationsExpendable (void);
	//void JettisonPld(void);
	void JettisonFair(void);
	void JettisonStg(void);
	void JettisonBoosters(void);
	void ConfigMain(void);
	void ConfigMain0(void);
	void ConfigRCS2(void);
	void RevertRCS2(void);
	void CalcEmptyMass(void);
	void DoAscent(void);
	void DoOrbit(void);
	double CalcAzimuth();
	void CalcFlightVector();
	void GoToFlightVector();
	double FinalV();
	double GetOS();
	void RollToAz();
	void SetCamDir();
	void KillAllRCS2();
	void CalcFuel();
	//void InitDialog(HWND hWnd);
	void CalcFuelRes();
	void DoLOT();
	void CalcAsdsPos();
	void FairTargeting();

	double red_fact;
	DEVMESHHANDLE dhMesh0;
	MESHHANDLE hMStg2;
	void SetMeshMat(void);

	double step_time, old_ap_dist, pitch_angle, bias, timer, vrot_lim, pst_lvl, pst_lvl2;
	int op_status, ap_status, rcs_sw2;
	int ap_function, cam_status, drg, nfr, no_legs;
	int start_sw, tu_sw, cntr_skin_sw, port_skin_sw, stbd_skin_sw;
	int large_fair, recoverable_fair;
	double ss_proc;
	SURFHANDLE bs1, bs2, bs3, bs4, bs5, bs6;

	double azimuth;	
	VECTOR3 azim_local, vazim1, vazim2, flight_vector;

	int he_vent_status;
	double he_vent_level, he_vent_timer;

	double core_fuel, booster_fuel;
	double core_throttle, bstr_throttle;

	double flight_path_angle;

	SpotLight *spotlight1;
	BEACONLIGHTSPEC beac;
	int lamp_status;

	bool time_acc_limit;
	double  FAIR_MASS;

};

f9h_stg2::f9h_stg2 (OBJHANDLE hVessel, int flightmodel)
: VESSEL3 (hVessel, flightmodel)
{
    

}

f9h_stg2::~f9h_stg2 ()
{
	delete ss1trans[0];
	delete ss1trans[1];
	delete ss1trans[2];
	delete ss1trans[3];
}

void f9h_stg2::DefineAnimations (void)
{
	anim_ss = CreateAnimation (0);

	static UINT ss1grp0[2] = {8,10};
	ss1trans[0] = new MGROUP_ROTATE(2, ss1grp0, 2, _V(0.497, 0, 19.734), _V(0,-1,0), (float)(PI05));
	Section0 = AddAnimationComponent (anim_ss, 0, 1, ss1trans[0]);

	static UINT ss1grp1[2] = {9,12};
	ss1trans[1] = new MGROUP_ROTATE(2, ss1grp1, 2, _V(-0.497, 0, 19.734), _V(0,1,0), (float)(PI05));
	Section1 = AddAnimationComponent (anim_ss, 0, 1, ss1trans[1]);

	static UINT ss1grp2[1] = {7};
	ss1trans[2] = new MGROUP_TRANSLATE(2, ss1grp2, 1, _V(-1.25, 0, 0));
	AddAnimationComponent (anim_ss, 0, 1, ss1trans[2], Section0);

	static UINT ss1grp3[1] = {13};
	ss1trans[3] = new MGROUP_TRANSLATE(2, ss1grp3, 1, _V(1.25, 0, 0));
	AddAnimationComponent (anim_ss, 0, 1, ss1trans[3], Section1);

	//static UINT ss1grp4[2] = {10,12};
	//ss1trans[4] = new MGROUP_SCALE(2, ss1grp4, 2, _V(0,0,21.3),_V(0.48,1,0.48));
	//AddAnimationComponent (anim_ss, 0, 1, ss1trans[4]);

}


void f9h_stg2::DefineAnimationsExpendable (void)
{
	anim_ss = CreateAnimation (0);

	static UINT ss1grp0[2] = {4,6};
	ss1trans[0] = new MGROUP_ROTATE(2, ss1grp0, 2, _V(0.497, 0, 19.734), _V(0,-1,0), (float)(PI05));
	Section0 = AddAnimationComponent (anim_ss, 0, 1, ss1trans[0]);

	static UINT ss1grp1[2] = {5,8};
	ss1trans[1] = new MGROUP_ROTATE(2, ss1grp1, 2, _V(-0.497, 0, 19.734), _V(0,1,0), (float)(PI05));
	Section1 = AddAnimationComponent (anim_ss, 0, 1, ss1trans[1]);

	static UINT ss1grp2[1] = {3};
	ss1trans[2] = new MGROUP_TRANSLATE(2, ss1grp2, 1, _V(-1.25, 0, 0));
	AddAnimationComponent (anim_ss, 0, 1, ss1trans[2], Section0);

	static UINT ss1grp3[1] = {9};
	ss1trans[3] = new MGROUP_TRANSLATE(2, ss1grp3, 1, _V(1.25, 0, 0));
	AddAnimationComponent (anim_ss, 0, 1, ss1trans[3], Section1);

	//static UINT ss1grp4[2] = {6,8};
	//ss1trans[4] = new MGROUP_SCALE(2, ss1grp4, 2, _V(0,0,21.3),_V(0.48,1,0.48));
	//AddAnimationComponent (anim_ss, 0, 1, ss1trans[4]);

}


// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void f9h_stg2::clbkSetClassCaps (FILEHANDLE cfg)
{
	strcpy(refname, "NULL");
	strcpy(refnamep, "NULL");
	strcpy(refnames, "NULL");
	for (int i = 0; i < 100; i++)
	{
	strcpy(pldname[i], "NULL");
	}

	step_time = 0;
	ap_status = 0;
	op_status = 0;
	cam_status = 0;
	ap_function = 0;
	old_ap_dist = 0;
	bias = 0;
	pitch_angle = 90*RAD;
	//intfair_status = 0;
	drg = 0;
	nfr = 0;
	//eng_status = 0;
	//calc_sw = 0;
	rcs_sw2 = 0;
	vrot_lim = 5*RAD;
	azimuth = 0;
	azim_local = _V(0,-1,0);
	vazim1 = _V(0,1,0);
	vazim2 = _V(0,1,0);
	flight_vector = _V(0,1,0);
	timer = 0;
	start_sw = 0;
	pst_lvl = 0;
	pst_lvl2 = 0;
	tu_sw = 0;
	core_fuel = 411000;
	ss_proc = 0;
	no_legs = 0;
	cntr_skin_sw = 0;
	port_skin_sw = 0;
	stbd_skin_sw = 0;

	he_vent_status = 0;
	he_vent_level = 0.0;
	he_vent_timer = 0.0;
	large_fair = 0;
	recoverable_fair = 0;

	core_throttle = 0;
	bstr_throttle = 0;

	flight_path_angle = 0;
	lamp_status = 0;

	time_acc_limit = false;
	FAIR_MASS = 1750;

	red_fact = 0;

	gcInitialize();

	// physical vessel parameters
	SetSize (30);
	oapiReadItem_int (cfg, "LargeFairing", large_fair);
	oapiReadItem_int (cfg, "RecoverableFairing", recoverable_fair);
	if(large_fair == 1)FAIR_MASS = 2500;
	SetEmptyMass (STG0DRYMASS+STG1DRYMASS+STG2DRYMASS+FAIR_MASS);
	SetPMI(_V(129,139,14));
	SetCrossSections (_V(159,359,30));
	SetCW(0.5,1,1,1);
	SetRotDrag (_V(0.5,0.5,0.1));
	SetTouchdownPoints ( tdvtx_geardown, 4);

	// camera parameters
	SetCameraOffset (_V(0,3,0));
	SetCameraRotationRange(PI,PI,(0.4*PI),(0.4*PI));
	//DefineAnimations ();

	if(oapiReadItem_int (cfg, "NoFairing", nfr) == true)
	{
		if(nfr == 1) intfair_status = 1;
	}

	

	if(oapiReadItem_int (cfg, "Dragon", drg) == true)
	{
		if (drg == 1)
		{
		hMStg2 = oapiLoadMeshGlobal("FalconHeavy/f9h_stg2_drg");
		SetMeshVisibilityMode(AddMesh(hMStg2, &STG1STG2_OFF),MESHVIS_ALWAYS);
		intfair_status = 1;
		accel_limit = 29.43;
		}
		else if (drg == 2)
		{
		hMStg2 = oapiLoadMeshGlobal("FalconHeavy/f9h_stg2_drg2");
		SetMeshVisibilityMode(AddMesh(hMStg2, &STG1STG2_OFF),MESHVIS_ALWAYS);
		intfair_status = 1;
		accel_limit = 29.43;
		}
		else
		{
		hMStg2 = oapiLoadMeshGlobal("FalconHeavy/f9h_stg2");
		SetMeshVisibilityMode(AddMesh(hMStg2, &STG1STG2_OFF),MESHVIS_ALWAYS);
		}
	}
	else 
	{
	hMStg2 = oapiLoadMeshGlobal("FalconHeavy/f9h_stg2");
	SetMeshVisibilityMode(AddMesh(hMStg2, &STG1STG2_OFF),MESHVIS_ALWAYS);
	}

	/////// extra insulation
	int ui = 0;
	if(oapiReadItem_int (cfg, "UpperInsulation", ui) == true)
	{
	if(ui == 1)
		{
		bs4 = oapiLoadTexture ("FalconHeavy/f9_stg2_geo.dds", true);
		if(drg == 0)bs3 = oapiGetTextureHandle (hMStg2, 2);
		else bs3 = oapiGetTextureHandle (hMStg2, 3);
		oapiBlt(bs3, bs4, 0, 0, 0, 0, 512, 512);
		oapiReleaseTexture (bs4);

		bs6 = oapiLoadTexture ("FalconHeavy/f9_stg2_adaptor_geo.dds", true);
		if(drg == 0)bs5 = oapiGetTextureHandle (hMStg2, 5);
		else bs5 = oapiGetTextureHandle (hMStg2, 4);
		oapiBlt(bs5, bs6, 0, 0, 0, 0, 512, 128);
		oapiReleaseTexture (bs6);
		}
	}
	///////////////////////////////

	MESHHANDLE hMsh_cntr;
	MESHHANDLE hMsh_stbd;
	MESHHANDLE hMsh_port;

	if(oapiReadItem_int (cfg, "Expendable", no_legs) == true)
	{
		if(no_legs == 1)
		{
		hMsh_cntr = oapiLoadMeshGlobal("FalconHeavy/f9h_stg1_centre_nolegs");
		SetMeshVisibilityMode(AddMesh(hMsh_cntr, &STG1_OFF),MESHVIS_ALWAYS);
		//SetMeshVisibilityMode(AddMesh("FalconHeavy/f9h_stg1_centre_nolegs", &STG1_OFF),MESHVIS_ALWAYS);
		SetMeshVisibilityMode(AddMesh("FalconHeavy/f9h_interstg_nolegs", &STG1INT_OFF),MESHVIS_ALWAYS);

		if(large_fair == 0)
		{
		SetMeshVisibilityMode(AddMesh("FalconHeavy/f9h_fair_r", &STG1FAIR_OFF),MESHVIS_ALWAYS);
		SetMeshVisibilityMode(AddMesh("FalconHeavy/f9h_fair_l", &STG1FAIR_OFF),MESHVIS_ALWAYS);
		}
		else
		{
		SetMeshVisibilityMode(AddMesh("FalconHeavy/f9h_fair_large_r", &STG1FAIR_LARGE_OFF),MESHVIS_ALWAYS);
		SetMeshVisibilityMode(AddMesh("FalconHeavy/f9h_fair_large_l", &STG1FAIR_LARGE_OFF),MESHVIS_ALWAYS);
		}

		hMsh_stbd = oapiLoadMeshGlobal("FalconHeavy/f9h_stg1_starboard");
		SetMeshVisibilityMode(AddMesh(hMsh_stbd, &STG0_STBD_OFF),MESHVIS_ALWAYS);

		hMsh_port = oapiLoadMeshGlobal("FalconHeavy/f9h_stg1_port");
		SetMeshVisibilityMode(AddMesh(hMsh_port, &STG0_PORT_OFF),MESHVIS_ALWAYS);
		}
		else if(no_legs == 2)
		{
		hMsh_cntr = oapiLoadMeshGlobal("FalconHeavy/f9h_stg1_centre_nolegs");
		SetMeshVisibilityMode(AddMesh(hMsh_cntr, &STG1_OFF),MESHVIS_ALWAYS);
		SetMeshVisibilityMode(AddMesh("FalconHeavy/f9h_interstg_nolegs", &STG1INT_OFF),MESHVIS_ALWAYS);

		if(large_fair == 0)
		{
		SetMeshVisibilityMode(AddMesh("FalconHeavy/f9h_fair_r", &STG1FAIR_OFF),MESHVIS_ALWAYS);
		SetMeshVisibilityMode(AddMesh("FalconHeavy/f9h_fair_l", &STG1FAIR_OFF),MESHVIS_ALWAYS);
		}
		else
		{
		SetMeshVisibilityMode(AddMesh("FalconHeavy/f9h_fair_large_r", &STG1FAIR_LARGE_OFF),MESHVIS_ALWAYS);
		SetMeshVisibilityMode(AddMesh("FalconHeavy/f9h_fair_large_l", &STG1FAIR_LARGE_OFF),MESHVIS_ALWAYS);
		}

		hMsh_stbd = oapiLoadMeshGlobal("FalconHeavy/f9h_stg1_starboard_nolegs");
		SetMeshVisibilityMode(AddMesh(hMsh_stbd, &STG0_STBD_OFF),MESHVIS_ALWAYS);

		hMsh_port = oapiLoadMeshGlobal("FalconHeavy/f9h_stg1_port_nolegs");
		SetMeshVisibilityMode(AddMesh(hMsh_port, &STG0_PORT_OFF),MESHVIS_ALWAYS);
		}
		else
		{
		hMsh_cntr = oapiLoadMeshGlobal("FalconHeavy/f9h_stg1_centre");
		SetMeshVisibilityMode(AddMesh(hMsh_cntr, &STG1_OFF),MESHVIS_ALWAYS);
		SetMeshVisibilityMode(AddMesh("FalconHeavy/f9h_interstg", &STG1INT_OFF),MESHVIS_ALWAYS);

		if(large_fair == 0)
		{
		SetMeshVisibilityMode(AddMesh("FalconHeavy/f9h_fair_r", &STG1FAIR_OFF),MESHVIS_ALWAYS);
		SetMeshVisibilityMode(AddMesh("FalconHeavy/f9h_fair_l", &STG1FAIR_OFF),MESHVIS_ALWAYS);
		}
		else
		{
		SetMeshVisibilityMode(AddMesh("FalconHeavy/f9h_fair_large_r", &STG1FAIR_LARGE_OFF),MESHVIS_ALWAYS);
		SetMeshVisibilityMode(AddMesh("FalconHeavy/f9h_fair_large_l", &STG1FAIR_LARGE_OFF),MESHVIS_ALWAYS);
		}

		hMsh_stbd = oapiLoadMeshGlobal("FalconHeavy/f9h_stg1_starboard");
		SetMeshVisibilityMode(AddMesh(hMsh_stbd, &STG0_STBD_OFF),MESHVIS_ALWAYS);

		hMsh_port = oapiLoadMeshGlobal("FalconHeavy/f9h_stg1_port");
		SetMeshVisibilityMode(AddMesh(hMsh_port, &STG0_PORT_OFF),MESHVIS_ALWAYS);
		}

	}
	else
	{
	hMsh_cntr = oapiLoadMeshGlobal("FalconHeavy/f9h_stg1_centre");
	SetMeshVisibilityMode(AddMesh(hMsh_cntr, &STG1_OFF),MESHVIS_ALWAYS);
	SetMeshVisibilityMode(AddMesh("FalconHeavy/f9h_interstg", &STG1INT_OFF),MESHVIS_ALWAYS);

	if(large_fair == 0)
		{
		SetMeshVisibilityMode(AddMesh("FalconHeavy/f9h_fair_r", &STG1FAIR_OFF),MESHVIS_ALWAYS);
		SetMeshVisibilityMode(AddMesh("FalconHeavy/f9h_fair_l", &STG1FAIR_OFF),MESHVIS_ALWAYS);
		}
		else
		{
		SetMeshVisibilityMode(AddMesh("FalconHeavy/f9h_fair_large_r", &STG1FAIR_LARGE_OFF),MESHVIS_ALWAYS);
		SetMeshVisibilityMode(AddMesh("FalconHeavy/f9h_fair_large_l", &STG1FAIR_LARGE_OFF),MESHVIS_ALWAYS);
		}

	hMsh_stbd = oapiLoadMeshGlobal("FalconHeavy/f9h_stg1_starboard");
	SetMeshVisibilityMode(AddMesh(hMsh_stbd, &STG0_STBD_OFF),MESHVIS_ALWAYS);

	hMsh_port = oapiLoadMeshGlobal("FalconHeavy/f9h_stg1_port");
	SetMeshVisibilityMode(AddMesh(hMsh_port, &STG0_PORT_OFF),MESHVIS_ALWAYS);
	}

	if(oapiReadItem_string(cfg, "Adaptor", adname) == TRUE)
	{
	SetMeshVisibilityMode(AddMesh(adname, &ADAPTOR_OFF),MESHVIS_ALWAYS);
	}
	//

	if(no_legs == 0)DefineAnimations ();
	else DefineAnimationsExpendable ();

	/////// charred skin
	if(oapiReadItem_int (cfg, "CntrUsed", cntr_skin_sw) == true)
	{
	if(cntr_skin_sw == 1)
		{
		bs2 = oapiLoadTexture ("FalconHeavy/f9_hull_logo_char.dds", true);
		bs1 = oapiGetTextureHandle (hMsh_cntr, 1);
		oapiBlt(bs1, bs2, 0, 0, 0, 0, 512, 1024);
		oapiReleaseTexture (bs2);
		}
	}

	if(oapiReadItem_int (cfg, "PortUsed", port_skin_sw) == true)
	{
	if(port_skin_sw == 1)
		{
		bs2 = oapiLoadTexture ("FalconHeavy/f9_hull_logo_char_port.dds", true);
		bs1 = oapiGetTextureHandle (hMsh_port, 1);
		oapiBlt(bs1, bs2, 0, 0, 0, 0, 512, 1024);
		oapiReleaseTexture (bs2);
		}
	}

	if(oapiReadItem_int (cfg, "StbdUsed", stbd_skin_sw) == true)
	{
	if(stbd_skin_sw == 1)
		{
		bs2 = oapiLoadTexture ("FalconHeavy/f9_hull_logo_char_stbd.dds", true);
		bs1 = oapiGetTextureHandle (hMsh_stbd, 1);
		oapiBlt(bs1, bs2, 0, 0, 0, 0, 512, 1024);
		oapiReleaseTexture (bs2);
		}
	}

	oapiReadItem_bool (cfg, "TimeAccLimit", time_acc_limit);

	// propellant resources
	
	hProp0 = CreatePropellantResource (STG0FUELMASS,STG0FUELMASS);
	hProp1 = CreatePropellantResource (STG1FUELMASS,0);
	hProp2 = CreatePropellantResource (STG2FUELMASS,STG2FUELMASS);
	hRCSProp2 = CreatePropellantResource (STG2RCSFUELMASS,STG2RCSFUELMASS);

	// ***************** thruster definitions *******************

	th_rcs0[0] = CreateThruster (_V( 0, 2, -31), _V( 1,0,0), STG0RCSTH, hProp0, STG1ISPV, STG1ISP0);
	th_rcs0[1] = CreateThruster (_V( 0, 2, -31), _V(-1,0,0), STG0RCSTH, hProp0, STG1ISPV, STG1ISP0);
	th_rcs0[2] = CreateThruster (_V( 0,-2, -31), _V( 1,0,0), STG0RCSTH, hProp0, STG1ISPV, STG1ISP0);
	th_rcs0[3] = CreateThruster (_V( 0,-2, -31), _V(-1,0,0), STG0RCSTH, hProp0, STG1ISPV, STG1ISP0);

	th_rcs0[4] = CreateThruster (_V( 0, 4, -31), _V( 1,0,0), STG0RCSTH, hProp0, STG1ISPV, STG1ISP0);
	th_rcs0[5] = CreateThruster (_V( 0, 4, -31), _V(-1,0,0), STG0RCSTH, hProp0, STG1ISPV, STG1ISP0);
	th_rcs0[6] = CreateThruster (_V( 0,-4, -31), _V( 1,0,0), STG0RCSTH, hProp0, STG1ISPV, STG1ISP0);
	th_rcs0[7] = CreateThruster (_V( 0,-4, -31), _V(-1,0,0), STG0RCSTH, hProp0, STG1ISPV, STG1ISP0);

	th_rcs0[8] = CreateThruster (_V( 2, 0, -31), _V(0, 1,0), STG0RCSTH, hProp0, STG1ISPV, STG1ISP0);
	th_rcs0[9] = CreateThruster (_V( 2, 0, -31), _V(0,-1,0), STG0RCSTH, hProp0, STG1ISPV, STG1ISP0);
	th_rcs0[10] = CreateThruster (_V(-2,0, -31), _V(0, 1,0), STG0RCSTH, hProp0, STG1ISPV, STG1ISP0);
	th_rcs0[11] = CreateThruster (_V(-2,0, -31), _V(0,-1,0), STG0RCSTH, hProp0, STG1ISPV, STG1ISP0);

///////////// post booster staging	

	th_rcs[0] = CreateThruster (_V( 0, 2, -31), _V( 1,0,0), STG1RCSTH, hProp1, STG1ISPV, STG1ISP0);
	th_rcs[1] = CreateThruster (_V( 0, 2, -31), _V(-1,0,0), STG1RCSTH, hProp1, STG1ISPV, STG1ISP0);
	th_rcs[2] = CreateThruster (_V( 0,-2, -31), _V( 1,0,0), STG1RCSTH, hProp1, STG1ISPV, STG1ISP0);
	th_rcs[3] = CreateThruster (_V( 0,-2, -31), _V(-1,0,0), STG1RCSTH, hProp1, STG1ISPV, STG1ISP0);

	th_rcs[4] = CreateThruster (_V( 0, 4, -31), _V( 1,0,0), STG1RCSTH, hProp1, STG1ISPV, STG1ISP0);
	th_rcs[5] = CreateThruster (_V( 0, 4, -31), _V(-1,0,0), STG1RCSTH, hProp1, STG1ISPV, STG1ISP0);
	th_rcs[6] = CreateThruster (_V( 0,-4, -31), _V( 1,0,0), STG1RCSTH, hProp1, STG1ISPV, STG1ISP0);
	th_rcs[7] = CreateThruster (_V( 0,-4, -31), _V(-1,0,0), STG1RCSTH, hProp1, STG1ISPV, STG1ISP0);

	th_rcs[8] = CreateThruster (_V( 2, 0, -31), _V(0, 1,0), STG1RCSTH, hProp1, STG1ISPV, STG1ISP0);
	th_rcs[9] = CreateThruster (_V( 2, 0, -31), _V(0,-1,0), STG1RCSTH, hProp1, STG1ISPV, STG1ISP0);
	th_rcs[10] = CreateThruster (_V(-2,0, -31), _V(0, 1,0), STG1RCSTH, hProp1, STG1ISPV, STG1ISP0);
	th_rcs[11] = CreateThruster (_V(-2,0, -31), _V(0,-1,0), STG1RCSTH, hProp1, STG1ISPV, STG1ISP0);


	////// stg2

	th_rcs[12] = CreateThruster (_V( 0, 0, -9.8), _V( 1,0,0), STG2RCSTH2, hProp2, STG2ISPV, STG2ISP0); //yl
	th_rcs[13] = CreateThruster (_V( 0, 0, -9.8), _V(-1,0,0), STG2RCSTH2, hProp2, STG2ISPV, STG2ISP0); //  yr
	th_rcs[14] = CreateThruster (_V( 0, 0, -9.8), _V( 0,-1,0), STG2RCSTH2, hProp2, STG2ISPV, STG2ISP0); // pup
	th_rcs[15] = CreateThruster (_V( 0, 0, -9.8), _V( 0,1,0), STG2RCSTH2, hProp2, STG2ISPV, STG2ISP0); // pd

	th_rcs[16] = CreateThruster (_V(1.706,0.123,-4.365), _V(-0.25,-0.866,0.433), STG2RCSTH1, hRCSProp2, STG2RCSISP, STG2RCSISP); // br
	th_rcs[17] = CreateThruster (_V(-1.706,0.123,-4.365), _V( 0.25,-0.866,0.433), STG2RCSTH1, hRCSProp2, STG2RCSISP, STG2RCSISP);// bl
	th_rcs[18] = CreateThruster (_V(1.706,-0.123,-4.365), _V(-0.25,0.866,0.433), STG2RCSTH1, hRCSProp2, STG2RCSISP, STG2RCSISP); // bl
	th_rcs[19] = CreateThruster (_V(-1.706,-0.123,-4.365), _V( 0.25,0.866,0.433), STG2RCSTH1, hRCSProp2, STG2RCSISP, STG2RCSISP);// br

	th_rcs[20] = CreateThruster (_V(0.123,1.706,-4.365), _V(-0.866,-0.25,0.433), STG2RCSTH1, hRCSProp2, STG2RCSISP, STG2RCSISP); // yl
	th_rcs[21] = CreateThruster (_V(-0.123,1.706,-4.365), _V( 0.866,-0.25,0.433), STG2RCSTH1, hRCSProp2, STG2RCSISP, STG2RCSISP);// yr
	th_rcs[22] = CreateThruster (_V(0.123,-1.706,-4.365), _V(-0.866,0.25,0.433), STG2RCSTH1, hRCSProp2, STG2RCSISP, STG2RCSISP); // yl
	th_rcs[23] = CreateThruster (_V(-0.123,-1.706,-4.365), _V( 0.866,0.25,0.433), STG2RCSTH1, hRCSProp2, STG2RCSISP, STG2RCSISP);// yr

	th_group[0] = th_rcs0[9];
	th_group[1] = th_rcs0[11];
	htgrppup2 = CreateThrusterGroup (th_group, 2, THGROUP_ATT_PITCHUP);

	th_group[0] = th_rcs0[8];
	th_group[1] = th_rcs0[10];
	htgrppdown2 = CreateThrusterGroup (th_group, 2, THGROUP_ATT_PITCHDOWN);

	th_group[0] = th_rcs0[1];
	th_group[1] = th_rcs0[3];
	htgrpyright2 = CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWRIGHT);

	th_group[0] = th_rcs0[0];
	th_group[1] = th_rcs0[2];
	htgrpyleft2 = CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWLEFT);

	th_group[0] = th_rcs0[4];
	th_group[1] = th_rcs0[7];
	htgrpbright2 = CreateThrusterGroup (th_group, 2, THGROUP_ATT_BANKRIGHT);

	th_group[0] = th_rcs0[5];
	th_group[1] = th_rcs0[6];
	htgrpbleft2 = CreateThrusterGroup (th_group, 2, THGROUP_ATT_BANKLEFT);

	th_main[0] = CreateThruster (_V( 0, 0, -31), _V( 0,0,1), 27*STG1MAINTH1, hProp0, STG1ISPV, STG1ISP0);
	th_main[1] = CreateThruster (_V( 0, 0, -31), _V( 0,0,1), 9*STG1MAINTH1, hProp1, STG1ISPV, STG1ISP0);
	th_main[2] = CreateThruster (_V( 0, 0, -9.8), _V( 0,0,1), STG2MAINTH1, hProp2, STG2ISPV, STG2ISP0);
	th_dump = CreateThruster (_V( 0, 0, -9.8), _V( 0,0,1), 4000, hProp2, 100, 100);

	th_group[0] = th_main[0];
	htgrpmain = CreateThrusterGroup (th_group, 1, THGROUP_MAIN);

	SURFHANDLE exhaust = oapiRegisterExhaustTexture ("FalconHeavy/KeroExhaust3");
	SURFHANDLE exhaustupper = oapiRegisterExhaustTexture ("Exhaust2");
	SURFHANDLE exhaustrcs = oapiRegisterExhaustTexture ("exhaust_atrcs");


	double exlen = 50;
	double exwid = 3.7;
	double z_coord = -31;
	if (gcEnabled() == false)z_coord = -30.5;

	MATRIX3 rm;
	VECTOR3 exhaust_pos[27];

	for (int i = 0; i < 8; i++) // core
	{
	rm = rotm(_V(0,0,1),(45*RAD*i));
	exhaust_pos[i] = mul(rm,_V(0,1.32,z_coord));

	main_exhaust[i].th = th_main[0];
	main_exhaust[i].level = NULL;
	main_exhaust[i].lpos = &exhaust_pos[i];
	main_exhaust[i].ldir = NULL; //&main2_exhaust_dir;
	main_exhaust[i].lsize = exlen;
	main_exhaust[i].wsize = exwid;
	main_exhaust[i].lofs = 0;
	main_exhaust[i].modulate = 0.8;
	main_exhaust[i].tex = exhaust;
	main_exhaust[i].flags = EXHAUST_CONSTANTPOS;
	main_exhaust[i].id = 0;
	AddExhaust (&main_exhaust[i]);
	}
	{
	int i = 8;
	exhaust_pos[i] = _V(0,0,z_coord);

	main_exhaust[i].th = th_main[0];
	main_exhaust[i].level = NULL;
	main_exhaust[i].lpos = &exhaust_pos[i];
	main_exhaust[i].ldir = NULL; //&main2_exhaust_dir;
	main_exhaust[i].lsize = exlen;
	main_exhaust[i].wsize = exwid;
	main_exhaust[i].lofs = 0;
	main_exhaust[i].modulate = 0.8;
	main_exhaust[i].tex = exhaust;
	main_exhaust[i].flags = EXHAUST_CONSTANTPOS;
	main_exhaust[i].id = 0;
	AddExhaust (&main_exhaust[i]);
	}

	for (int i = 9; i < 17; i++) // stbd
	{
	rm = rotm(_V(0,0,1),(45*RAD*i));
	exhaust_pos[i] = mul(rm,_V(0,1.32,z_coord));
	exhaust_pos[i].x = exhaust_pos[i].x + 4.242;

	main_exhaust[i].th = th_main[0];
	main_exhaust[i].level = NULL;
	main_exhaust[i].lpos = &exhaust_pos[i];
	main_exhaust[i].ldir = NULL; //&main2_exhaust_dir;
	main_exhaust[i].lsize = exlen;
	main_exhaust[i].wsize = exwid;
	main_exhaust[i].lofs = 0;
	main_exhaust[i].modulate = 0.8;
	main_exhaust[i].tex = exhaust;
	main_exhaust[i].flags = EXHAUST_CONSTANTPOS;
	main_exhaust[i].id = 0;
	AddExhaust (&main_exhaust[i]);
	}
	{
	int i = 17;
	exhaust_pos[i] = _V(4.242,0,z_coord);

	main_exhaust[i].th = th_main[0];
	main_exhaust[i].level = NULL;
	main_exhaust[i].lpos = &exhaust_pos[i];
	main_exhaust[i].ldir = NULL; //&main2_exhaust_dir;
	main_exhaust[i].lsize = exlen;
	main_exhaust[i].wsize = exwid;
	main_exhaust[i].lofs = 0;
	main_exhaust[i].modulate = 0.8;
	main_exhaust[i].tex = exhaust;
	main_exhaust[i].flags = EXHAUST_CONSTANTPOS;
	main_exhaust[i].id = 0;
	AddExhaust (&main_exhaust[i]);
	}

	for (int i = 18; i < 26; i++) // port
	{
	rm = rotm(_V(0,0,1),(45*RAD*i));
	exhaust_pos[i] = mul(rm,_V(0,1.32,z_coord));
	exhaust_pos[i].x = exhaust_pos[i].x - 4.242;

	main_exhaust[i].th = th_main[0];
	main_exhaust[i].level = NULL;
	main_exhaust[i].lpos = &exhaust_pos[i];
	main_exhaust[i].ldir = NULL; //&main2_exhaust_dir;
	main_exhaust[i].lsize = exlen;
	main_exhaust[i].wsize = exwid;
	main_exhaust[i].lofs = 0;
	main_exhaust[i].modulate = 0.8;
	main_exhaust[i].tex = exhaust;
	main_exhaust[i].flags = EXHAUST_CONSTANTPOS;
	main_exhaust[i].id = 0;
	AddExhaust (&main_exhaust[i]);
	}
	{
	int i = 26;
	exhaust_pos[i] = _V(-4.242,0,z_coord);

	main_exhaust[i].th = th_main[0];
	main_exhaust[i].level = NULL;
	main_exhaust[i].lpos = &exhaust_pos[i];
	main_exhaust[i].ldir = NULL; //&main2_exhaust_dir;
	main_exhaust[i].lsize = exlen;
	main_exhaust[i].wsize = exwid;
	main_exhaust[i].lofs = 0;
	main_exhaust[i].modulate = 0.8;
	main_exhaust[i].tex = exhaust;
	main_exhaust[i].flags = EXHAUST_CONSTANTPOS;
	main_exhaust[i].id = 0;
	AddExhaust (&main_exhaust[i]);
	}
	///////////////////////////////////////////////// core only
	for (int i = 0; i < 8; i++) // core
	{
	//rm = rotm(_V(0,0,1),(45*RAD*i));
	//exhaust_pos[i] = mul(rm,_V(0,1.32,-31));

	main_exhaust[i].th = th_main[1];
	main_exhaust[i].level = NULL;
	main_exhaust[i].lpos = &exhaust_pos[i];
	main_exhaust[i].ldir = NULL; //&main2_exhaust_dir;
	main_exhaust[i].lsize = exlen;
	main_exhaust[i].wsize = exwid;
	main_exhaust[i].lofs = 0;
	main_exhaust[i].modulate = 0.8;
	main_exhaust[i].tex = exhaust;
	main_exhaust[i].flags = EXHAUST_CONSTANTPOS;
	main_exhaust[i].id = 0;
	AddExhaust (&main_exhaust[i]);
	}
	{
	int i = 8;
	exhaust_pos[i] = _V(0,0,z_coord);

	main_exhaust[i].th = th_main[1];
	main_exhaust[i].level = NULL;
	main_exhaust[i].lpos = &exhaust_pos[i];
	main_exhaust[i].ldir = NULL; //&main2_exhaust_dir;
	main_exhaust[i].lsize = exlen;
	main_exhaust[i].wsize = exwid;
	main_exhaust[i].lofs = 0;
	main_exhaust[i].modulate = 0.8;
	main_exhaust[i].tex = exhaust;
	main_exhaust[i].flags = EXHAUST_CONSTANTPOS;
	main_exhaust[i].id = 0;
	AddExhaust (&main_exhaust[i]);
	}



	////// boosters
	//AddExhaust(th_main[0],exlen,exwid,_V(0,0,-31),_V(0,0,-1),exhaust);
	//AddExhaust(th_main[0],exlen,exwid,_V(1.32,0,-31),_V(0,0,-1),exhaust);
	//AddExhaust(th_main[0],exlen,exwid,_V(-1.32,0,-31),_V(0,0,-1),exhaust);
	//AddExhaust(th_main[0],exlen,exwid,_V(0,1.32,-31),_V(0,0,-1),exhaust);
	//AddExhaust(th_main[0],exlen,exwid,_V(0,-1.32,-31),_V(0,0,-1),exhaust);
	//AddExhaust(th_main[0],exlen,exwid,_V(0.931,0.931,-31),_V(0,0,-1),exhaust);
	//AddExhaust(th_main[0],exlen,exwid,_V(-0.931,-0.931,-31),_V(0,0,-1),exhaust);
	//AddExhaust(th_main[0],exlen,exwid,_V(-0.931,0.931,-31),_V(0,0,-1),exhaust);
	//AddExhaust(th_main[0],exlen,exwid,_V(0.931,-0.931,-31),_V(0,0,-1),exhaust);

	//AddExhaust(th_main[0],exlen,exwid,_V(4.242,0,-31),_V(0,0,-1),exhaust);
	//AddExhaust(th_main[0],exlen,exwid,_V(5.562,0,-31),_V(0,0,-1),exhaust);
	//AddExhaust(th_main[0],exlen,exwid,_V(2.922,0,-31),_V(0,0,-1),exhaust);
	//AddExhaust(th_main[0],exlen,exwid,_V(4.242,1.32,-31),_V(0,0,-1),exhaust);
	//AddExhaust(th_main[0],exlen,exwid,_V(4.242,-1.32,-31),_V(0,0,-1),exhaust);
	//AddExhaust(th_main[0],exlen,exwid,_V(5.173,0.931,-31),_V(0,0,-1),exhaust);
	//AddExhaust(th_main[0],exlen,exwid,_V(3.311,-0.931,-31),_V(0,0,-1),exhaust);
	//AddExhaust(th_main[0],exlen,exwid,_V(3.311,0.931,-31),_V(0,0,-1),exhaust);
	//AddExhaust(th_main[0],exlen,exwid,_V(5.173,-0.931,-31),_V(0,0,-1),exhaust);

	//AddExhaust(th_main[0],exlen,exwid,_V(-4.242,0,-31),_V(0,0,-1),exhaust);
	//AddExhaust(th_main[0],exlen,exwid,_V(-5.562,0,-31),_V(0,0,-1),exhaust);
	//AddExhaust(th_main[0],exlen,exwid,_V(-2.922,0,-31),_V(0,0,-1),exhaust);
	//AddExhaust(th_main[0],exlen,exwid,_V(-4.242,1.32,-31),_V(0,0,-1),exhaust);
	//AddExhaust(th_main[0],exlen,exwid,_V(-4.242,-1.32,-31),_V(0,0,-1),exhaust);
	//AddExhaust(th_main[0],exlen,exwid,_V(-5.173,0.931,-31),_V(0,0,-1),exhaust);
	//AddExhaust(th_main[0],exlen,exwid,_V(-3.311,-0.931,-31),_V(0,0,-1),exhaust);
	//AddExhaust(th_main[0],exlen,exwid,_V(-3.311,0.931,-31),_V(0,0,-1),exhaust);
	//AddExhaust(th_main[0],exlen,exwid,_V(-5.173,-0.931,-31),_V(0,0,-1),exhaust);

	///////// core only
	//AddExhaust(th_main[1],exlen,exwid,_V(0,0,-31),_V(0,0,-1),exhaust);
	//AddExhaust(th_main[1],exlen,exwid,_V(1.32,0,-31),_V(0,0,-1),exhaust);
	//AddExhaust(th_main[1],exlen,exwid,_V(-1.32,0,-31),_V(0,0,-1),exhaust);
	//AddExhaust(th_main[1],exlen,exwid,_V(0,1.32,-31),_V(0,0,-1),exhaust);
	//AddExhaust(th_main[1],exlen,exwid,_V(0,-1.32,-31),_V(0,0,-1),exhaust);
	//AddExhaust(th_main[1],exlen,exwid,_V(0.931,0.931,-31),_V(0,0,-1),exhaust);
	//AddExhaust(th_main[1],exlen,exwid,_V(-0.931,-0.931,-31),_V(0,0,-1),exhaust);
	//AddExhaust(th_main[1],exlen,exwid,_V(-0.931,0.931,-31),_V(0,0,-1),exhaust);
	//AddExhaust(th_main[1],exlen,exwid,_V(0.931,-0.931,-31),_V(0,0,-1),exhaust);


	//// stg2
	AddExhaust(th_main[2],20,2,_V(0,0,-10),_V(0,0,-1),exhaustupper);

	double exlenrcs = 2;
	double exwidrcs = 0.2;

	AddExhaust(th_rcs[16],exlenrcs,exwidrcs,exhaustrcs);
	AddExhaust(th_rcs[17],exlenrcs,exwidrcs,exhaustrcs);
	AddExhaust(th_rcs[18],exlenrcs,exwidrcs,exhaustrcs);
	AddExhaust(th_rcs[19],exlenrcs,exwidrcs,exhaustrcs);

	AddExhaust(th_rcs[20],exlenrcs,exwidrcs,exhaustrcs);
	AddExhaust(th_rcs[21],exlenrcs,exwidrcs,exhaustrcs);
	AddExhaust(th_rcs[22],exlenrcs,exwidrcs,exhaustrcs);
	AddExhaust(th_rcs[23],exlenrcs,exwidrcs,exhaustrcs);

	////// p streams
	SURFHANDLE exhaust2 = oapiRegisterParticleTexture ("FalconHeavy/Contrail4x");

	double lifetime = 0.1;
	double lifetime2 = 0.1;
	double spread = 0.02;
	double rate = 20;
	double flat = 1;

	if (gcEnabled() == false)
	{
	lifetime = 0.06;
	lifetime2 = 0.08;
	spread = 0.02;
	rate = 4;
	flat = 0.5;
	}

	PARTICLESTREAMSPEC exhaust_main1 = {
		/*0, 1, 20, 120, 0.044, 0.14, 2.6, 0.3, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-7, 1,*/
		0, 1.0, rate, 120, spread, lifetime, 2.6, 0.3, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_FLAT, flat, 0,
		PARTICLESTREAMSPEC::ATM_FLAT, flat, flat,
		exhaust2
	};

	SURFHANDLE exhaust2x = oapiRegisterParticleTexture ("FalconHeavy/Contrail3");

	PARTICLESTREAMSPEC exhaust_main2 = {
		/*0, 1, 20, 110, 0.041, 0.11, 2.5, 0.3, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-7, 1,*/
		0, 1.0, rate, 120, spread, lifetime2, 2.5, 0.3, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_FLAT, flat, 0,
		PARTICLESTREAMSPEC::ATM_FLAT, flat, flat,
		exhaust2x
	};

	///max q pstream
	SURFHANDLE exhaust3 = oapiRegisterParticleTexture ("Contrail1");
	PARTICLESTREAMSPEC maxqtrail = {
		0, 3.5, 20, 50, 0.03, 0.1, 1, 3, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-7, 1,
		exhaust3
	};
	AddParticleStream(&maxqtrail,_V(0,0,26),_V(0,0,-1), &pst_lvl);
	AddParticleStream(&maxqtrail,_V(-4,0,10),_V(0,0,-1), &pst_lvl);
	AddParticleStream(&maxqtrail,_V( 4,0,10),_V(0,0,-1), &pst_lvl);

	////// condensation
	PARTICLESTREAMSPEC condensation = {
		0, 2.0, 4, 0.5, 0.25, 4, 0.0, 0.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0.5, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-7, 1,
		exhaust3
	};

	///// fuel dump
	PARTICLESTREAMSPEC fuel_dump = {
		0, 3.0, 4, 150, 0.5, 6, 20.0, 10.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0.5, 0.5,
		PARTICLESTREAMSPEC::ATM_FLAT, 0.5, 0.5,
		exhaust3
	};

	///// he_vent
	PARTICLESTREAMSPEC he_vent = {
		0, 0.04, 10, 10, 0.05, 0.05, 4, 1, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0.5, 0.5,
		PARTICLESTREAMSPEC::ATM_FLAT, 0.5, 0.5,
		exhaust3
	};
	AddParticleStream(&he_vent,_V(-0.46,0,-5.964),_V(-0.7071,0,-0.7071), &he_vent_level);


	if (gcEnabled() == false)
	{
		AddParticleStream(&condensation,_V(0,0,8),_V(0,0,-1), &pst_lvl2);
		AddParticleStream(&condensation,_V(0,0,24),_V(0,0,-1), &pst_lvl2);
		AddParticleStream(&condensation,_V(4.242,0,8),_V(0,0,-1), &pst_lvl2);
		AddParticleStream(&condensation,_V(-4.242,0,8),_V(0,0,-1), &pst_lvl2);
	}
	/*else
	{
	exhaust_main1.srcrate = 10;
	exhaust_main2.srcrate = 10;
	exhaust_main1.atmslowdown = 1;
	exhaust_main2.atmslowdown = 1;
	exhaust_main1.lifetime = 0.2;
	exhaust_main2.lifetime = 0.2;
	exhaust_main1.srcspread = 0.00;
	exhaust_main2.srcspread = 0.00;
	exhaust_main2.growthrate = -6;
	}*/
	//////////

	AddExhaustStream(th_main[0],_V(0,0,z_coord),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(1.32,0,z_coord),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(-1.32,0,z_coord),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(0,1.32,z_coord),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(0,-1.32,z_coord),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(0.931,0.931,z_coord),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(-0.931,-0.931,z_coord),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(-0.931,0.931,z_coord),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(0.931,-0.931,z_coord),&exhaust_main1);

	AddExhaustStream(th_main[0],_V(4.242,0,z_coord),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(5.562,0,z_coord),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(2.922,0,z_coord),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(4.242,1.32,z_coord),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(4.242,-1.32,z_coord),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(5.173,0.931,z_coord),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(3.311,-0.931,z_coord),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(3.311,0.931,z_coord),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(5.173,-0.931,z_coord),&exhaust_main1);

	AddExhaustStream(th_main[0],_V(-4.242,0,z_coord),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(-5.562,0,z_coord),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(-2.922,0,z_coord),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(-4.242,1.32,z_coord),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(-4.242,-1.32,z_coord),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(-5.173,0.931,z_coord),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(-3.311,-0.931,z_coord),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(-3.311,0.931,z_coord),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(-5.173,-0.931,z_coord),&exhaust_main1);


	AddExhaustStream(th_main[0],_V(0,0,z_coord),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(1.32,0,z_coord),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(-1.32,0,z_coord),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(0,1.32,z_coord),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(0,-1.32,z_coord),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(0.931,0.931,z_coord),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(-0.931,-0.931,z_coord),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(-0.931,0.931,z_coord),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(0.931,-0.931,z_coord),&exhaust_main2);

	AddExhaustStream(th_main[0],_V(4.242,0,z_coord),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(5.562,0,z_coord),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(2.922,0,z_coord),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(4.242,1.32,z_coord),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(4.242,-1.32,z_coord),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(5.173,0.931,z_coord),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(3.311,-0.931,z_coord),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(3.311,0.931,z_coord),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(5.173,-0.931,z_coord),&exhaust_main2);

	AddExhaustStream(th_main[0],_V(-4.242,0,z_coord),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(-5.562,0,z_coord),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(-2.922,0,z_coord),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(-4.242,1.32,z_coord),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(-4.242,-1.32,z_coord),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(-5.173,0.931,z_coord),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(-3.311,-0.931,z_coord),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(-3.311,0.931,z_coord),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(-5.173,-0.931,z_coord),&exhaust_main2);

	//////////////////////////////////////////// core only

	AddExhaustStream(th_main[1],_V(0,0,z_coord),&exhaust_main1);
	AddExhaustStream(th_main[1],_V(1.32,0,z_coord),&exhaust_main1);
	AddExhaustStream(th_main[1],_V(-1.32,0,z_coord),&exhaust_main1);
	AddExhaustStream(th_main[1],_V(0,1.32,z_coord),&exhaust_main1);
	AddExhaustStream(th_main[1],_V(0,-1.32,z_coord),&exhaust_main1);
	AddExhaustStream(th_main[1],_V(0.931,0.931,z_coord),&exhaust_main1);
	AddExhaustStream(th_main[1],_V(-0.931,-0.931,z_coord),&exhaust_main1);
	AddExhaustStream(th_main[1],_V(-0.931,0.931,z_coord),&exhaust_main1);
	AddExhaustStream(th_main[1],_V(0.931,-0.931,z_coord),&exhaust_main1);

	AddExhaustStream(th_main[1],_V(0,0,z_coord),&exhaust_main2);
	AddExhaustStream(th_main[1],_V(1.32,0,z_coord),&exhaust_main2);
	AddExhaustStream(th_main[1],_V(-1.32,0,z_coord),&exhaust_main2);
	AddExhaustStream(th_main[1],_V(0,1.32,z_coord),&exhaust_main2);
	AddExhaustStream(th_main[1],_V(0,-1.32,z_coord),&exhaust_main2);
	AddExhaustStream(th_main[1],_V(0.931,0.931,z_coord),&exhaust_main2);
	AddExhaustStream(th_main[1],_V(-0.931,-0.931,z_coord),&exhaust_main2);
	AddExhaustStream(th_main[1],_V(-0.931,0.931,z_coord),&exhaust_main2);
	AddExhaustStream(th_main[1],_V(0.931,-0.931,z_coord),&exhaust_main2);

	AddExhaustStream(th_dump, _V(0,0, -9.7), &fuel_dump);
	///////////////////////////////////////////////////////
	/////AirFoils

	CreateAirfoil (LIFT_VERTICAL, _V(0,0,0), VLiftCoeff, 25, 100, 0.16);
	CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,0), HLiftCoeff, 25, 100, 0.16);
	CreateAirfoil (LIFT_VERTICAL, _V(4,0,0), VLiftCoeff, 20, 80, 0.16);
	CreateAirfoil (LIFT_HORIZONTAL, _V(4,0,0), HLiftCoeff, 20, 80, 0.16);
	CreateAirfoil (LIFT_VERTICAL, _V(-4,0,0), VLiftCoeff, 20, 80, 0.16);
	CreateAirfoil (LIFT_HORIZONTAL, _V(-4,0,0), HLiftCoeff, 20, 80, 0.16);

	////////
	char att_params[256];
	char tag_nbr[256];
	VECTOR3 att_pos = _V(0,0,0);
	VECTOR3 att_dir = _V(0,0,1);
	VECTOR3 att_rot = _V(0,1,0);

	for (int i = 0; i < 100; i++)
	{
	
	sprintf (tag_nbr, "PayloadAttach%d", i);
	if(oapiReadItem_string(cfg, tag_nbr, att_params) == TRUE)
		{
			sscanf (att_params, "%lf%lf%lf%lf%lf%lf%lf%lf%lf", &att_pos.x, &att_pos.y, &att_pos.z, &att_dir.x, &att_dir.y, &att_dir.z, &att_rot.x, &att_rot.y, &att_rot.z);
			pld_attach[i] = CreateAttachment (false, att_pos, att_dir, att_rot, "F9");
		}
	else pld_attach[i] = CreateAttachment (false, _V( 0, 0, 26.646), _V( 0,0,1), _V(0,1,0), "F9");
	}

	lpd_attach = CreateAttachment (true, _V( 0, 0, -29.45), _V( 0,0,-1), _V(0,1,0), "LP");
	//////////

	//local light
	COLOUR4 col_d = {0.125,0.1,0.06125,0};
	COLOUR4 col_s = {0,0,0,0};
	COLOUR4 col_a = {0,0,0,0};
	COLOUR4 col_white = {1,1,1,0};
	//spt_light = (SpotLight*)AddSpotLight(_V(0,0,-21), _V(0,0,-1),5000, 1e-1, 0, 1e-7, RAD*170, RAD*180, col_d, col_s, col_a);
	spt_light = (PointLight*)AddPointLight(_V(0,0,-27),5000, 1e-1, 0, 1e-6, col_d, col_s, col_a);
	//pnt_light->Activate(false);
	spt_light->Activate(false);

	COLOUR4 col_d2 = {1,1,1,0};
	COLOUR4 col_s2 = {1,1,1,0};
	COLOUR4 col_a2 = {0,0,0,0};
	
	spotlight1 = (SpotLight*)AddSpotLight(bspos, _V(0,0,1),200, 1, 0.2, 2.5e-5, (120*RAD), (180*RAD), col_d2, col_s2, col_a2);
	spotlight1->Activate(false);

	beac.shape = BEACONSHAPE_STAR;
	beac.pos = &bspos;
	beac.col = &bscol;
	beac.size = 0.2;
	beac.falloff = 0.5;
	beac.period = 0;
	beac.duration = 0;
	beac.tofs = 0;
	beac.active = false;
	AddBeacon (&beac);

}

void f9h_stg2::clbkPostCreation ()
{
 hGref = GetGravityRef();
 hMe = GetHandle();
 OBJHANDLE bbtgt = oapiGetObjectByName(refname);
 if (bbtgt == NULL)bbtgt = oapiGetBaseByName(hGref, refname);
 if (bbtgt == NULL)strcpy(refname, "NULL");

 bbtgt = oapiGetObjectByName(refnamep);
 if (bbtgt == NULL)bbtgt = oapiGetBaseByName(hGref, refnamep);
 if (bbtgt == NULL)strcpy(refnamep, "NULL");

 bbtgt = oapiGetObjectByName(refnames);
 if (bbtgt == NULL)bbtgt = oapiGetBaseByName(hGref, refnames);
 if (bbtgt == NULL)strcpy(refnames, "NULL");
 

 if (drg == 1)intfair_status = 1;
 if (nfr == 1)intfair_status = 1;

 if(intfair_status == 1)
  {
	if (eng_status == 1)
	 {
	 DelMesh(6);
	 DelMesh(5);
	 DelMesh(4);
	 DelMesh(3);
	 ConfigMain0();
	 }
	else if (eng_status == 2)
	 {
	 DelMesh(6);
	 DelMesh(5);
	 DelMesh(4);
	 DelMesh(3);
	 DelMesh(2);
	 DelMesh(1);
	 ConfigMain();
	 }
	else
	 {	
	 DelMesh(4);
	 DelMesh(3);
	 }
  }
  else if (eng_status == 1)
  {
	  DelMesh(6);
	  DelMesh(5);
	  ConfigMain0();
  }
 else if (eng_status == 2)
  {
	  DelMesh(6);
	  DelMesh(5);
	  DelMesh(2);
	  DelMesh(1);
	  ConfigMain();
  }

  
 

 if (eng_status == 0) SetPropellantMass(hProp1, 0);


	double lat, lng, rds;
	OBJHANDLE hStrongback = GetAttachmentStatus(lpd_attach);
	if(hStrongback == NULL) GetEquPos(lng,lat,rds);
	else oapiGetEquPos(hStrongback, &lng, &lat, &rds);

	min_inc = lat;
	if (abs(tgt_inc) < abs(lat))
	{
	if (tgt_inc < 0) tgt_inc = -(abs(lat));
	else tgt_inc = abs(lat);
	}
	///////////// sound
	MyID=ConnectToOrbiterSoundDLL(GetHandle());

	RequestLoadVesselWave(MyID,CTDN,"Sound\\Vessel\\countdown1.wav",DEFAULT);
	RequestLoadVesselWave(MyID,SEP,"Sound\\Vessel\\undocking.wav",BOTHVIEW_FADED_FAR);
	RequestLoadVesselWave(MyID,SEPST,"Sound\\_CustomVesselsSounds\\FalconHeavy\\stgsep.wav",DEFAULT);
	RequestLoadVesselWave(MyID,SEPFR,"Sound\\_CustomVesselsSounds\\FalconHeavy\\frsep.wav",DEFAULT);
	
	SoundOptionOnOff(MyID,PLAYRADIOATC,FALSE);
	//SoundOptionOnOff3(MyID,PLAYWHENATTITUDEMODECHANGE,FALSE);
	//SoundOptionOnOff3(MyID,PLAYGPWS,FALSE);
	SoundOptionOnOff(MyID,PLAYCOUNTDOWNWHENTAKEOFF,FALSE);
	SoundOptionOnOff(MyID,PLAYLANDINGANDGROUNDSOUND,FALSE);
	SoundOptionOnOff(MyID,PLAYCABINAIRCONDITIONING,FALSE);
	SoundOptionOnOff(MyID,PLAYCABINRANDOMAMBIANCE,FALSE);
	SoundOptionOnOff(MyID,DISPLAYTIMER,FALSE);
	SoundOptionOnOff(MyID,PLAYDOCKLANDCLEARANCE,FALSE);
	//SoundOptionOnOff3(MyID,PLAYRADARBIP,FALSE);
	pld_att_status = 0;
	for (int i = 0; i < 100; i++)
	{
	OBJHANDLE hPayload = GetAttachmentStatus(pld_attach[i]);
	if(hPayload)pld_att_status = pld_att_status+1;
	}

	if (eng_status == 2 && lamp_status == 1)
	{
	beac.active = true;
	spotlight1->Activate(true);
	}
	else
	{
	lamp_status = 0;
	beac.active = false;
	spotlight1->Activate(false);
	}


	CalcEmptyMass();

	CalcAzimuth();
	CalcAsdsPos();
	CalcFuelRes();
	
}


// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new f9h_stg2 (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (f9h_stg2*)vessel;
}


void f9h_stg2::clbkSaveState (FILEHANDLE scn)
{
	char cbuf[256];

	for (int i = 0; i < (int)strlen(refname); i++)
	{
		if (refname[i] == ' ')
		{
			refname[i] = '+';
			break;
		}
	}

	// default vessel parameters
	VESSEL3::clbkSaveState(scn);

	//sprintf (cbuf, "%d %0.4f",);
	//oapiWriteScenario_string (scn, "LEGS", cbuf);

	sprintf (cbuf, "%d %0.4f %d %d",intfair_status, ss_proc, edlo_sw_core, edlo_sw_boosters);
	oapiWriteScenario_string (scn, "FAIRS", cbuf);

	sprintf (cbuf, "%d %d %d %d %d %0.1f %d %d %d", eng_status, ap_status, ap_function, op_status, start_sw, timer, afs_status, bafs_status, tu_sw);
	oapiWriteScenario_string (scn, "SOST", cbuf);

	sprintf (cbuf, "%0.6f %0.1f %0.1f %0.1f %0.1f %0.1f %s %s %s", tgt_inc, op_meco_alt, op_tgt, fuel_reserve,booster_fuel_reserve, ap_tgt, refname, refnamep,refnames);
	oapiWriteScenario_string (scn, "SOPR", cbuf);

	sprintf (cbuf, "%0.1f %0.4f %d %d %d %d %d %0.2f", op_insertion, core_fuel, lot_hh, lot_mm, lot_ss, lot_status, lamp_status, accel_limit);
	oapiWriteScenario_string (scn, "LTMR", cbuf);

	sprintf (cbuf, "%0.4f %0.4f", core_throttle, bstr_throttle);
	oapiWriteScenario_string (scn, "TRTL", cbuf);

	sprintf (cbuf, "%0.4f", red_fact);
	oapiWriteScenario_string (scn, "NOZL", cbuf);


}
void f9h_stg2::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
    char *line;
	while (oapiReadScenario_nextline (scn, line)) {
		if (!strnicmp (line, "FAIRS", 5)) {
		sscanf (line+5, "%d%lf%d%d", &intfair_status, &ss_proc, &edlo_sw_core, &edlo_sw_boosters);
		} else if (!strnicmp (line, "SOST", 4)) {
		sscanf (line+4, "%d%d%d%d%d%lf%d%d%d", &eng_status, &ap_status, &ap_function, &op_status, &start_sw, &timer, &afs_status, &bafs_status, &tu_sw);
		} else if (!strnicmp (line, "SOPR", 4)) {
			sscanf (line+4, "%lf%lf%lf%lf%lf%lf%s%s%s",&tgt_inc, &op_meco_alt, &op_tgt, &fuel_reserve, &booster_fuel_reserve, &ap_tgt, &refname, &refnamep, &refnames);
		} else if (!strnicmp (line, "LTMR", 4)) {
			sscanf (line+4, "%lf%lf%d%d%d%d%d%lf",&op_insertion, &core_fuel, &lot_hh, &lot_mm, &lot_ss, &lot_status, &lamp_status, &accel_limit);
		} else if (!strnicmp (line, "TRTL", 4)) {
			sscanf (line+4, "%lf%lf",&core_throttle, &bstr_throttle);
		} else if (!strnicmp (line, "NOZL", 4)) {
			sscanf (line+4, "%lf",&red_fact);
		}else {
            ParseScenarioLineEx (line, vs);
			// unrecognised option - pass to Orbiter's generic parser
        }
   }


	for (int i = 0; i < (int)strlen(refname); i++)
	{
		if (refname[i] == '+')
		{
			refname[i] = ' ';
			break;
		}
	}

}

bool SetCoreFBMode(void *id, char *str, void *data)
{
	float new_mode = 3;
	sscanf(str, "%f", &new_mode);
	if (new_mode == 0)
		{
		edlo_sw_core = 0;
		return true;
		}
	else if (new_mode == 1)
		{
		edlo_sw_core = 1;
		return true;
		}
	else if (new_mode == 2)
		{
		edlo_sw_core = 2;
		return true;
		}
	else return false;
}

bool SetBoostersFBMode(void *id, char *str, void *data)
{
	float new_mode = 3;
	sscanf(str, "%f", &new_mode);
	if (new_mode == 0)
		{
		edlo_sw_boosters = 0;
		return true;
		}
	else if (new_mode == 1)
		{
		edlo_sw_boosters = 1;
		return true;
		}
	else if (new_mode == 2)
		{
		edlo_sw_boosters = 2;
		return true;
		}
	else return false;
}

bool SetINC(void *id, char *str, void *data)
{
	float new_inc;
	sscanf(str, "%f", &new_inc);
	//if (abs(new_inc) < abs(min_inc))return false;
	if(new_inc < -180) new_inc = 360 + new_inc;
	else if(new_inc > 180) new_inc = new_inc - 360;

	if (abs(new_inc) < abs(min_inc*DEG))
	{
		if(new_inc < 0) new_inc = -(abs(min_inc*DEG));
		else new_inc = (abs(min_inc*DEG));
	}
	//else 
		//{
		tgt_inc = new_inc*RAD;
		return true;
		//}
}

bool SetRF(void *id, char *str, void *data)
{
	float new_rf;
	sscanf(str, "%f", &new_rf);
	if (new_rf < 0)return false;
	else if (new_rf > 130000)return false;
	else 
		{
		fuel_reserve = new_rf;
		if (ap_tgt > (1000000 - (5*fuel_reserve)))ap_tgt = (1000000 - (5*fuel_reserve));
		if (ap_tgt < 100000)ap_tgt = 100000;
		return true;
		}
}

bool SetRF0(void *id, char *str, void *data)
{
	float new_op_insertion;
	sscanf(str, "%f", &new_op_insertion);
	if (new_op_insertion < 120)return false;
	else if (new_op_insertion < (op_meco_alt*0.001))return false;
	else if (new_op_insertion > (op_tgt*0.001))return false;

	else 
		{
		op_insertion = new_op_insertion*1000;
		double aoma = 0.001 * op_insertion;
		double aoma2 = aoma*aoma;
		//ap_tgt = 1000*((-0.000657143*aoma2) + (1.14143*aoma) - 27.00);
		//ap_tgt = 1000*((-0.002105263158*aoma2) + (1.657894737*aoma) - 72.36842105);
		double aoma3 = aoma*aoma*aoma;
		ap_tgt = 1000*((0.00000531880345*aoma3) - (0.006505647169*aoma2) + (2.841484141*aoma) - 175.621369);
		if (ap_tgt > (1000000 - (5*fuel_reserve)))ap_tgt = (1000000 - (5*fuel_reserve));
		if (ap_tgt < 100000)ap_tgt = 100000;
		return true;
		}
}

bool SetRF1(void *id, char *str, void *data)
{
	float new_rf;
	sscanf(str, "%f", &new_rf);
	if (new_rf < 0)return false;
	else if (new_rf > 60000)return false;
	else 
		{
		booster_fuel_reserve = new_rf;
		return true;
		}
}

bool SetAP(void *id, char *str, void *data)
{
	float new_ap;
	sscanf(str, "%f", &new_ap);
	if (new_ap < 100)return false;
	else if (new_ap > 1000)return false;
	else 
		{
		ap_tgt = new_ap*1000;
		if (fuel_reserve > ((1000000 - ap_tgt)/5)) fuel_reserve = ((1000000 - ap_tgt)/5);
		if (fuel_reserve < 0)fuel_reserve = 0;
		return true;
		}
}

bool SetBBTgt(void *id, char *str, void *data)
{
	char cbuf[256];
	sscanf(str, "%s", &cbuf);
	if (!strnicmp(cbuf,"NULL",4))
	{
		strcpy(refname,"NULL");
		afs_status = 0;
		return true;
	}
	OBJHANDLE bbtgt = oapiGetObjectByName(cbuf);
	if (bbtgt == NULL)bbtgt = oapiGetBaseByName(hGref, cbuf);
	if (bbtgt == NULL)return false;
	else 
		{
		strcpy(refname,cbuf);
		return true;
		}
}

bool SetBBTgtP(void *id, char *str, void *data)
{
	char cbuf[256];
	sscanf(str, "%s", &cbuf);
	if (!strnicmp(cbuf,"NULL",4))
	{
		strcpy(refnamep,"NULL");
		return true;
	}
	OBJHANDLE bbtgt = oapiGetObjectByName(cbuf);
	if (bbtgt == NULL)bbtgt = oapiGetBaseByName(hGref, cbuf);
	if (bbtgt == NULL)return false;
	else 
		{
		strcpy(refnamep,cbuf);
		return true;
		}
}

bool SetBBTgtS(void *id, char *str, void *data)
{
	char cbuf[256];
	sscanf(str, "%s", &cbuf);
	if (!strnicmp(cbuf,"NULL",4))
	{
		strcpy(refnames,"NULL");
		return true;
	}
	OBJHANDLE bbtgt = oapiGetObjectByName(cbuf);
	if (bbtgt == NULL)bbtgt = oapiGetBaseByName(hGref, cbuf);
	if (bbtgt == NULL)return false;
	else 
		{
		strcpy(refnames,cbuf);
		return true;
		}
}

bool SetOPE(void *id, char *str, void *data)
{
	float new_ope;
	sscanf(str, "%f", &new_ope);
	if (new_ope < 0)return false;
	else 
		{
			if (new_ope > op_tgt/1000)
			{
			op_meco_alt = op_tgt;
			op_tgt = new_ope*1000;
			} 
			else op_meco_alt = new_ope*1000;

			if(op_insertion < op_meco_alt)op_insertion = op_meco_alt;
			if(op_insertion > op_tgt)op_insertion = op_tgt;


			//ap_tgt = (0.75*(op_meco_alt-160000))+145000;
			double aoma = 0.001 * op_insertion;
			double aoma2 = aoma*aoma;
			//ap_tgt = 1000*((-0.000657143*aoma2) + (1.14143*aoma) - 27.00);
			//ap_tgt = 1000*((-0.002105263158*aoma2) + (1.657894737*aoma) - 72.36842105);
			double aoma3 = aoma*aoma*aoma;
			ap_tgt = 1000*((0.00000531880345*aoma3) - (0.006505647169*aoma2) + (2.841484141*aoma) - 175.621369);
			if (ap_tgt > (1000000 - (5*fuel_reserve)))ap_tgt = (1000000 - (5*fuel_reserve));
			if (ap_tgt < 100000)ap_tgt = 100000;

		return true;
		}
}

bool SetOAP(void *id, char *str, void *data)
{
	float new_oap;
	sscanf(str, "%f", &new_oap);
	if (new_oap < 160)return false;
	else 
		{
			if (new_oap < op_meco_alt/1000)
			{
			op_tgt = op_meco_alt;
			op_meco_alt = new_oap*1000;
			} 
			else op_tgt = new_oap*1000;

			if(op_insertion < op_meco_alt)op_insertion = op_meco_alt;
			if(op_insertion > op_tgt)op_insertion = op_tgt;
			//ap_tgt = (0.75*(op_meco_alt-160000))+145000;
			double aoma = 0.001 * op_insertion;
			double aoma2 = aoma*aoma;
			//ap_tgt = 1000*((-0.000657143*aoma2) + (1.14143*aoma) - 27.00);
			//ap_tgt = 1000*((-0.002105263158*aoma2) + (1.657894737*aoma) - 72.36842105);
			double aoma3 = aoma*aoma*aoma;
			ap_tgt = 1000*((0.00000531880345*aoma3) - (0.006505647169*aoma2) + (2.841484141*aoma) - 175.621369);
			if (ap_tgt > (1000000 - (5*fuel_reserve)))ap_tgt = (1000000 - (5*fuel_reserve));
			if (ap_tgt < 100000)ap_tgt = 100000;
		return true;
		}
}

bool SetPLD(void *id, char *str, void *data)
{
	if(pld_att_status > 99)return false;
	char cbuf[256];
	sscanf(str, "%s", &cbuf);
	OBJHANDLE hPld = oapiGetObjectByName(cbuf);
	if (hPld == NULL)return false;
	else
	{
	VESSEL2* v = (VESSEL2*)oapiGetVesselInterface(hPld);
	DWORD ac = v->AttachmentCount(true);
	if (ac == 0) return false;
	for (DWORD j = 0; j < ac; j++)
		{
		ATTACHMENTHANDLE hAtt = v->GetAttachmentHandle(true, j);
		OBJHANDLE hOp = v->GetAttachmentStatus(hAtt);
		if (hOp == NULL)
			{
			VESSEL2* vme = (VESSEL2*)oapiGetVesselInterface(hMe);
			if (vme->AttachChild(hPld, pld_attach[pld_att_status], hAtt) == true)
				{
					calc_sw = 0;
					strcpy(pldname[pld_att_status], cbuf);
					pld_att_status = pld_att_status + 1;
					return true;
				}
			else return false;
			}
		else if (j == ac-1) return false;
		}
	
	}
	
}

bool SetLOT(char *str1, char *str2, char *str3)
{
	float new_hh;
	sscanf(str1, "%f", &new_hh);
	float new_mm;
	sscanf(str2, "%f", &new_mm);
	float new_ss;
	sscanf(str3, "%f", &new_ss);

	if (new_hh > 23) return false;
	else if (new_hh < 0) return false;
	else if (new_mm > 59) return false;
	else if (new_mm < 0) return false;
	else if (new_ss > 59) return false;
	else if (new_ss < 0) return false;
	else
		{
			lot_hh = (int)(new_hh);
			lot_mm = (int)(new_mm);
			lot_ss = (int)(new_ss);
			return true;
		}
	return true;
}

BOOL CALLBACK DlgProc (HWND, UINT, WPARAM, LPARAM);

int f9h_stg2::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate) {

	HWND hDbox;

if (!down) return 0; // only process keydown events

  if (KEYMOD_SHIFT (kstate)) 
  {	 
	switch (key)
	{
	

	} 
  }

  else if (KEYMOD_CONTROL (kstate)) 
  { 
	switch (key)
	{
		
	} 
  }

else { // unmodified keys
    switch (key) {

		//case OAPI_KEY_G: // remove fairings
		//return 1;

		//case OAPI_KEY_B: //
		//RESETKEY (kstate, OAPI_KEY_H);
		///*if (dlgh_status == 0)
		//{
		//oapiOpenInputBox("Core Flyback Mode [0=Off 1=FB+EDL 2=EDL Only]",SetCoreFBMode, 0, 50, 0);
		//dlgh_status = 1;
		//}
		//else if (dlgh_status == 1)
		//{
		//oapiOpenInputBox("Boosters Flyback Mode [0=Off 1=FB+EDL 2=EDL Only]",SetBoostersFBMode, 0, 50, 0);
		//dlgh_status = 0;
		//}*/
		//if(edlo_sw_core == 0)edlo_sw_core = 1;
		//else edlo_sw_core = 0;
		//if(edlo_sw_boosters == 0)edlo_sw_boosters = 1;
		//else edlo_sw_boosters = 0;
		//return 1;

		//case OAPI_KEY_L: // 
		//RESETKEY (kstate, OAPI_KEY_L);
		//if (dlgl_status == 0)
		//{
		//double lat, lng, rds;
		//GetEquPos(lng,lat,rds);
		//min_inc = DEG*lat;
		//oapiOpenInputBox("Enter Equ.Inc(+North : -South)",SetINC, 0, 50, 0);
		//dlgl_status = 1;
		//}
		//else if (dlgl_status == 1)
		//{
		////oapiOpenInputBox("Enter Perigee Alt.> 160 (km)",SetOPE, 0, 50, 0);
		//oapiOpenInputBox("Enter Apogee Alt.> 160 (km)",SetOAP, 0, 50, 0);
		//dlgl_status = 2;
		//}
		//else if (dlgl_status == 2)
		//{
		////oapiOpenInputBox("Enter Apogee Alt.> 160 (km)",SetOAP, 0, 50, 0);
		//oapiOpenInputBox("Enter Perigee Alt.> 160 (km)",SetOPE, 0, 50, 0);
		//dlgl_status = 0;
		//}
		//return 1;
		
		

		case OAPI_KEY_J: // jettison pld
		if (intfair_status == 1)
		{
		for (int j = 0; j < 100; j++)
			{
			hpd = GetAttachmentStatus(pld_attach[99-j]);
			if (hpd != NULL)
				{
				DetachChild(pld_attach[99-j], 1.0);
				strcpy(pldname[99-j], "NULL");
				pld_att_status = pld_att_status-1;
				CalcEmptyMass();
				//oapiSetFocusObject(hpd);
				break;
				}
			}
		}
		else if (intfair_status == 0)
		{
		JettisonFair();
		}
		return 1;

		//case OAPI_KEY_P: // add payload
		//hMe = GetHandle();
		//if(pld_att_status == 0)oapiOpenInputBox("Enter Payload1 Name", SetPLD, 0, 50, 0);
		//else if(pld_att_status == 1)oapiOpenInputBox("Enter Payload2 Name", SetPLD, 0, 50, 0);
		//return 1;

		/*case OAPI_KEY_K:
		hGref = GetGravityRef();
		if (dlgb_status == 0)
		{
		oapiOpenInputBox("Enter Core Flyback Target Name [or NULL]",SetBBTgt, 0, 50, 0);
		dlgb_status = 1;
		}
		else if (dlgb_status == 1)
		{
		oapiOpenInputBox("Enter Port Booster Flyback Tgt.Name [or NULL]",SetBBTgtP, 0, 50, 0);
		dlgb_status = 2;
		}
		else if (dlgb_status == 2)
		{
		oapiOpenInputBox("Enter Stbd.Booster Flyback Tgt.Name [or NULL]",SetBBTgtS, 0, 50, 0);
		dlgb_status = 0;
		}
		return 1;*/

		case OAPI_KEY_K:
			hDbox = oapiFindDialog (g_Param.hDLL, IDD_CTRL);
			if (hDbox) oapiCloseDialog (hDbox);
			else hDbox = oapiOpenDialogEx(g_Param.hDLL, IDD_CTRL, DlgProc,DLG_CAPTIONCLOSE, this);
		return 1;

		case COMM_CALC_MASS:
		CalcEmptyMass();
		return 1;

		case COMM_CALC_FRE:
		CalcFuelRes();
		return 1;

		case COMM_CALC_ASDS:
		CalcAsdsPos();
		return 1;

		/*case OAPI_KEY_M:
		if (dlg_status == 0)
		{
		oapiOpenInputBox("Enter Apogee at Core MECO 100-800 (km alt.)",SetAP, 0, 50, 0);
		dlg_status = 1;
		}
		else if (dlg_status == 1)
		{
		oapiOpenInputBox("Enter Fuel CrossFeed 0-1800 (kg/s)",SetRF0, 0, 50, 0);
		dlg_status = 0;
		}
		return 1;*/


		/*case OAPI_KEY_G:
		if (dlgf_status == 0)
		{
		oapiOpenInputBox("Enter Core Flyback Fuel Res.(kg)",SetRF, 0, 50, 0);
		dlgf_status = 1;
		}
		else if (dlgf_status == 1)
		{
		oapiOpenInputBox("Enter Booster Flyback Fuel Res.(kg, min.300)",SetRF1, 0, 50, 0);
		dlgf_status = 0;
		}
		return 1;*/

		case OAPI_KEY_V: // ascent pilot
		if (eng_status < 2)
			{
			if(ap_status == 0)ap_status = 1;
			else ap_status = 0;
			op_status = 0;
			KillAllRCS2();
			}
		else if (eng_status == 2)
			{
			if(op_status == 0)op_status = 1;
			else op_status = 0;
			ap_status = 0;
			KillAllRCS2();
			}
		apop_status = ap_status + op_status;
		return 1;

		case OAPI_KEY_E:
			if (cam_status == 0)cam_status = 1;
			else if (cam_status == 1)cam_status = 0;
			SetCamDir();
		return 1;

		case OAPI_KEY_B:
		if(GetThrusterGroupLevel(THGROUP_MAIN) == 0)
		{
			if(GetThrusterLevel(th_dump) == 0) SetThrusterLevel(th_dump, 1);
			else if(GetThrusterLevel(th_dump) > 0) SetThrusterLevel(th_dump, 0);
		}
		else SetThrusterLevel(th_dump, 0);
		return 1;

		case OAPI_KEY_N:
			if (eng_status == 0)JettisonBoosters();
			else if (eng_status == 1)JettisonStg();
		return 1;

		case OAPI_KEY_M:
			if (eng_status == 2)
			{
			if(lamp_status == 0) 
				{
				lamp_status = 1;
				beac.active = true;
				spotlight1->Activate(true);
				}
			else 
				{
				lamp_status = 0;
				beac.active = false;
				spotlight1->Activate(false);
				}
			}
			else lamp_status = 0;
		return 1;
	}
  }
  return 0;
}


DLLCLBK void InitModule (HINSTANCE hModule)
{
	g_Param.hDLL = hModule;
	oapiRegisterCustomControls (hModule);
	
	
}

DLLCLBK void ExitModule (HINSTANCE hModule)
{
	oapiUnregisterCustomControls (hModule);
	
}
void InitDialog(HWND hWnd)
{
	

	char cbuf[256];
	char lotbuf[2];
	char lotbuftemp[2];
	strcpy(lotbuftemp, "0");
	
	sprintf (cbuf, "%0.0f", fuel_reserve);
	SetWindowText (GetDlgItem (hWnd, IDC_FUEL_RES), cbuf);
	if (eng_status == 0)sprintf (cbuf, "Enter");
	else sprintf (cbuf, "N/A");
	SetWindowText (GetDlgItem (hWnd, IDC_FUEL_APPLY), cbuf);

	sprintf (cbuf, "%0.0f", booster_fuel_reserve); //////// boosters
	SetWindowText (GetDlgItem (hWnd, IDC_FUELB_RES), cbuf);
	if (eng_status == 0)sprintf (cbuf, "Enter");
	else sprintf (cbuf, "N/A");
	SetWindowText (GetDlgItem (hWnd, IDC_FUELB_APPLY), cbuf);

	sprintf (cbuf, "%s", refname);
	SetWindowText (GetDlgItem (hWnd, IDC_TGTNM), cbuf);
	if (eng_status == 0)sprintf (cbuf, "Enter");
	else sprintf (cbuf, "N/A");
	SetWindowText (GetDlgItem (hWnd, IDC_TGTNM_APPLY), cbuf);

	sprintf (cbuf, "%s", refnamep); ///////// PORT
	SetWindowText (GetDlgItem (hWnd, IDC_TGTNMP), cbuf);
	if (eng_status == 0)sprintf (cbuf, "Enter");
	else sprintf (cbuf, "N/A");
	SetWindowText (GetDlgItem (hWnd, IDC_TGTNMP_APPLY), cbuf);

	sprintf (cbuf, "%s", refnames); ///////// STBD
	SetWindowText (GetDlgItem (hWnd, IDC_TGTNMS), cbuf);
	if (eng_status == 0)sprintf (cbuf, "Enter");
	else sprintf (cbuf, "N/A");
	SetWindowText (GetDlgItem (hWnd, IDC_TGTNMS_APPLY), cbuf);

	if (afs_status == 0)sprintf(cbuf,"OFF");
	else sprintf(cbuf,"ENABLED");
	SetWindowText (GetDlgItem (hWnd, IDC_FB_STATUS), cbuf);
	if (eng_status == 0)sprintf (cbuf, "On/Off");
	else sprintf (cbuf, "N/A");
	SetWindowText (GetDlgItem (hWnd, IDC_FB_STATUS_APPLY), cbuf);

	if (bafs_status == 0)sprintf(cbuf,"OFF"); ////// boosters
	else sprintf(cbuf,"ENABLED");
	SetWindowText (GetDlgItem (hWnd, IDC_FBB_STATUS), cbuf);
	if (eng_status == 0)sprintf (cbuf, "On/Off");
	else sprintf (cbuf, "N/A");
	SetWindowText (GetDlgItem (hWnd, IDC_FBB_STATUS_APPLY), cbuf);

	strcpy (cbuf, "Enter Name");
	SetWindowText (GetDlgItem (hWnd, IDC_PLD_NM), cbuf);
	if (pld_att_status < 100)SetWindowText (GetDlgItem (hWnd, IDC_PLD_NM_APPLY), "Attach");
	else SetWindowText (GetDlgItem (hWnd, IDC_PLD_NM_APPLY), "N/A");

	//strcpy (cbuf, "NAME");
	if (pld_att_status > 0)
	{
		SetWindowText (GetDlgItem (hWnd, IDC_PLD_JET), pldname[pld_att_status - 1]);
		if (intfair_status == 0)SetWindowText (GetDlgItem (hWnd, IDC_PLD_JET_APPLY), "N/A");
		else SetWindowText (GetDlgItem (hWnd, IDC_PLD_JET_APPLY), "Jettison");
	}
	else 
	{
		SetWindowText (GetDlgItem (hWnd, IDC_PLD_JET), "NULL");
		SetWindowText (GetDlgItem (hWnd, IDC_PLD_JET_APPLY), "N/A");
	}
	//sprintf(oapiDebugString(), "pld_att_status %d", pld_att_status);

	sprintf (cbuf, "Total %0.0fkg", (pld_mass));
	SetWindowText (GetDlgItem (hWnd, IDC_PLD_MASS), cbuf);

	sprintf (cbuf, "%0.4f", (tgt_inc*DEG));
	SetWindowText (GetDlgItem (hWnd, IDC_INC), cbuf);

	sprintf (cbuf, "%0.1f", (op_meco_alt*0.001));
	SetWindowText (GetDlgItem (hWnd, IDC_PER), cbuf);

	sprintf (cbuf, "%0.1f", (op_tgt*0.001));
	SetWindowText (GetDlgItem (hWnd, IDC_APO), cbuf);

	sprintf (cbuf, "%0.1f", (ap_tgt*0.001));
	SetWindowText (GetDlgItem (hWnd, IDC_APO1), cbuf);
	if (eng_status == 0)sprintf (cbuf, "Enter");
	else sprintf (cbuf, "N/A");
	SetWindowText (GetDlgItem (hWnd, IDC_APO1_APPLY), cbuf);

	sprintf (cbuf, "%0.2f", (accel_limit));
	SetWindowText (GetDlgItem (hWnd, IDC_ACCLIM), cbuf);
	sprintf (cbuf, "Enter");
	SetWindowText (GetDlgItem (hWnd, IDC_ACCLIM_APPLY), cbuf);

	sprintf (cbuf, "%0.1f", (op_insertion*0.001));
	SetWindowText (GetDlgItem (hWnd, IDC_XFID), cbuf);
	if (eng_status == 0)sprintf (cbuf, "Enter");
	else sprintf (cbuf, "N/A");
	SetWindowText (GetDlgItem (hWnd, IDC_XFID_APPLY), cbuf);

	

	////////////////////
	if(lot_status == 1)
	{
		if (apop_status == 0)sprintf (cbuf, "ENABLED");
		else sprintf (cbuf, "ACTIVE");
	}
	else if (apop_status == 0)sprintf (cbuf, "OFF");
	else sprintf (cbuf, "ACTIVE");
	SetWindowText (GetDlgItem (hWnd, IDC_APOP), cbuf);

	/////////// launch timer
	sprintf (lotbuf, "%d", (lot_hh));
	if(lot_hh < 10) 
		{
		strcat(lotbuftemp,lotbuf);
		SetWindowText (GetDlgItem (hWnd, IDC_LOT_HH), lotbuftemp);
		strcpy(lotbuftemp, "0");
		}
	else SetWindowText (GetDlgItem (hWnd, IDC_LOT_HH), lotbuf);

	sprintf (lotbuf, "%d", (lot_mm));
	if(lot_mm < 10)
		{
		strcat(lotbuftemp,lotbuf);
		SetWindowText (GetDlgItem (hWnd, IDC_LOT_MM), lotbuftemp);
		strcpy(lotbuftemp, "0");
		}
	else SetWindowText (GetDlgItem (hWnd, IDC_LOT_MM), lotbuf);

	sprintf (lotbuf, "%d", (lot_ss));
	if(lot_ss < 10)
		{
		strcat(lotbuftemp,lotbuf);
		SetWindowText (GetDlgItem (hWnd, IDC_LOT_SS), lotbuftemp);
		strcpy(lotbuftemp, "0");
		}
	else SetWindowText (GetDlgItem (hWnd, IDC_LOT_SS), lotbuf);

	if(lot_status == 0)sprintf (cbuf, "OFF");
	else if(lot_status == 3)sprintf (cbuf, "OFF");
	else sprintf (cbuf, "ON");
	SetWindowText (GetDlgItem (hWnd, IDC_ALOT), cbuf);

	if (intfair_status == 0)sprintf (cbuf, "Jettison");
	else sprintf (cbuf, "N/A");
	SetWindowText (GetDlgItem (hWnd, IDC_FAIR_APPLY), cbuf);

	if (eng_status < 2)sprintf (cbuf, "Jettison");
	else sprintf (cbuf, "N/A");
	SetWindowText (GetDlgItem (hWnd, IDC_JSTG_APPLY), cbuf);

	VESSEL2* v = (VESSEL2*)oapiGetVesselInterface(hMe);
	v->SendBufferedKey(COMM_CALC_FRE);
	sprintf (cbuf, "%0.0f kg", fuel_res_est);
	SetWindowText (GetDlgItem (hWnd, IDC_FRE), cbuf);
	if (eng_status == 0)sprintf (cbuf, "Apply");
	else sprintf (cbuf, "N/A");
	SetWindowText (GetDlgItem (hWnd, IDC_FRE_APPLY), cbuf);

	v->SendBufferedKey(COMM_CALC_FRE); /////////////// boosters
	sprintf (cbuf, "%0.0f kg", booster_fuel_res_est);
	SetWindowText (GetDlgItem (hWnd, IDC_FREB), cbuf);
	if (eng_status == 0)sprintf (cbuf, "Apply");
	else sprintf (cbuf, "N/A");
	SetWindowText (GetDlgItem (hWnd, IDC_FREB_APPLY), cbuf);

	/////// asds pos
	sprintf (cbuf, "Lng. %0.3f   Lat. %0.3f", (asds_core_lng*DEG), (asds_core_lat*DEG));
	SetWindowText (GetDlgItem (hWnd, IDC_ASDS_CORE), cbuf);
	sprintf (cbuf, "Lng. %0.3f   Lat. %0.3f", (asds_bstr_lng*DEG), (asds_bstr_lat*DEG));
	SetWindowText (GetDlgItem (hWnd, IDC_ASDS_BSTR), cbuf);
	sprintf (cbuf, "Lng. %0.3f   Lat. %0.3f", (frv_core_lng*DEG), (frv_core_lat*DEG));
	SetWindowText (GetDlgItem (hWnd, IDC_FRV), cbuf);
	
	OBJHANDLE hSLC = v->GetAttachmentStatus(lpd_attach);
	if (hSLC)SetWindowText (GetDlgItem (hWnd, IDC_HDBK), "ENGAGE/RETRACT");
	else SetWindowText (GetDlgItem (hWnd, IDC_HDBK), "N/A");

	if (edlo_sw_core == 0)sprintf (cbuf, "FLYBACK-EDL");
	else sprintf (cbuf, "EDL ONLY");
	SetWindowText (GetDlgItem (hWnd, IDC_EDLO), cbuf);
	if (eng_status == 0)sprintf (cbuf, "Change");
	else sprintf (cbuf, "N/A");
	SetWindowText (GetDlgItem (hWnd, IDC_EDLO_APPLY), cbuf);

	if (edlo_sw_boosters == 0)sprintf (cbuf, "FLYBACK-EDL"); //////// boosters
	else sprintf (cbuf, "EDL ONLY");
	SetWindowText (GetDlgItem (hWnd, IDC_EDLOB), cbuf);
	if (eng_status == 0)sprintf (cbuf, "Change");
	else sprintf (cbuf, "N/A");
	SetWindowText (GetDlgItem (hWnd, IDC_EDLOB_APPLY), cbuf);
}

BOOL CALLBACK DlgProc (HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	
	switch (uMsg) {
	case WM_INITDIALOG:
		InitDialog(hWnd);
		return TRUE;

	case WM_COMMAND:
		switch (LOWORD(wParam)) 
		{
		char cbuf[256];
		char chh[256];
		char cmm[256];
		char css[256];

		case IDCANCEL:
			oapiCloseDialog (hWnd);
			return TRUE;

		case IDC_FUEL_APPLY:
			if (eng_status == 0)
			{
			GetWindowText (GetDlgItem (hWnd, IDC_FUEL_RES), cbuf, 256);
			SetRF(0,cbuf,0);
			}
			InitDialog(hWnd);
			return TRUE;

		case IDC_FUELB_APPLY: ///////// boosters
			if (eng_status == 0)
			{
			GetWindowText (GetDlgItem (hWnd, IDC_FUELB_RES), cbuf, 256);
			SetRF1(0,cbuf,0);
			}
			InitDialog(hWnd);
			return TRUE;

		case IDC_FRE_APPLY:
			if (eng_status == 0)
			{
			GetWindowText (GetDlgItem (hWnd, IDC_FRE), cbuf, 256);
			SetRF(0,cbuf,0);
			}
			InitDialog(hWnd);
			return TRUE;

		case IDC_FREB_APPLY: ////// boosters
			if (eng_status == 0)
			{
			GetWindowText (GetDlgItem (hWnd, IDC_FREB), cbuf, 256);
			SetRF1(0,cbuf,0);
			}
			InitDialog(hWnd);
			return TRUE;
		
		case IDC_TGTNM_APPLY:
			if (eng_status == 0)
			{
			GetWindowText (GetDlgItem (hWnd, IDC_TGTNM), cbuf, 256);
			SetBBTgt(0,cbuf,0);
			}
			InitDialog(hWnd);
			return TRUE;

		case IDC_TGTNMP_APPLY: /////// booster PORT
			if (eng_status == 0)
			{
			GetWindowText (GetDlgItem (hWnd, IDC_TGTNMP), cbuf, 256);
			SetBBTgtP(0,cbuf,0);
			}
			InitDialog(hWnd);
			return TRUE;

		case IDC_TGTNMS_APPLY: /////// booster STBD
			if (eng_status == 0)
			{
			GetWindowText (GetDlgItem (hWnd, IDC_TGTNMS), cbuf, 256);
			SetBBTgtS(0,cbuf,0);
			}
			InitDialog(hWnd);
			return TRUE;

		case IDC_PLD_NM_APPLY:
			if (pld_att_status < 20)
			{
			GetWindowText (GetDlgItem (hWnd, IDC_PLD_NM), cbuf, 256);
			SetPLD(0,cbuf,0);
			VESSEL2* v = (VESSEL2*)oapiGetVesselInterface(hMe);
			v->SendBufferedKey(COMM_CALC_MASS);
			InitDialog(hWnd);
			}
			return TRUE;
			
		case IDC_PLD_JET_APPLY:
			if (pld_att_status > 0 && intfair_status == 1)
			{
			VESSEL2* v = (VESSEL2*)oapiGetVesselInterface(hMe);
			v->SendBufferedKey(OAPI_KEY_J);
			InitDialog(hWnd);
			}
			return TRUE;

		case IDC_FB_STATUS_APPLY:
			if (eng_status == 0)
			{
			bchk = oapiGetObjectByName(refname);
			if (bchk == NULL)bchk = oapiGetBaseByName(hGref, refname);
			if (bchk == NULL)afs_status = 0;
			else if(afs_status == 0)afs_status = 1;
			else afs_status = 0;
			}
			InitDialog(hWnd);
			return TRUE;

		case IDC_FBB_STATUS_APPLY: ///////// boosters
			if (eng_status == 0)
			{
			if(bafs_status == 0)bafs_status = 1;
			else bafs_status = 0;
			}
			InitDialog(hWnd);
			return TRUE;

			case IDC_INC_APPLY:
			GetWindowText (GetDlgItem (hWnd, IDC_INC), cbuf, 256);
			SetINC(0,cbuf,0);
			InitDialog(hWnd);
			return TRUE;

			case IDC_PER_APPLY:
			GetWindowText (GetDlgItem (hWnd, IDC_PER), cbuf, 256);
			SetOPE(0,cbuf,0);
			InitDialog(hWnd);
			return TRUE;

			case IDC_APO_APPLY:
			GetWindowText (GetDlgItem (hWnd, IDC_APO), cbuf, 256);
			SetOAP(0,cbuf,0);
			InitDialog(hWnd);
			return TRUE;

			case IDC_APO1_APPLY:
			if (eng_status == 0)
			{
			GetWindowText (GetDlgItem (hWnd, IDC_APO1), cbuf, 256);
			SetAP(0,cbuf,0);
			}
			InitDialog(hWnd);
			return TRUE;

			case IDC_XFID_APPLY:
			if (eng_status == 0)
			{
			GetWindowText (GetDlgItem (hWnd, IDC_XFID), cbuf, 256);
			SetRF0(0,cbuf,0);
			}
			InitDialog(hWnd);
			return TRUE;

			case IDC_ACCLIM_APPLY:
			{
			GetWindowText (GetDlgItem (hWnd, IDC_ACCLIM), cbuf, 256);
			float new_rf;
			sscanf(cbuf, "%f", &new_rf);
			if (new_rf > 25) accel_limit = new_rf;	
			}
			InitDialog(hWnd);
			return TRUE;

			case IDC_APOP_APPLY:
			{
			VESSEL2* v = (VESSEL2*)oapiGetVesselInterface(hMe);
			v->SendBufferedKey(OAPI_KEY_V);
			}
			InitDialog(hWnd);
			return TRUE;

			case IDC_LOT_APPLY:
			{
			GetWindowText (GetDlgItem (hWnd, IDC_LOT_HH), chh, 256);
			GetWindowText (GetDlgItem (hWnd, IDC_LOT_MM), cmm, 256);
			GetWindowText (GetDlgItem (hWnd, IDC_LOT_SS), css, 256);
			
			SetLOT(chh,cmm,css);
			}
			InitDialog(hWnd);
			return TRUE;

			case IDC_ALOT_APPLY:
			{
			if(lot_status == 0)lot_status = 1;
			else if(lot_status == 1)lot_status = 0;
			oapiDelAnnotation (hNote);
			}
			InitDialog(hWnd);
			return TRUE;

			case IDC_FAIR_APPLY:
			if(intfair_status == 0)
			{
			VESSEL2* v = (VESSEL2*)oapiGetVesselInterface(hMe);
			v->SendBufferedKey(OAPI_KEY_J);
			}
			InitDialog(hWnd);
			return TRUE;

			case IDC_JSTG_APPLY:
			if(eng_status < 2)
			{
			VESSEL2* v = (VESSEL2*)oapiGetVesselInterface(hMe);
			v->SendBufferedKey(OAPI_KEY_N);
			}
			InitDialog(hWnd);
			return TRUE;

			case IDC_HDBK:
			{
			VESSEL2* v = (VESSEL2*)oapiGetVesselInterface(hMe);
			OBJHANDLE hLpad = v->GetAttachmentStatus(lpd_attach);
			if(hLpad) 
				{
				v = (VESSEL2*)oapiGetVesselInterface(hLpad);
				v->SendBufferedKey(OAPI_KEY_G);
				}
			}
			InitDialog(hWnd);
			return TRUE;

			case IDC_EDLO_APPLY:
			if(eng_status == 0)
			{
			if(edlo_sw_core == 0) edlo_sw_core = 1;
			else edlo_sw_core = 0;
			}
			InitDialog(hWnd);
			return TRUE;

			case IDC_EDLOB_APPLY: ///////// boosters
			if(eng_status == 0)
			{
			if(edlo_sw_boosters == 0) edlo_sw_boosters = 1;
			else edlo_sw_boosters = 0;
			}
			InitDialog(hWnd);
			return TRUE;

			case IDC_REFRESH:
			InitDialog(hWnd);
			return TRUE;

		}
		
		break;
	}
	return oapiDefDialogProc (hWnd, uMsg, wParam, lParam);
}

void f9h_stg2::clbkPreStep(double SimT, double SimDT, double MJD)
{

////////// launch timer
if(lot_status == 1)DoLOT();
else if(lot_status == 2)DoLOT();

////////////////////////// l pad condensation
if (GetAttachmentStatus(lpd_attach) == NULL) pst_lvl2 = 0;
else if (oapiGetTimeAcceleration() > 10) pst_lvl2 = 0;
else pst_lvl2 = 0.7;

step_time = SimDT;///////
double simdt = SimDT;

//////// booster sep
if (eng_status == 0)
			{
			CalcFuel();
			if(no_legs < 2)
				{
				if ((booster_fuel - (booster_fuel_reserve)) < 300) JettisonBoosters();
				}
			else
				{
				if ((booster_fuel - (booster_fuel_reserve)) < 1) JettisonBoosters();
				}
			}
else if (eng_status == 1)
{
	CalcFuel();
	if (ss_proc < 1) ss_proc = ss_proc+(SimDT*SS_SPEED);
	if (ss_proc > 1)ss_proc = 1;
	SetAnimation(anim_ss, ss_proc);
}
/////


double mth_lev = GetThrusterLevel(th_main[eng_status]);
if(eng_status == 2)
{
if (mth_lev == 0 && rcs_sw2 == 0) ConfigRCS2();
else if (mth_lev > 0 && rcs_sw2 == 1) RevertRCS2();

if(mth_lev > 0)he_vent_status = 1;
else if(mth_lev == 0 && he_vent_status == 1)
	{
		he_vent_timer = he_vent_timer + SimDT;
		if (he_vent_timer < 6) he_vent_level = 1;
		else
		{
			he_vent_level = 0;
			he_vent_timer = 0;
			he_vent_status = 0;
		}
	}
}

if(mth_lev > 0)SetThrusterLevel(th_dump, 0);

double dynp = GetDynPressure();
if(dynp > 25000)
{
	pst_lvl = (dynp-25000)/3000;
	if (pst_lvl > 1)pst_lvl = 1;
	if(eng_status > 0)pst_lvl = 0;
}
else pst_lvl = 0;

if (calc_sw == 0)
{
CalcEmptyMass();
calc_sw = 1;
}

if (ap_status == 1) DoAscent();
else if (op_status == 1) DoOrbit();
apop_status = op_status+ap_status;

////////////////engine light
if (mth_lev > 0 && eng_status < 2)
	{
	double rndm = 0.15*(oapiRand());
	rndm = (mth_lev*0.9) - rndm;
	if (rndm < 0)rndm = 0;
	else if (rndm > 1)rndm = 1;
	spt_light->Activate(true);
	spt_light->SetIntensity(rndm);

	}
	else spt_light->Activate(false);
//////////
	if(eng_status == 2)SetMeshMat();
}



void f9h_stg2::clbkPostStep (double simt, double simdt, double mjd)
{
	
}




bool f9h_stg2::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp)
{
	// draw the default HUD
	VESSEL3::clbkDrawHUD (mode, hps, skp);

	int s = hps->H;
	double d = (s*0.00130208);
	int sw = ((hps->W));
	int lw =  (int)(16*sw/1024);
	int lwoffset = sw - (18*lw);
	int hlw = (int) (lw/2);

	//int chw = 16;

	
	char test[256];
	strcpy(test,"abcdefghijklmnopqrstuvwxyz");
	int it = skp->GetTextWidth(test, 26);
	it = (int)(it/26);
	int chw = it;

	int roxl = 0;
	int royl = 0;

	double ds = s;
	double dsw = sw;
	double sc_ratio = ds/dsw;


	if (sc_ratio < 0.7284)
	{
	roxl = (lw*10);
	royl = (int) (-88*d);
	}



	int wd = (int)(152*d);
	int wc = (int)(168*d);
	int w0 = (int)(184*d);
	int w1 = (int) (200*d);
	int w2 = (int)(216*d);
	int w3 = (int)(232*d);
	int w4 = (int)(248*d);
	int w5 = (int)(264*d);
	int w6 = (int)(280*d);
	int w7 = (int)(296*d);
	int w8 = (int)(312*d);
	int w9 = (int)(328*d);
	int w10 = (int)(344*d);
	int w11 = (int)(360*d);
	int w12 = (int)(376*d);
	int w13 = (int)(392*d);
	int w14 = (int)(408*d);
	int w15 = (int)(424*d);


	{
	skp->SetTextColor(0x00FFFFFF);
	char abuf[256];
	sprintf(abuf,"Next Payload: %s", pldname[pld_att_status-1]);
	skp->Text ((10+roxl), (wd+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x00FFFFFF);
	char abuf[256];
	sprintf(abuf,"Payload Total: %dkg", int(pld_mass));
	skp->Text ((10+roxl), (wc+royl), abuf,strlen(abuf));
	}


	if (eng_status == 2)
	{
	skp->SetTextColor(0x00FFFFFF);
	char abuf[256];
	double m1 = GetMass();
	double m2 = GetPropellantMass(hProp2);
	double test = (m1/(m1-m2));
	double isp_curr = GetThrusterIsp(th_main[eng_status]);
	double zdv = (isp_curr*(log(test)));

	sprintf(abuf,"Remaining dV = %0.2f m/s : RCS Prop. = %0.1fkg", zdv, GetPropellantMass(hRCSProp2));
	skp->Text ((10+roxl), (w0+royl), abuf,strlen(abuf));
	}
	else
	{
	skp->SetTextColor(0x00FFFFFF);
	char abuf[256];
	double m1 = GetMass();
	double m2 = 0;
	if (eng_status == 0)m2 = GetPropellantMass(hProp0);
	else m2 = GetPropellantMass(hProp1);
	double test = (m1/(m1-m2));
	double isp_curr = GetThrusterIsp(th_main[eng_status]);
	double zdv = (isp_curr*(log(test)));

	sprintf(abuf,"Remaining dV = %0.2f m/s", zdv);
	skp->Text ((10+roxl), (w0+royl), abuf,strlen(abuf));
	}



	{
	skp->SetTextColor(0x00FFFFFF);
	char abuf[256];
	sprintf(abuf,"Final Orbit Params: Inc.%0.2f ApA.%d PeA.%d",(tgt_inc*DEG),int(op_tgt/1000),int(op_meco_alt/1000));
	skp->Text ((10+roxl), (w1+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x00FFFFFF);
	char abuf[256];
	sprintf(abuf,"Core Apogee Tgt.%d : Orbit Insertion Alt.%d",int(ap_tgt/1000), int(op_insertion/1000));
	skp->Text ((10+roxl), (w2+royl), abuf,strlen(abuf));
	}

	if(eng_status < 2)
	{
	//if(eng_status == 1) core_throttle = GetThrusterLevel(th_main[1]);
	skp->SetTextColor(0x00FFFFFF);
	char abuf[256];
	sprintf(abuf,"Core Fuel    %dkg  Reserve %dkg  Throttle %d%%",int(core_fuel),int(fuel_reserve), int(core_throttle*100));
	skp->Text ((10+roxl), (w3+royl), abuf,strlen(abuf));
	}
	if(eng_status == 0)
	{
	skp->SetTextColor(0x00FFFFFF);
	char abuf[256];
	sprintf(abuf,"Booster Fuel %dkg  Reserve %dkg  Throttle %d%%",int(booster_fuel), int(booster_fuel_reserve), int(bstr_throttle*100));
	skp->Text ((10+roxl), (w4+royl), abuf,strlen(abuf));
	}
	

	{
	char edlobuf[256];
	if(afs_status == 0)strcpy(edlobuf, "OFF");
	else if (edlo_sw_core == 0)strcpy(edlobuf, "Flyback+EDL");
	else strcpy(edlobuf, "EDL Only");
	skp->SetTextColor(0x00FFFFFF);
	char abuf[256];
	sprintf(abuf,"CORE -> %s : %s", refname, edlobuf);
	skp->Text ((10+roxl), (w5+royl), abuf,strlen(abuf));
	}

	{
	char edlobuf[256];
	if(bafs_status == 0)strcpy(edlobuf, "OFF");
	else if (edlo_sw_boosters == 0)strcpy(edlobuf, "Flyback+EDL");
	else strcpy(edlobuf, "EDL Only");
	skp->SetTextColor(0x00FFFFFF);
	char abuf[256];
	sprintf(abuf,"PORT -> %s : %s",refnamep, edlobuf);
	skp->Text ((10+roxl), (w6+royl), abuf,strlen(abuf));
	}

	{
	char edlobuf[256];
	if(bafs_status == 0)strcpy(edlobuf, "OFF");
	else if (edlo_sw_boosters == 0)strcpy(edlobuf, "Flyback+EDL");
	else strcpy(edlobuf, "EDL Only");
	skp->SetTextColor(0x00FFFFFF);
	char abuf[256];
	sprintf(abuf,"STBD -> %s : %s",refnames, edlobuf);
	skp->Text ((10+roxl), (w7+royl), abuf,strlen(abuf));
	}

	


	/*if (afs_status == 0)
	{
	skp->SetTextColor(0x00FFFFFF);
	char abuf[256];
	sprintf(abuf,"B = Enable 1st Stage Auto Flyback-EDL");
	skp->Text ((10+roxl), (w8+royl), abuf,strlen(abuf));
	}
	else
	{
	skp->SetTextColor(0x006699FF);
	char abuf[256];
	sprintf(abuf,"B = 1st Stage Auto Flyback-EDL Enabled");
	skp->Text ((10+roxl), (w8+royl), abuf,strlen(abuf));
	}*/

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"K = Open/Close Launch Control Panel");
	skp->Text ((10+roxl), (w9+royl), abuf,strlen(abuf));
	}

	if (ap_status == 0 && op_status == 0)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"V = Start Launch Autopilot");
	skp->Text ((10+roxl), (w10+royl), abuf,strlen(abuf));
	}
	else
	{
	skp->SetTextColor(0x0066FFFF);
	char abuf[256];
	sprintf(abuf,"V = Launch Autopilot Active");
	skp->Text ((10+roxl), (w10+royl), abuf,strlen(abuf));
	}


	if (eng_status == 0)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"N = Jettison Boosters");
	skp->Text ((10+roxl), (w11+royl), abuf,strlen(abuf));
	}
	else if (eng_status == 1)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"N = Jettison 1st Stage");
	skp->Text ((10+roxl), (w11+royl), abuf,strlen(abuf));
	}


	if (intfair_status == 0)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"J = Jettison Fairing");
	skp->Text ((10+roxl), (w12+royl), abuf,strlen(abuf));
	}
	else if (pld_att_status > 0)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"J = Jettison Payload");
	skp->Text ((10+roxl), (w12+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"E = Set View +Z/-Z");
	skp->Text ((10+roxl), (w13+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"B = 2nd Stage Fuel Dump");
	skp->Text ((10+roxl), (w14+royl), abuf,strlen(abuf));
	}

	if (eng_status == 2)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"M = Fwd.Lamp");
	skp->Text ((10+roxl), (w15+royl), abuf,strlen(abuf));
	}

	return true;
}

void f9h_stg2::RevertRCS2 (void)
{
	SetThrusterGroupLevel(htgrppup2,0);
	SetThrusterGroupLevel(htgrppdown2,0);
	SetThrusterGroupLevel(htgrpyright2,0);
	SetThrusterGroupLevel(htgrpyleft2,0);

	DelThrusterGroup(htgrppup2,THGROUP_ATT_PITCHUP, false);
	DelThrusterGroup(htgrppdown2,THGROUP_ATT_PITCHDOWN, false);
	DelThrusterGroup(htgrpyright2,THGROUP_ATT_YAWRIGHT, false);
	DelThrusterGroup(htgrpyleft2,THGROUP_ATT_YAWLEFT, false);

	th_group[0] = th_rcs[14];
	htgrppup2 = CreateThrusterGroup (th_group, 1, THGROUP_ATT_PITCHUP);

	th_group[0] = th_rcs[15];
	htgrppdown2 = CreateThrusterGroup (th_group, 1, THGROUP_ATT_PITCHDOWN);

	th_group[0] = th_rcs[13];
	htgrpyright2 = CreateThrusterGroup (th_group, 1, THGROUP_ATT_YAWRIGHT);

	th_group[0] = th_rcs[12];
	htgrpyleft2 = CreateThrusterGroup (th_group, 1, THGROUP_ATT_YAWLEFT);

	rcs_sw2 = 0;
	vrot_lim = 2*RAD;
}

void f9h_stg2::ConfigRCS2 (void)
{
	SetThrusterGroupLevel(htgrppup2,0);
	SetThrusterGroupLevel(htgrppdown2,0);
	SetThrusterGroupLevel(htgrpyright2,0);
	SetThrusterGroupLevel(htgrpyleft2,0);

	DelThrusterGroup(htgrppup2,THGROUP_ATT_PITCHUP, false);
	DelThrusterGroup(htgrppdown2,THGROUP_ATT_PITCHDOWN, false);
	DelThrusterGroup(htgrpyright2,THGROUP_ATT_YAWRIGHT, false);
	DelThrusterGroup(htgrpyleft2,THGROUP_ATT_YAWLEFT, false);

	th_group[0] = th_rcs[16];
	th_group[1] = th_rcs[17];
	htgrppup2 = CreateThrusterGroup (th_group, 2, THGROUP_ATT_PITCHUP);

	th_group[0] = th_rcs[18];
	th_group[1] = th_rcs[19];
	htgrppdown2 = CreateThrusterGroup (th_group, 2, THGROUP_ATT_PITCHDOWN);

	th_group[0] = th_rcs[20];
	th_group[1] = th_rcs[22];
	htgrpyright2 = CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWRIGHT);

	th_group[0] = th_rcs[21];
	th_group[1] = th_rcs[23];
	htgrpyleft2 = CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWLEFT);

	rcs_sw2 = 1;
	vrot_lim = 2*RAD;
}

void f9h_stg2::JettisonFair (void)
{
if(time_acc_limit == true)
	{
	if(oapiGetTimeAcceleration() > 1)oapiSetTimeAcceleration(1);
	}
//////////////////////////////

VESSELSTATUS vs;
 GetStatus(vs); // get mother vessel status vs
  VECTOR3 rvel;
  if(large_fair == 0)
  {
  Local2Rel (STG1_FAIR1_OFF, vs.rpos);
  if (eng_status == 2)Local2Rel (STG2_FAIR1_OFF, vs.rpos);
  }
  else
  {
  Local2Rel (STG1_FAIR1_LARGE_OFF, vs.rpos);
  if (eng_status == 2)Local2Rel (STG2_FAIR1_LARGE_OFF, vs.rpos);
  }
  GlobalRot (_V(0,-4,0), rvel);  // child vessel velocity relative to mother, convert to global frame
  vs.rvel = vs.rvel + rvel;  // child vessel velocity absolute in global frame
  vs.eng_main = vs.eng_hovr = 0.0; // engines off
 vs.status = 0; // in flight
  vs.vrot.x = vs.vrot.x - 0.2;	// child vessel rotation relative to mother
  vs.vrot.z = vs.vrot.z;	// child vessel rotation relative to mother
  vs.vrot.y = vs.vrot.y; // child vessel rotation relative to mother
 /* char myname[256];
  oapiGetObjectName(hMe,myname,30);
  char fairname[256];
  strcpy(fairname, myname);
  strcat(fairname, "_Fair1");*/
  if (large_fair == 0) 
  {
	 if (recoverable_fair == 0) oapiCreateVessel ("Fair1", "FalconHeavy/f9h_fair", vs);
	 else oapiCreateVessel ("Fair1", "FalconHeavy/f9h_fair_1", vs);
  }
  else oapiCreateVessel ("Fair1", "FalconHeavy/f9h_fair_large", vs);

  GetStatus(vs);
  if(large_fair == 0)
  {
  Local2Rel (STG1_FAIR2_OFF, vs.rpos);
  if (eng_status == 2)Local2Rel (STG2_FAIR2_OFF, vs.rpos);
  }
  else
  {
  Local2Rel (STG1_FAIR2_LARGE_OFF, vs.rpos);
  if (eng_status == 2)Local2Rel (STG2_FAIR2_LARGE_OFF, vs.rpos);
  }
  GlobalRot (_V(0,4,0), rvel);
  vs.rvel = vs.rvel + rvel;
  vs.eng_main = vs.eng_hovr = 0.0;
  vs.status = 0;
  vs.vrot.x = -(vs.vrot.x + 0.2);
  vs.vrot.z = vs.vrot.z;
  vs.vrot.y = -vs.vrot.y;

	VECTOR3 xright;
	VECTOR3 yup;
	VECTOR3 zforward;

	GlobalRot(_V(-1,0,0), xright);
	GlobalRot(_V(0,-1,0), yup);
	GlobalRot(_V(0,0, 1), zforward);

	double gamma = atan2( xright.x, yup.x);
	double beta = -1*(asin(zforward.x));
	double alpha = atan2(zforward.y, zforward.z);
	
	vs.arot = _V(alpha, beta, (PI/2)-gamma);

 /* strcpy(fairname, myname);
  strcat(fairname, "_Fair2");*/

  if (large_fair == 0) 
  {
	 if (recoverable_fair == 0) oapiCreateVessel ("Fair2", "FalconHeavy/f9h_fair", vs);
	 else oapiCreateVessel ("Fair2", "FalconHeavy/f9h_fair_2", vs);
  }
  else oapiCreateVessel ("Fair2", "FalconHeavy/f9h_fair_large", vs);

  /*if (eng_status == 1)
  {
  DelMesh(4);
  DelMesh(3);
  DelMesh(2);
  DelMesh(1);
  }
  else
  {*/
  DelMesh(4);
  DelMesh(3);
  //}

intfair_status = 1;
CalcEmptyMass();

PlayVesselWave(MyID, SEP, 0, 200, 30000);
PlayVesselWave(MyID, SEPFR, 0, 225, 0);
}

void f9h_stg2::JettisonBoosters (void)
{
	if(time_acc_limit == true)
	{
	if(oapiGetTimeAcceleration() > 1)oapiSetTimeAcceleration(1);
	}

	char cbuf[256];
	char dbuf[256];
	oapiGetObjectName(hMe, cbuf, 30);
	strcpy(dbuf,cbuf);

  VESSELSTATUS vs;
  //char name[256];
  GetStatus(vs);
  VECTOR3 rvel;
  Local2Rel (STG0_PORT_OFF, vs.rpos);
  GlobalRot (_V(-8,0,0), rvel);
  vs.rvel = vs.rvel + rvel;
  vs.eng_main = vs.eng_hovr = 0.0;
  vs.status = 0;
  vs.vrot.x = vs.vrot.x;
  vs.vrot.z = vs.vrot.z;
  vs.vrot.y = vs.vrot.y+0.3;
  strcat(cbuf,"_Port");
  OBJHANDLE hStg1;
  if(no_legs < 2) hStg1 = oapiCreateVessel (cbuf, "FalconHeavy/f9h_stg1_port", vs);
  else hStg1 = oapiCreateVessel (cbuf, "FalconHeavy/f9h_stg1_port_expendable", vs);

  double fuel_trans = booster_fuel;
  if(fuel_trans < 0)fuel_trans = 0;
  VESSEL2* v =(VESSEL2*)oapiGetVesselInterface(hStg1);
  //VESSEL *v = oapiGetVesselInterface (hStg1);
  DWORD nAttach = v->GetPropellantCount ();
  for (DWORD j = 0; j < nAttach; j++)
	{
	PROPELLANT_HANDLE ph = v->GetPropellantHandleByIndex (j);
	if(j == 0)v->SetPropellantMass(ph,fuel_trans);
	else
		{
		double maxfuel = v->GetPropellantMaxMass(ph);
		v->SetPropellantMass(ph,maxfuel);
		}
	}
  if(no_legs < 2)
  {
  v->clbkConsumeBufferedKey(COMM_SEND_STR,false,refnamep);
  if(bafs_status == 1)v->clbkConsumeBufferedKey(COMM_SEND_AFS,false,refnamep);
  if(edlo_sw_boosters == 1)v->clbkConsumeBufferedKey(COMM_SEND_EDLO,false,refnamep);
  }
  

  //////////////// starboard

  GetStatus(vs);
  Local2Rel (STG0_STBD_OFF, vs.rpos);
  GlobalRot (_V(8,0,0), rvel);
  vs.rvel = vs.rvel + rvel;
  vs.eng_main = vs.eng_hovr = 0.0;
  vs.status = 0;
  vs.vrot.x = vs.vrot.x;
  vs.vrot.z = vs.vrot.z;
  vs.vrot.y = vs.vrot.y-0.3;
  strcat(dbuf,"_Stbd");
  if(no_legs < 2) hStg1 = oapiCreateVessel (dbuf, "FalconHeavy/f9h_stg1_starboard", vs);
  else hStg1 = oapiCreateVessel (dbuf, "FalconHeavy/f9h_stg1_starboard_expendable", vs);
  fuel_trans = booster_fuel;
  if(fuel_trans < 0)fuel_trans = 0;
  v =(VESSEL2*)oapiGetVesselInterface(hStg1);
  //VESSEL *v = oapiGetVesselInterface (hStg1);
  nAttach = v->GetPropellantCount ();
  for (DWORD j = 0; j < nAttach; j++)
	{
	PROPELLANT_HANDLE ph = v->GetPropellantHandleByIndex (j);
	if(j == 0)v->SetPropellantMass(ph,fuel_trans);
	else
		{
		double maxfuel = v->GetPropellantMaxMass(ph);
		v->SetPropellantMass(ph,maxfuel);
		}
	}
  if(no_legs < 2)
  {
  v->clbkConsumeBufferedKey(COMM_SEND_STR,false,refnames);
  if(bafs_status == 1)v->clbkConsumeBufferedKey(COMM_SEND_AFS,false,refnames);
  if(edlo_sw_boosters == 1)v->clbkConsumeBufferedKey(COMM_SEND_EDLO,false,refnames);
  }
 

 // VESSELSTATUS2 vss;
 // memset (&vss, 0, sizeof(vss));
 // vss.version = 2;
 // GetStatusEx(&vss);
//  Local2Rel (STG1STG2_OFF, vss.rpos);
 // DefSetStateEx (&vss);

	DelMesh(6);
	DelMesh(5);
	eng_status = 1;

	

	CalcEmptyMass();
	ConfigMain0();

	SetPropellantMass(hProp1, core_fuel);
	SetPropellantMass(hProp0, 0);
	
	PlayVesselWave(MyID, SEP, 0, 200, 25000);
	PlayVesselWave(MyID, SEPST, 0, 225, 0);
}


void f9h_stg2::JettisonStg (void)
{
	if(time_acc_limit == true)
	{
	if(oapiGetTimeAcceleration() > 1)oapiSetTimeAcceleration(1);
	}
	/////////////////////

  char cbuf[256];
  oapiGetObjectName(hMe, cbuf, 30);

  VESSELSTATUS vs;
  //char name[256];
  GetStatus(vs);
  VECTOR3 rvel;
  Local2Rel (STG1_OFF, vs.rpos);
  GlobalRot (_V(0,0,-5), rvel);
  vs.rvel = vs.rvel + rvel;
  vs.eng_main = vs.eng_hovr = 0.0;
  vs.status = 0;
  vs.vrot.x = vs.vrot.x;
  vs.vrot.z = vs.vrot.z;
  vs.vrot.y = vs.vrot.y;
  strcat(cbuf,"_Core");
  OBJHANDLE hStg1;
  if(no_legs == 0) hStg1 = oapiCreateVessel (cbuf, "FalconHeavy/f9h_stg1_centre", vs);
  else hStg1 = oapiCreateVessel (cbuf, "FalconHeavy/f9h_stg1_centre_expendable", vs);
  double fuel_trans = GetPropellantMass(hProp1);
  VESSEL2* v =(VESSEL2*)oapiGetVesselInterface(hStg1);
  //VESSEL *v = oapiGetVesselInterface (hStg1);
  DWORD nAttach = v->GetPropellantCount ();
  for (DWORD j = 0; j < nAttach; j++)
	{
	PROPELLANT_HANDLE ph = v->GetPropellantHandleByIndex (j);
	if(j == 0)v->SetPropellantMass(ph,fuel_trans);
	else
		{
		double maxfuel = v->GetPropellantMaxMass(ph);
		v->SetPropellantMass(ph,maxfuel);
		}
	}
  v->clbkConsumeBufferedKey(COMM_SEND_STR,false,refname);
  if(afs_status == 1)v->clbkConsumeBufferedKey(COMM_SEND_AFS,false,refname);
  if(edlo_sw_core == 1)v->clbkConsumeBufferedKey(COMM_SEND_EDLO,false,refname);
  VESSELSTATUS2 vss;
  memset (&vss, 0, sizeof(vss));
  vss.version = 2;
  GetStatusEx(&vss);
  Local2Rel (STG1STG2_OFF, vss.rpos);
  DefSetStateEx (&vss);

	DelMesh(2);
	DelMesh(1);
	eng_status = 2;


	CalcEmptyMass();
	ConfigMain();

	PlayVesselWave(MyID, SEP, 0, 200, 25000);
	PlayVesselWave(MyID, SEPST, 0, 225, 0);

}

void f9h_stg2::ConfigMain0 (void)
{
	DelThrusterGroup(htgrpmain,THGROUP_MAIN, true);
	DelThrusterGroup(htgrpbleft2,THGROUP_ATT_BANKLEFT, true);
	DelThrusterGroup(htgrpbright2,THGROUP_ATT_BANKRIGHT, true);
	DelThrusterGroup(htgrppup2,THGROUP_ATT_PITCHUP, true);
	DelThrusterGroup(htgrppdown2,THGROUP_ATT_PITCHDOWN, true);
	DelThrusterGroup(htgrpyleft2,THGROUP_ATT_YAWLEFT, true);
	DelThrusterGroup(htgrpyright2,THGROUP_ATT_YAWRIGHT, true);


	th_group[0] = th_rcs[9];
	th_group[1] = th_rcs[11];
	htgrppup2 = CreateThrusterGroup (th_group, 2, THGROUP_ATT_PITCHUP);

	th_group[0] = th_rcs[8];
	th_group[1] = th_rcs[10];
	htgrppdown2 = CreateThrusterGroup (th_group, 2, THGROUP_ATT_PITCHDOWN);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[3];
	htgrpyright2 = CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWRIGHT);

	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[2];
	htgrpyleft2 = CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWLEFT);

	th_group[0] = th_rcs[4];
	th_group[1] = th_rcs[7];
	htgrpbright2 = CreateThrusterGroup (th_group, 2, THGROUP_ATT_BANKRIGHT);

	th_group[0] = th_rcs[5];
	th_group[1] = th_rcs[6];
	htgrpbleft2 = CreateThrusterGroup (th_group, 2, THGROUP_ATT_BANKLEFT);

	th_group[0] = th_main[1];
	htgrpmain = CreateThrusterGroup (th_group, 1, THGROUP_MAIN);

	SetPMI(_V(129,129,1.9));
	SetCrossSections (_V(159,159,16));
	ClearAirfoilDefinitions ();
	CreateAirfoil (LIFT_VERTICAL, _V(0,0,0), VLiftCoeff, 25, 100, 0.16);
	CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,0), HLiftCoeff, 25, 100, 0.16);

	SetDefaultPropellantResource(hProp1);
}

void f9h_stg2::ConfigMain (void)
{
	DelThrusterGroup(htgrpmain,THGROUP_MAIN, true);
	DelThrusterGroup(htgrpbleft2,THGROUP_ATT_BANKLEFT, true);
	DelThrusterGroup(htgrpbright2,THGROUP_ATT_BANKRIGHT, true);
	DelThrusterGroup(htgrppup2,THGROUP_ATT_PITCHUP, true);
	DelThrusterGroup(htgrppdown2,THGROUP_ATT_PITCHDOWN, true);
	DelThrusterGroup(htgrpyleft2,THGROUP_ATT_YAWLEFT, true);
	DelThrusterGroup(htgrpyright2,THGROUP_ATT_YAWRIGHT, true);

	//DelPropellantResource(hProp1);
	SetPropellantMass(hProp1, 0);
	SetDefaultPropellantResource(hProp2);

	th_group[0] = th_rcs[14];
	htgrppup2 = CreateThrusterGroup (th_group, 1, THGROUP_ATT_PITCHUP);

	th_group[0] = th_rcs[15];
	htgrppdown2 = CreateThrusterGroup (th_group, 1, THGROUP_ATT_PITCHDOWN);

	th_group[0] = th_rcs[13];
	htgrpyright2 = CreateThrusterGroup (th_group, 1, THGROUP_ATT_YAWRIGHT);

	th_group[0] = th_rcs[12];
	htgrpyleft2 = CreateThrusterGroup (th_group, 1, THGROUP_ATT_YAWLEFT);

	th_group[0] = th_rcs[16];
	th_group[1] = th_rcs[19];
	htgrpbright2 = CreateThrusterGroup (th_group, 2, THGROUP_ATT_BANKRIGHT);

	th_group[0] = th_rcs[17];
	th_group[1] = th_rcs[18];
	htgrpbleft2 = CreateThrusterGroup (th_group, 2, THGROUP_ATT_BANKLEFT);
	
	th_group[0] = th_main[2];
	htgrpmain = CreateThrusterGroup (th_group, 1, THGROUP_MAIN);


	for (int k = 0; k < 100; k++)
	{
	VECTOR3 at_dir, at_rot, at_pos;
	GetAttachmentParams(pld_attach[k],at_pos,at_dir,at_rot);
	at_pos.z = at_pos.z - 19.596;
	SetAttachmentParams(pld_attach[k],at_pos,at_dir,at_rot);
	}

	for (UINT i = 0; i < GetMeshCount(); i++)
	{
	ShiftMesh(i,-STG1STG2_OFF);
	}

	
	

	SetSize (10);
	SetPMI(_V(18,18,1.9));
	SetCrossSections (_V(44,44,10));
	SetTouchdownPoints ( _V(0,-5, -10), _V(-5, 3, -10), _V( 5, 3,-10));
	ClearAirfoilDefinitions ();
	CreateAirfoil (LIFT_VERTICAL, _V(0,0,0), VLiftCoeff, 5, 20, 0.16);
	CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,0), HLiftCoeff, 5, 20, 0.16);

	beac.pos =  &new_beac_pos;
	spotlight1->SetPosition(new_beac_pos);
}

void f9h_stg2::KillAllRCS2()
{
SetThrusterGroupLevel(THGROUP_ATT_PITCHUP, 0);
SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN, 0);
SetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT, 0);
SetThrusterGroupLevel(THGROUP_ATT_YAWLEFT, 0);
SetThrusterGroupLevel(THGROUP_ATT_BANKRIGHT, 0);
SetThrusterGroupLevel(THGROUP_ATT_BANKLEFT, 0);
}

void f9h_stg2::CalcEmptyMass(void)
{
	double totalmass = STG2DRYMASS;
	
	if (eng_status == 0) totalmass = totalmass + STG1DRYMASS + STG0DRYMASS;
	else if (eng_status == 1) totalmass = totalmass +  STG1DRYMASS;
	if (intfair_status == 0) totalmass = totalmass +  FAIR_MASS;

	pld_mass = 0;
	for (int i = 0; i < 100; i++)
	{
	OBJHANDLE hpld = GetAttachmentStatus(pld_attach[i]);
	if (hpld == NULL) continue;
	else
		{
		double new_pld_mass = oapiGetMass(hpld);
		totalmass = totalmass + new_pld_mass;
		pld_mass = pld_mass + new_pld_mass;
		oapiGetObjectName(hpld, pldname[i], 30);
		}
	
	}
	SetEmptyMass(totalmass);

}


void f9h_stg2::DoAscent(void)
{
if (eng_status == 2)return;

///////// check if attached to pad
OBJHANDLE hLpad;
VESSEL2* v;
if (tu_sw == 0)
{
hLpad = GetAttachmentStatus(lpd_attach);
if (!hLpad) tu_sw = 1;
else
	{
	tu_sw = 2;
	v = (VESSEL2*)oapiGetVesselInterface(hLpad);
	}
}




//////// throttle up down
double englev = GetThrusterGroupLevel(THGROUP_MAIN);

//double pl_fct = (pld_mass*0.000015625)*0.1; // payload compensation
//double ap_fct = (ap_tgt*0.0000001)-0.0125; // apogee compensation
//double pmq_fact = pmq_throttle * 0.01; // user throttle control

double englim = 1; //0.70
double core_throttle_lim = 1;
double bstr_throttle_lim = 1;

//if(ap_function < 2) englev = 1;
//else if(ap_function == 2)englim = 0.8;
//else if (ap_function == 3)englim = 0.8 + pmq_fact*(-0.07 + pl_fct + ap_fct);// 0.730 + pl_fct;// + ap_fct;
//else if (ap_function == 4)
//{
//	if(booster_fuel < 100000)englim = 0.8 + pmq_fact*(-0.20 + pl_fct + ap_fct);//0.600;
//	else englim = 0.8 + pmq_fact*(-0.07 + pl_fct + ap_fct);//englim = 0.730;
//}

if (timer == 0) PlayVesselWave(MyID, CTDN, 0, 200, 0);

if (tu_sw == 1)
{
	if(timer > 10)
	{
	core_throttle = 1;
	bstr_throttle = 1;
	tu_sw = 3;
	}
}
else if (tu_sw == 2)
{
	if(timer > 8)
	{
	core_throttle = 0.5*(timer-8);
	bstr_throttle = 0.5*(timer-8);
	if (core_throttle > 1)
		{
		core_throttle = 1;
		bstr_throttle = 1;
		tu_sw = 3;
		}
	}
	else if (timer == 0)v->SendBufferedKey(OAPI_KEY_M);
}
else if (tu_sw == 3)
{
if (timer > 40) core_throttle_lim = 0.4;
if (timer > 47) bstr_throttle_lim = 0.9;
if (timer > 73) bstr_throttle_lim = 1.0;
if (timer > 90) core_throttle_lim = 0.3;
if (booster_fuel < (booster_fuel_reserve+42000)) bstr_throttle_lim = 0.6; // 130
if (booster_fuel < (booster_fuel_reserve+24000)) bstr_throttle_lim = 0.4; // 140
if (booster_fuel < (booster_fuel_reserve+1000)) core_throttle_lim = 1.0; // 


if (eng_status == 1)
{
double th_max = GetThrusterMax(th_main[eng_status]);
double mass = GetMass();
double maxacc = th_max/mass;
double englim_max = accel_limit/maxacc;
core_throttle_lim = 1;
if(core_throttle_lim > englim_max)core_throttle_lim = englim_max;
}




{
	if (core_throttle > core_throttle_lim)
	{	
	core_throttle = core_throttle-(step_time*0.1);
	if (core_throttle < core_throttle_lim)core_throttle = core_throttle_lim;
	}
	else if (core_throttle < core_throttle_lim)
	{	
	core_throttle = core_throttle+(step_time*0.1);
	if (core_throttle > core_throttle_lim)core_throttle = core_throttle_lim;
	}
}

{
	if (bstr_throttle > bstr_throttle_lim)
	{	
	bstr_throttle = bstr_throttle-(step_time*0.1);
	if (bstr_throttle < bstr_throttle_lim)bstr_throttle = bstr_throttle_lim;
	}
	else if (bstr_throttle < bstr_throttle_lim)
	{	
	bstr_throttle = bstr_throttle+(step_time*0.1);
	if (bstr_throttle > bstr_throttle_lim)bstr_throttle = bstr_throttle_lim;
	}
}
}

englim = (core_throttle+bstr_throttle+bstr_throttle)/3;
if (eng_status == 1) englim = core_throttle;
englev = englim;

timer = timer + step_time;
///////////////////////////////////////////////////

//{
//	if (englev > englim)
//	{	
//	englev = englev-(step_time*0.1);
//	if (englev < englim)englev = englim;
//	}
//	else if (englev < englim)
//	{	
//	englev = englev+(step_time*0.1);
//	if (englev > englim)englev = englim;
//	}
//}


//sprintf(oapiDebugString(), "core_throttle %0.4f bstr_throttle %0.4f timer %0.4f", core_throttle, bstr_throttle,timer);
////////////////////////////


//if(ap_function == 0)
//{
//
//}
SetThrusterGroupLevel(THGROUP_MAIN,englev);

CalcAzimuth();
CalcFlightVector();


if (GetAltitude(ALTMODE_GROUND) > 60 && ap_function == 0) 
{
	ap_function = 1;
	//timer = 0;
}
if (timer > 20) RollToAz();
if (ap_function == 1) GoToFlightVector();
if (ap_function == 2) GoToFlightVector();
if (ap_function == 3) GoToFlightVector();
if (ap_function == 4) GoToFlightVector();
if (ap_function == 5)
{
	KillAllRCS2();
	SetThrusterGroupLevel(THGROUP_MAIN,0);
	ActivateNavmode(NAVMODE_KILLROT);
	ap_function = 0;
	ap_status = 0;
	JettisonStg();
	op_status = 1;
	timer = 0;
}



}

double f9h_stg2::CalcAzimuth()
{
	VESSEL2* v;
	OBJHANDLE hPad = GetAttachmentStatus(lpd_attach);
	if(hPad != NULL) v = (VESSEL2*)oapiGetVesselInterface(hPad);
	else v = (VESSEL2*)oapiGetVesselInterface(GetHandle());

	double lon,lat,rad;
	v->GetEquPos(lon,lat,rad);
	/*if(abs(tgt_inc)<abs(lat)){
		tgt_inc=abs(lat);
	}else{
		tgt_inc=tgt_inc;
	}*/
	if (abs(tgt_inc) < abs(lat))
	{
	if (tgt_inc < 0) tgt_inc = -(abs(lat));
	else tgt_inc = abs(lat);
	}
	/////////////
	OBJHANDLE hearth=v->GetSurfaceRef();
	VECTOR3 relvel = _V(0,0,0);
	VECTOR3 globpos = _V(0,0,0);
	v->GetRelativeVel(hearth,relvel);
	v->GetGlobalPos(globpos);
	VECTOR3 relvel_glob = relvel+globpos;
	VECTOR3 rel_vel_loc = _V(0,0,0);
	v->Global2Local(relvel_glob,rel_vel_loc);

	VECTOR3 ov_hrz = _V(0,0,0);
	v->HorizonRot(rel_vel_loc, ov_hrz);
	double orb_ground_spd = sqrt((ov_hrz.x*ov_hrz.x)+(ov_hrz.z*ov_hrz.z));
	VECTOR3 orb_ground_vector = _V(ov_hrz.x,0,ov_hrz.z);

	VECTOR3 actual_gspd;
	GetGroundspeedVector(FRAME_HORIZON, actual_gspd);

	////////
	double tgt_inc_sc = tgt_inc;
	if (v->GetAltitude(ALTMODE_GROUND) > 20000) // bloody winds at launch!!
	{	
		if(lat > 0 && tgt_inc > 0)
		{
			if (actual_gspd.z < 0) tgt_inc_sc = -tgt_inc;
		}
		else if(lat < 0 && tgt_inc < 0)
		{
			if (actual_gspd.z > 0) tgt_inc_sc = -tgt_inc;
		}
	}
	double final_azimuth = asin(cos(tgt_inc_sc)/cos(lat));
	final_azimuth+=(90*RAD-final_azimuth)*(1-(tgt_inc_sc/abs(tgt_inc_sc)));
	VECTOR3 final_ground_vector = _V((FinalV()*sin(final_azimuth)),0,(FinalV()*cos(final_azimuth)));

	VECTOR3 req_ground_vector = final_ground_vector - orb_ground_vector;

	azimuth = atan2(req_ground_vector.x,req_ground_vector.z);

	/////////////
	//azimuth=atan2(FinalV()*(sin(asin(cos(tgt_inc)/cos(lat))))-GetOS(),FinalV()*(cos(asin(cos(tgt_inc)/cos(lat)))));
	//azimuth+=(90*RAD-azimuth)*(1-(tgt_inc/abs(tgt_inc)));
	
	vazim1=_V(sin(azimuth),0,cos(azimuth));
	
	v->HorizonInvRot(vazim1,azim_local);
	//sprintf(oapiDebugString(), "ap_function %d", ap_function);
	return azimuth;
}

double f9h_stg2::GetOS(){
	OBJHANDLE hearth=GetSurfaceRef();
	VECTOR3 relvel;
	GetRelativeVel(hearth,relvel);
	double os=length(relvel);
	return os;
}

//double f9h_stg2::FinalV()
//{
////double tgtapo = 180000;
////double tgtperi = -6318000;
//double tgtapo = 200000;
//double tgtperi = 2000000;
//double abside=tgtapo;
//	if(tgtapo<=260000){
//			abside=tgtapo;
//		}else{
//			abside=tgtperi;}
//if (op_status == 1) return sqrt(mu*(2/(op_meco_alt+rt)-1/(0.5*(op_tgt+op_meco_alt+(2*rt)))));
//else return sqrt(mu*(2/(abside+rt)-1/(0.5*(tgtapo+tgtperi+2*rt))));
//
//}

double f9h_stg2::FinalV()
{
//double tgtapo = 180000;
//double tgtperi = -6318000;
double tgtapo = 200000;
double tgtperi = 2000000;
double abside=tgtapo;
	if(tgtapo<=260000){
			abside=tgtapo;
		}else{
			abside=tgtperi;}

if(op_meco_alt == op_tgt)return sqrt(mu*(2/(op_meco_alt+rt)-1/(0.5*(op_tgt+op_meco_alt+(2*rt)))));
else if(op_meco_alt == op_insertion)return sqrt(mu*(2/(op_meco_alt+rt)-1/(0.5*(op_tgt+op_meco_alt+(2*rt)))));
else if (op_status == 1) //return sqrt(mu*(2/(op_meco_alt+rt)-1/(0.5*(op_tgt+op_meco_alt+(2*rt)))));
{
	double a = 0.5*(op_tgt+op_meco_alt+(2*rt)); // semi major axis
	double ecc = (a - (op_meco_alt+rt))/a; // eccentricity
	double ecc_anom = acos((1-((op_insertion+rt)/a))/ecc); // eccentric anomaly at insertion
	double tempy = (sqrt(1 + ecc))*(sin(ecc_anom*0.5));
	double tempx = (sqrt(1 - ecc))*(cos(ecc_anom*0.5));
	double true_anom = 2*(atan2(tempy,tempx));

	VECTOR3 pos, vel;
	pos.x = (op_insertion+rt)*(cos(true_anom));
	pos.y = (op_insertion+rt)*(sin(true_anom));
	pos.z = 0;

	vel.x = ((sqrt(mu*a))/(op_insertion+rt))*(-sin(ecc_anom));
	vel.y = ((sqrt(mu*a))/(op_insertion+rt))*(sqrt(1-(ecc*ecc)))*(cos(ecc_anom));
	vel.z = 0;

	VECTOR3 spec_ang_mom = crossp(pos, vel);
	double h = length(spec_ang_mom);
	double r = (op_insertion+rt);
	double v = length(vel);

	flight_path_angle = acos(h/(r*v));

	//double vx = v*(h/(r*v));
	//double vy = v*(sin(flight_path_angle));

	return v;
}
else return sqrt(mu*(2/(abside+rt)-1/(0.5*(tgtapo+tgtperi+2*rt))));

}

void f9h_stg2::CalcFlightVector()
{
	//double bstr_sinpangle = 0;
	double initial_pitch = RAD*(77.00+(2.5*((ap_tgt - 150000)*0.00001))); //77.25

	double fuel_res_fctr = sin(PI05*fuel_reserve/200000);
	if (fuel_res_fctr < 0)fuel_res_fctr = 0;
	else if (fuel_res_fctr > 1)fuel_res_fctr = 1;
	fuel_res_fctr=2.5*fuel_res_fctr*RAD; // 3

	initial_pitch = initial_pitch + fuel_res_fctr;

	double azmth = CalcAzimuth();
	double azmth_fctr = RAD*(-0.25*(sin(azmth)));

	initial_pitch = initial_pitch + azmth_fctr;

	double pld_factor = 2*RAD*(pld_mass - 45000)*0.00001;
	initial_pitch = initial_pitch + pld_factor;

	if (ap_function == 2) 
	{
		VECTOR3 haspdv = _V(0,1,0);
		GetGroundspeedVector (FRAME_HORIZON, haspdv);
		double mass = GetMass();
		VECTOR3 thrustvector = _V(0,0,0);
		GetThrustVector(thrustvector);
		double trst = length(thrustvector);
		double acc = trst/mass;
		double time_avail = (200-length(haspdv))/(acc-9.8);
		double pitch_rate = ((pitch_angle)-initial_pitch)/time_avail;
		if(time_avail < 0)pitch_rate = 0;
		pitch_angle = pitch_angle - (step_time*pitch_rate);

		//sprintf(oapiDebugString(), "initial_pitch %0.4f pitch_angle %0.4f", initial_pitch*DEG,pitch_angle*DEG);

		if (length(haspdv) > 245)ap_function = 3;
	}
	
	
	else if (ap_function == 3)
	{
		VECTOR3 haspdv = _V(0,1,0);
		GetGroundspeedVector (FRAME_HORIZON, haspdv);
		double xzlngth = sqrt((haspdv.x*haspdv.x)+(haspdv.z*haspdv.z));
		if (haspdv.y < 0)haspdv.y = 0;
		pitch_angle = atan2(haspdv.y,xzlngth);
		pitch_angle = pitch_angle;
		if (GetAltitude() > 50000) ap_function = 4;
		//if(eng_status == 1)ap_function = 4;
		vrot_lim = 2*RAD;
	}

	
	else if (ap_function == 4)
	{
		//double ap_dist = 0;
		//GetApDist(ap_dist);
		
		double englev = 1;//GetThrusterGroupLevel(THGROUP_MAIN);
		//if (englev > 0.45 && eng_status == 0)englev = 0.45;
		//if (eng_status == 0)englev = 0.45;
		double fuel_remaining = GetPropellantMass(hProp0) - ((2*booster_fuel_reserve)+fuel_reserve);
		if(eng_status == 1)fuel_remaining = GetPropellantMass(hProp1) - fuel_reserve;
		double isp_curr = GetThrusterIsp(th_main[eng_status]);
		double th_max = GetThrusterMax(th_main[eng_status]);
		if(eng_status == 0)th_max = th_max;
		double fuel_rate = englev*th_max/isp_curr;
		double mintime = fuel_remaining/fuel_rate;
		if (mintime < 0)mintime = 0;
		/////////////////////////////
		double sinp = sin(GetPitch());
		if(eng_status == 0) 
		{
		
		double stick_th_max = th_max/3;
		double stick_fr_max = stick_th_max/isp_curr;
		double bstr_fuel_total = booster_fuel - booster_fuel_reserve;
		double bstr_fuel_100 = booster_fuel - booster_fuel_reserve - 42000;
		if(bstr_fuel_100 < 0)bstr_fuel_100 = 0;
		double bstr_fuel_60 = booster_fuel - bstr_fuel_100 - booster_fuel_reserve - 24000;
		if(bstr_fuel_60 < 0)bstr_fuel_60 = 0;
		double bstr_fuel_40 = booster_fuel - bstr_fuel_100 - bstr_fuel_60 - booster_fuel_reserve;
		if(bstr_fuel_40 < 0)bstr_fuel_40 = 0;

		double stick_fr_max_inv_100 = 1/stick_fr_max;
		double stick_fr_max_inv_60 = 1/(stick_fr_max*0.6);
		double stick_fr_max_inv_40 = 1/(stick_fr_max*0.4);
		double bstr_bt = (bstr_fuel_100*stick_fr_max_inv_100) + (bstr_fuel_60*stick_fr_max_inv_60) + (bstr_fuel_40*stick_fr_max_inv_40);

		double core_fuel_total = stick_fr_max*0.3*bstr_bt;

		double core_fuel_rem = core_fuel - core_fuel_total - fuel_reserve;

		double core_bt = core_fuel_rem/stick_fr_max;

		mintime = (core_bt + (bstr_bt*1.0));

	//	double mass = GetMass();
	//	double bstr_sep_mass = mass - core_fuel_total - (2*bstr_fuel_total);
	//	double dv_bstr = isp_curr*(log(mass/bstr_sep_mass));

	//	double post_sep_mass = mass - STG0DRYMASS - GetPropellantMass(hProp0) + core_fuel_rem + fuel_reserve;
	//	double meco_mass = mass - STG0DRYMASS - GetPropellantMass(hProp0) + fuel_reserve;
	//	double dv_core = isp_curr*(log(post_sep_mass/meco_mass));

	//	double dv_total = dv_bstr + dv_core;
	//	//sprintf(oapiDebugString(), "dv_bstr %0.1f dv_core %0.1f", dv_bstr, dv_core);

	//	double s = ap_tgt - GetAltitude();
	//	double g = 9.81;
	//	VECTOR3 haspdv = _V(0,1,0);
	//	GetGroundspeedVector (FRAME_HORIZON, haspdv);
	//	double v0 = haspdv.y;

	//	bstr_sinpangle = (2*(s - (v0*mintime)) + (g*mintime*mintime))/(mintime*dv_total);
	////	sprintf(oapiDebugString(), "bstr_sinpangle %0.4f", bstr_sinpangle);
	////	double kfact = 0.000009;
	////	double kkonst = 0.78;//0.63;//0.78;
	////	//double core_factor = (0.3*36000/booster_fuel_reserve);
	////	double core_factor = (kkonst - (kfact*booster_fuel_reserve));
	////	core_factor = core_factor + 0.8;
	////	if(core_factor > 1.4)core_factor = 1.4;//1.0/////////////////////////////////////////////
	////	mintime = (mintime * core_factor) + 0;//+20

	//	//sprintf(oapiDebugString(), "mintime %0.1f", mintime);
	//	//sprintf(oapiDebugString(), "core_bt %0.1f bstr_bt %0.1f", core_bt,bstr_bt);
		}
		
		/////////////////////////

		VECTOR3 haspdv = _V(0,1,0);
		GetGroundspeedVector (FRAME_HORIZON, haspdv);
		//double xzlngth = sqrt((haspdv.x*haspdv.x)+(haspdv.z*haspdv.z));
		if (haspdv.y < 0)haspdv.y = 0;

		///////////////////////////////////////////////////////
		double vmass = GetMass();

		OBJHANDLE hearth=GetSurfaceRef();
		VECTOR3 relvel = _V(0,0,0);
		GetRelativeVel(hearth,relvel);
		double orb_vel = length(relvel);
		double orb_grnd_spd = sqrt((orb_vel*orb_vel)-(haspdv.y*haspdv.y));
		double circ = PI2*(rt+GetAltitude());
		double ang_vel = PI2*orb_grnd_spd/circ;
		VECTOR3 angmomvect = _V(0,(-1*ang_vel),0);
		VECTOR3 velvect = _V(0,0,orb_grnd_spd);
		VECTOR3 corvect = crossp(angmomvect,velvect);
		double acc_cor = length(corvect);

		//double sinp = sin(GetPitch());
		double dv_grnd = isp_curr*log(vmass/(vmass-fuel_remaining));
		dv_grnd = dv_grnd*sinp;

		double final_orb_grnd_spd = orb_grnd_spd+dv_grnd;
		velvect = _V(0,0,final_orb_grnd_spd);
		VECTOR3 final_corvect = crossp(angmomvect,velvect);
		double final_acc_cor = length(corvect);

		VECTOR3 weight = _V(0,0,0);
		GetWeightVector(weight);
		
		double gravt = length(weight);
		double gg = (gravt/vmass)-final_acc_cor;
		double aa = (mintime*mintime)/(2*gg);
		double bb = (haspdv.y*mintime/gg)+((mintime*mintime)/2);
		double cc = -((ap_tgt*0.99) - GetAltitude(ALTMODE_MEANRAD) - (haspdv.y*mintime) - ((haspdv.y*haspdv.y)/(2*gg)) );

		double req_acc_1 = (-bb+(sqrt((bb*bb)-(4*aa*cc))))/(2*aa);
		double req_acc_2 = (-bb-(sqrt((bb*bb)-(4*aa*cc))))/(2*aa);

		double curr_acc = th_max*englev/(vmass-(fuel_remaining*0.5));

		if(eng_status == 0)
		{
			th_max = GetThrusterMax(th_main[1]);
			vmass = vmass - STG0DRYMASS - (booster_fuel*2.5) + (fuel_reserve*0.25) + (booster_fuel_reserve*0.25);
			curr_acc = th_max/(vmass);
		}


		if (eng_status == 1)
		{
		if (fuel_remaining < (0.5*fuel_rate*step_time))
			{
			ap_function = 5;
			//sprintf(oapiDebugString(), "frate %0.1f st %0.4f frem %0.1f", fuel_rate, step_time, fuel_remaining);
			}
		}
		double total_req_acc = req_acc_1+gg-acc_cor;

		double psc = total_req_acc/curr_acc;
		if (psc > 0.985)psc = 0.985;
		else if (psc < -0.985)psc = -0.985;
		double pangle = asin(psc);
		
	//	if (eng_status == 0)
	//	{
	//		pangle = asin(bstr_sinpangle);
	//	}
		//sprintf(oapiDebugString(), "mintime %0.1f", mintime);
		
		if (mintime < 3)pitch_angle = pitch_angle;
		else pitch_angle = pangle;

		//sprintf(oapiDebugString(), "af %d bias %0.4f reqaprt %0.2f aprate %0.2f", ap_function, bias, req_ap_rate, ap_rate);
		//sprintf(oapiDebugString(), "pitch_angle %0.4f mintime %0.4f", pitch_angle*DEG,mintime);

	}

		
	vazim2 = _V(vazim1.x, (tan(pitch_angle)), vazim1.z);

	if (ap_function < 2)
	{
	VECTOR3 haspdv = _V(0,1,0);
	GetGroundspeedVector (FRAME_HORIZON, haspdv);
	vazim2 = _V(-haspdv.x*0.5,haspdv.y,-haspdv.z*0.5);
	normalise(vazim2);
	if (haspdv.y > 60)ap_function = 2;
	}

	HorizonInvRot(vazim2,flight_vector);

	//sprintf(oapiDebugString(), "af %d", ap_function);
}

void f9h_stg2::RollToAz()
{
DeactivateNavmode(NAVMODE_KILLROT);
double roll_angle = atan2(azim_local.x,-azim_local.y);
if (roll_angle > PI) roll_angle = roll_angle - PI2;
else if (roll_angle < -PI) roll_angle = roll_angle + PI2;

	double mass = GetMass();

	double max_rcs = GetThrusterMax(th_rcs0[0]);
	double max_torque = 2*max_rcs*4;
	double inertia = 14*mass;

	if (eng_status == 1)
	{
	max_rcs = GetThrusterMax(th_rcs[0]);
	max_torque = 2*max_rcs*4;
	inertia = 1.9*mass;
	}

	else if (eng_status == 2)
	{
	max_rcs = GetThrusterMax(th_rcs[19]);
	max_torque = 2*max_rcs*((1.706*0.866)-(0.123*0.25));
	inertia = 1.9*mass;
	}
	VECTOR3 avel = _V(0,0,0);
	GetAngularVel(avel);


	double max_acc = max_torque/inertia;

	double max_drot = max_acc*step_time;

	double reqzv = 0.6*(sqrt(abs(roll_angle*max_acc*2)));//0.9*
	if (roll_angle > 0)reqzv = -reqzv;

	double req_drot = reqzv - avel.z;

	double thz_lvl = req_drot/max_drot;


	if((roll_angle*DEG) < 1 && (roll_angle*DEG) > -1)
	{
	reqzv = 1*-roll_angle;
	double acc_reqz = (reqzv-avel.z)/0.1;
	thz_lvl = acc_reqz/max_acc;
	}


	if(thz_lvl > 1)thz_lvl = 1;
	else if(thz_lvl < -1)thz_lvl = -1;


	if(thz_lvl > 0)
	{
		SetThrusterGroupLevel(THGROUP_ATT_BANKRIGHT, thz_lvl);
		SetThrusterGroupLevel(THGROUP_ATT_BANKLEFT, 0);
	}
	else if(thz_lvl < 0)
	{
		SetThrusterGroupLevel(THGROUP_ATT_BANKLEFT, -thz_lvl);
		SetThrusterGroupLevel(THGROUP_ATT_BANKRIGHT, 0);
	}
	else 
	{
		SetThrusterGroupLevel(THGROUP_ATT_BANKRIGHT, 0);
		SetThrusterGroupLevel(THGROUP_ATT_BANKLEFT, 0);
	}

	/*if (ap_function == 1)
	{

		if (roll_angle > -0.6 && roll_angle < 0.6)
		{
		VECTOR3 aspdv = _V(0,1,0);
		GetAirspeedVector(FRAME_HORIZON, aspdv);
		if (aspdv.y > 60)ap_function = 2;
		
		}
	}*/
}



void f9h_stg2::GoToFlightVector()
{
	//VECTOR3 wv = _V(0,0,0);
	//GetWeightVector(wv);
	//wv = -wv;
	
	//double angoa = atan2(-wv.y, wv.z);
	//double slipage = atan2(-wv.x, wv.z);
	DeactivateNavmode(NAVMODE_KILLROT);

	double angoa = atan2(-flight_vector.y, flight_vector.z);
	double slipage = atan2(flight_vector.x, flight_vector.z);

	if(angoa > PI)angoa = (PI*2)-angoa;
	else if(angoa < -PI)angoa = (PI*2)+angoa;

	if(slipage > PI)slipage = (PI*2)-slipage;
	else if(slipage < -PI)slipage = (PI*2)+slipage;

	double mass = GetMass();

	double max_rcs = GetThrusterMax(th_rcs0[0]);
	double max_torque = 2*max_rcs*21;
	double inertiax = 129*mass;
	double inertiay = 139*mass;

	if (eng_status == 1)
	{
	max_rcs = GetThrusterMax(th_rcs[0]);
	max_torque = 2*max_rcs*21;
	inertiax = 129*mass;
	inertiay = inertiax;
	}

	else if (eng_status == 2)
	{
	max_rcs = GetThrusterMax(th_rcs[13]);
	max_torque = 1*max_rcs*9.8;
	inertiax = 18*mass;
	inertiay = inertiax;
	if (rcs_sw2 == 1)
		{
		max_rcs = GetThrusterMax(th_rcs[16]);
		max_torque = 2*max_rcs*((4.365*0.866)-(0.123*0.433));
		}
	}

	VECTOR3 avel = _V(0,0,0);
	GetAngularVel(avel);
	double max_accx = max_torque/inertiax;
	double max_drotx = max_accx*step_time;

	double max_accy = max_torque/inertiay;
	double max_droty = max_accy*step_time;

	double reqxv = 0.9*(sqrt(abs(angoa*max_accx*2)));
	if (reqxv > vrot_lim)reqxv = vrot_lim;
	if (angoa > 0)reqxv = -reqxv;
	double req_drotx = reqxv - avel.x;
	double thx_lvl = req_drotx/max_drotx;
	if((angoa*DEG) < 1 && (angoa*DEG) > -1)
	{
	reqxv = 1*-angoa;
	double acc_reqx = (reqxv-avel.x)/0.1;
	thx_lvl = acc_reqx/max_accx;
	}

	double reqyv = 0.9*(sqrt(abs(slipage*max_accy*2)));
	if (reqyv > vrot_lim)reqyv = vrot_lim;
	if (slipage > 0)reqyv = -reqyv;
	double req_droty = reqyv - avel.y;
	double thy_lvl = req_droty/max_droty;
	if((slipage*DEG) < 1 && (slipage*DEG) > -1)
	{
	reqyv = 1*-slipage;
	double acc_reqy = (reqyv-avel.y)/0.1;
	thy_lvl = acc_reqy/max_accy;
	}

	//sprintf(oapiDebugString(), "slip %0.4f yrot %0.4f reqyv%0.4f", (slipage*DEG),(avel.y*DEG), (reqyv*DEG));


	if(thx_lvl > 1)thx_lvl = 1;
	else if(thx_lvl < -1)thx_lvl = -1;

	if(thy_lvl > 1)thy_lvl = 1;
	else if(thy_lvl < -1)thy_lvl = -1;


	if(thx_lvl > 0)
		{
		SetThrusterGroupLevel(THGROUP_ATT_PITCHUP, thx_lvl);
		SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN, 0);
		}
	else if(thx_lvl < 0)
	{
		SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN, -thx_lvl);
		SetThrusterGroupLevel(THGROUP_ATT_PITCHUP, 0);
	}
	else 
	{
		SetThrusterGroupLevel(THGROUP_ATT_PITCHUP, 0);
		SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN, 0);
	}


	if(thy_lvl > 0)
		{
		SetThrusterGroupLevel(THGROUP_ATT_YAWLEFT, thy_lvl);
		SetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT, 0);
		}
	else if(thy_lvl < 0)
		{
		SetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT, -thy_lvl);
		SetThrusterGroupLevel(THGROUP_ATT_YAWLEFT, 0);
		}
	else 
	{
		SetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT, 0);
		SetThrusterGroupLevel(THGROUP_ATT_YAWLEFT, 0);
	}


}

bool f9h_stg2::clbkLoadGenericCockpit ()
{
	SetCamDir();
return true;
}

void f9h_stg2::SetCamDir()
{
	VECTOR3 camdefdir = _V(0,0,1);
	if(cam_status == 1)camdefdir = _V(0,0,-1);
	SetCameraDefaultDirection(camdefdir, 0);
	oapiCameraAttach(GetHandle(), 2);
}

void f9h_stg2::CalcFuel()
{
	if(eng_status == 1)core_fuel = GetPropellantMass(hProp1);
	else
	{
	

	double max_flow_rate = GetThrusterMax (th_main[1])/GetThrusterIsp (th_main[1]);
	double flow_rate_core = max_flow_rate*core_throttle;
	double flow_rate_booster = max_flow_rate*bstr_throttle;

	double core_dfuel = step_time*flow_rate_core;
	core_fuel = core_fuel - core_dfuel;
	//sprintf(oapiDebugString(), "corefuel %0.1f", core_fuel);

	booster_fuel = GetPropellantMass(hProp0) - core_fuel;
	booster_fuel = booster_fuel*0.5;

	//sprintf(oapiDebugString(), "cf %0.1f  bf %0.1f", core_throttle_hud*100, bstr_throttle_hud*100);
	}
}

void f9h_stg2::DoOrbit()
{
	double englevel = GetThrusterLevel(th_main[2]);
	
	

	if (start_sw == 0)
	{
	timer = timer+step_time;
	if (timer < 5) return;
	else
		{
		start_sw = 1;
		timer = 0;
		SetThrusterLevel(th_main[2],1);
		englevel = 1;
		DeactivateNavmode(NAVMODE_KILLROT);
		}
	}

	else if (start_sw == 1 && intfair_status == 0)
	{
	timer = timer+step_time;

	if (timer > 5)
	{
		if(recoverable_fair == 1 && large_fair == 0)FairTargeting();
	}


	if (timer > 20)
		{
			JettisonFair();
			timer = 0;
		}
	}



CalcAzimuth();

//double fnl_vel = sqrt(mu*(2/(op_meco_alt+rt)-1/(0.5*(op_tgt+op_meco_alt+(2*rt)))));

double fnl_vel = FinalV();
double vx = fnl_vel*(cos(flight_path_angle));
double vy = fnl_vel*(sin(flight_path_angle));

//sprintf(oapiDebugString(), "fnl_vel %0.1f vx %0.1f vy %0.1f flight_path_angle %0.2f", fnl_vel, vx,vy, flight_path_angle*DEG);

double emp_mass = GetEmptyMass();
double vmass = GetMass();

OBJHANDLE hearth=GetSurfaceRef();

VECTOR3 haspdv = _V(0,0,0);
GetGroundspeedVector (FRAME_HORIZON, haspdv);
	
VECTOR3 weight = _V(0,0,0);
GetWeightVector(weight);

//centrifugal acc
	VECTOR3 relvel = _V(0,0,0);
	GetRelativeVel(hearth,relvel);
	double orb_vel = length(relvel);
	double orb_grnd_spd = sqrt((orb_vel*orb_vel)-(haspdv.y*haspdv.y));
	double rds = (rt + GetAltitude());
	double ang_vel = orb_grnd_spd/rds;
	double acc_cent_fact = 1;
	double acc_cent = ang_vel*ang_vel*rds*acc_cent_fact;

		
	double gravt = length(weight);
	double g = -(gravt/vmass)+acc_cent;

	double aa = g/2;
	double bb = haspdv.y;
	//double cc = GetAltitude()-op_meco_alt;
	double cc = GetAltitude() - op_insertion;

	//double timetoimp = (-bb-(sqrt((bb*bb)-(4*aa*cc))))/(2*aa);

	double max_th = GetThrusterMax(th_main[2]);
	double max_acc = max_th/vmass;
	double acc_lim = 50;
	if (drg > 0) acc_lim = 30;
	double max_eng_lvl = 1;
	if (max_acc > acc_lim) max_eng_lvl = acc_lim/max_acc;
	if (max_eng_lvl < 0.7)max_eng_lvl = 0.7;
	if ((fnl_vel-orb_vel) < 200)
	{
	if (max_eng_lvl > 0.7)max_eng_lvl = 0.7;
	}

	VECTOR3 zed_axis = _V(0,0,1);
	VECTOR3 zed_axis_hrz = _V(0,0,0);
	HorizonRot(zed_axis, zed_axis_hrz);
	double zed_axis_hrz_comp = sqrt((zed_axis_hrz.x*zed_axis_hrz.x)+(zed_axis_hrz.z*zed_axis_hrz.z));
	double current_pitch = atan2(zed_axis_hrz.y, zed_axis_hrz_comp);

	double sinp = sin(current_pitch);
	double cosp = cos(current_pitch);

	//double sinp = sin(GetPitch());
	//double cosp = cos(GetPitch());
	
	//double dv_req = fnl_vel - orb_grnd_spd;
	//double dv_req = fnl_vel - orb_vel;
	double dv_req = vx - orb_grnd_spd;
	double fraction = exp(dv_req/GetThrusterIsp(th_main[2]));
	double fuelmass = vmass - (vmass/fraction);

	double dth = step_time * 0.2;

	//double englevel = GetThrusterGroupLevel(THGROUP_MAIN);
	
	double fuelrate = GetThrusterMax(th_main[2])*englevel/GetThrusterIsp(th_main[2]);
	double dvtime = fuelmass/fuelrate;
	dvtime = dvtime/cosp;

	GetElements (hearth,el,&prm,0,FRAME_EQU);
	double ap_time = prm.ApT;
	//sprintf(oapiDebugString(), "dvtime %0.1f ap_time %0.1f", dvtime, ap_time);
	double ap_time_limiter = dvtime/ap_time;
	if (ap_time_limiter > 1)ap_time_limiter = 1;
	else if (ap_time_limiter < 0.7)ap_time_limiter = 0.7;
	if (ap_time_limiter < max_eng_lvl && bb > vy) max_eng_lvl = ap_time_limiter;
	

	double vvel_fnl = vy;

	//double acc_req = 2*((vvel_fnl*dvtime)+GetAltitude()-op_meco_alt)/(dvtime*dvtime);
	//double acc_req = 2*2*(op_meco_alt-GetAltitude()-(haspdv.y*dvtime))/(dvtime*dvtime);
	//double acc_req = ((6*(op_meco_alt-GetAltitude() - (haspdv.y*dvtime)))+(2*dvtime*haspdv.y))/(dvtime*dvtime);

	
	double acc_req = ((6*(op_insertion-GetAltitude() - ((haspdv.y)*dvtime)))+(2*dvtime*(haspdv.y)))/(dvtime*dvtime);
	acc_req = acc_req - g;


	//sprintf(oapiDebugString(), "acc_req %0.1f", acc_req);

	double srp = acc_req/(max_acc*englevel);
	if (srp > 1)srp = 1;
	else if (srp < -1)srp = -1;
	double req_pitch = asin(srp);
	double crp = cos(req_pitch);

	VECTOR3 fv2d = _V(0, srp, crp);
	VECTOR3 fv_hrz = _V(0,0,0);
	fv_hrz.x = sin(azimuth)*crp;
	fv_hrz.y = srp;
	fv_hrz.z = cos(azimuth)*crp;

	if(dvtime < 30 && op_insertion > op_meco_alt)
	{

	double op_meco_alt_adjust = op_meco_alt;
	if (GetAltitude() < op_meco_alt_adjust)op_meco_alt_adjust = GetAltitude();
double a = 0.5*(op_tgt+op_meco_alt_adjust+(2*rt)); // semi major axis
	double ecc = (a - (op_meco_alt_adjust+rt))/a; // eccentricity
	double ecc_anom = acos((1-((rds)/a))/ecc); // eccentric anomaly here
	double tempy = (sqrt(1 + ecc))*(sin(ecc_anom*0.5));
	double tempx = (sqrt(1 - ecc))*(cos(ecc_anom*0.5));
	double true_anom = 2*(atan2(tempy,tempx));

	VECTOR3 pos, vel;
	pos.x = (rds)*(cos(true_anom));
	pos.y = (rds)*(sin(true_anom));
	pos.z = 0;

	vel.x = ((sqrt(mu*a))/(rds))*(-sin(ecc_anom));
	vel.y = ((sqrt(mu*a))/(rds))*(sqrt(1-(ecc*ecc)))*(cos(ecc_anom));
	vel.z = 0;

	VECTOR3 spec_ang_mom = crossp(pos, vel);
	double h = length(spec_ang_mom);
	double r = (rds);
	double v = length(vel);

	flight_path_angle = acos(h/(r*v));

	double req_vx = cos(flight_path_angle)*v;
	double req_vy = sin(flight_path_angle)*v;

	VECTOR3 fin_fvr_hrz = _V(sin(azimuth)*req_vx, req_vy, cos(azimuth)*req_vx);

	VECTOR3 glob_pos;
	GetGlobalPos(glob_pos);
	VECTOR3 relvel_globpos = relvel + glob_pos;
	VECTOR3 vel_local;
	Global2Local(relvel_globpos, vel_local);
	VECTOR3 vel_hrz;
	HorizonRot(vel_local, vel_hrz);

	VECTOR3 req_fvr_hrz = fin_fvr_hrz - vel_hrz;
	double req_fvr_hrz_x = length(_V(req_fvr_hrz.x,0,req_fvr_hrz.z));
	req_fvr_hrz = _V(req_fvr_hrz_x*sin(azimuth),req_fvr_hrz.y,req_fvr_hrz_x*cos(azimuth));
	fv_hrz = req_fvr_hrz;
	fnl_vel = v;

	//sprintf(oapiDebugString(), "fnl_vel %0.1f req_fvr_hrz %0.1f %0.1f %0.1f flight_path_angle %0.2f", fnl_vel, req_fvr_hrz.x,req_fvr_hrz.y,req_fvr_hrz.z, flight_path_angle*DEG);

	}

	double max_dv = step_time*max_acc*englevel;
	double req_englevel = (fnl_vel-orb_vel)/max_dv;
	if (req_englevel < 0)
	{
		op_status = 0;
		englevel = 0;
		KillAllRCS2();
		SetThrusterLevel(th_main[2],0);
		ActivateNavmode(NAVMODE_KILLROT);
		return;
	}
	//englevel = (fnl_vel-orb_vel)/max_dv;
	if (req_englevel > max_eng_lvl)req_englevel = max_eng_lvl;
	
	if(englevel > 0)
	{
	if (englevel > req_englevel)
		{
		englevel = englevel-dth;
		if (englevel < req_englevel) englevel = req_englevel;
		}
	else if (englevel < req_englevel)
		{
		englevel = englevel+dth;
		if (englevel > req_englevel) englevel = req_englevel;
		}
	}
	else englevel = req_englevel;

	if ((fnl_vel-orb_vel) < 30)
	{
		//flight_vector = flight_vector;
		ActivateNavmode(NAVMODE_KILLROT);
		///if(englevel > 0)englevel = 0.7;
		///if (englevel > max_eng_lvl)englevel = max_eng_lvl;
		//sprintf(oapiDebugString(), "fv-ov %0.1f", (fnl_vel-orb_vel));
	}

	//else if ((fnl_vel-orb_vel) < 400)
	//{
	//	if(englevel > 0)englevel = 0.7;
	//	if (englevel > max_eng_lvl)englevel = max_eng_lvl;
	//	HorizonInvRot(fv_hrz, flight_vector);
	//	GoToFlightVector();
	//}
	else 
	{
	HorizonInvRot(fv_hrz, flight_vector);
	GoToFlightVector();
	//sprintf(oapiDebugString(), "fv_hrz %0.4f %0.4f %0.4f",fv_hrz.x, fv_hrz.y, fv_hrz.z);
	}

	if (start_sw == 0)englevel = 0;
	SetThrusterLevel(th_main[2],englevel);




}

void f9h_stg2::CalcFuelRes(void)
{
	//////// check if landed
	VESSELSTATUS vs;
	GetStatus(vs);
	if (vs.status == 0) 
	{
		fuel_res_est = 0;
		booster_fuel_res_est = 0;
		return;
	}

	

	// stg1 ap at meco in km
	double algo_ap = ap_tgt * 0.001;
	double algo_ap2 = algo_ap * algo_ap;

	///////// crossfeed compensation
	//double xfeed_factor = 0;

	//////////// tgt pos compensation
	double tgt_pos_factor = 0;
	double azm_angle = CalcAzimuth();

	VECTOR3 tgt_gpos = _V(0,0,0);
	VECTOR3 tgt_lpos = _V(0,0,0);
	VECTOR3 tgt_hpos = _V(0,0,0);

	OBJHANDLE bbtgt = oapiGetObjectByName(refname);
	if (bbtgt == NULL)bbtgt = oapiGetBaseByName(hGref, refname);
	if (bbtgt == NULL)
		{
		tgt_pos_factor = 0;
		fuel_res_est = 0;
		}
	else
		{
		oapiGetGlobalPos(bbtgt,&tgt_gpos);
		Global2Local(tgt_gpos, tgt_lpos);
		HorizonRot(tgt_lpos, tgt_hpos);
		tgt_hpos.y = 0;

		VECTOR3 bal_imp_point = _V((sin(azm_angle)*1300000), 0, (cos(azm_angle)*1300000));
		double imp_tgt_dist = dist(tgt_hpos, bal_imp_point);
		imp_tgt_dist = imp_tgt_dist*0.001;
		double imp_tgt_dist_sq = imp_tgt_dist*imp_tgt_dist;
		double imp_tgt_dist_cb = imp_tgt_dist*imp_tgt_dist*imp_tgt_dist;

		double fuel_res_est_excess = 1000*((1.276759042E-7*imp_tgt_dist_cb) - (2.004473031E-4*imp_tgt_dist_sq) + (9.942460056E-2*imp_tgt_dist));

		double stg1_apo_factor = (7.314256686E-7*algo_ap2) - (2.14728177E-3*algo_ap) + 1.353374399;

		fuel_res_est = 50000 + (fuel_res_est_excess*stg1_apo_factor);

		double stg1_apo_factor2 = (ap_tgt - 300000)*0.1; ////// further core apo adjustment!!
		if(stg1_apo_factor2 < 0)stg1_apo_factor2 = 0;

		fuel_res_est = fuel_res_est - stg1_apo_factor2;

		if (edlo_sw_core == 1)fuel_res_est = 45000;
		if (fuel_res_est > 130000)fuel_res_est = 130000;
		}

	/////////////// boosters ///////////////////////
	int both_null = 0;
	
	bbtgt = oapiGetObjectByName(refnamep);
	if (bbtgt == NULL)bbtgt = oapiGetBaseByName(hGref, refnamep);
	if (bbtgt == NULL) both_null = 1;

	OBJHANDLE bbtgt_stbd = oapiGetObjectByName(refnames);
	if (bbtgt_stbd == NULL)bbtgt_stbd = oapiGetBaseByName(hGref, refnames);
	if (bbtgt_stbd == NULL) 
		{
		if(both_null == 0)both_null = 2;
		else both_null = 3;
		}
		
	if(both_null == 3) booster_fuel_res_est = 0; // no targets
	else
	{
	VECTOR3 bal_imp_point_bstr = _V((sin(azm_angle)*440000), 0, (cos(azm_angle)*440000));
	

	double dist_port = 0;
	if(bbtgt)
		{
		oapiGetGlobalPos(bbtgt,&tgt_gpos);
		Global2Local(tgt_gpos, tgt_lpos);
		HorizonRot(tgt_lpos, tgt_hpos);
		tgt_hpos.y = 0;
		dist_port = dist(bal_imp_point_bstr, tgt_hpos);
		}

	double dist_stbd = 0;
	if(bbtgt_stbd)
		{
		oapiGetGlobalPos(bbtgt_stbd,&tgt_gpos);
		Global2Local(tgt_gpos, tgt_lpos);
		HorizonRot(tgt_lpos, tgt_hpos);
		tgt_hpos.y = 0;
		dist_stbd = dist(bal_imp_point_bstr, tgt_hpos);
		}

	double dist_max = 0;
	if (dist_stbd > dist_port)dist_max = dist_stbd;
	else dist_max = dist_port;
	dist_max = 0.001*dist_max; // km
	double dist_max_sq = dist_max*dist_max;

	double booster_fuel_res_est_excess = 1000*((3.787878788E-5*dist_max_sq) + (3.181818182E-2*dist_max) + 3.666666667);

	double booster_apo_factor = 0.01*((1.613014271E-4 * algo_ap2) - (2.462679374E-1 * algo_ap) + 138.1570328);

	double apo_fuel_adjust = ((-8.065071356E-3 * algo_ap2) + (12.31339687 * algo_ap) - 1907.851642);
	double azm_fuel_fact  = sin(azm_angle);
	if(azm_fuel_fact < 0)azm_fuel_fact = 0;
	azm_fuel_fact = azm_fuel_fact*apo_fuel_adjust;
	//booster_fuel_res_est = 1000*((0.00004033*algo_ap2) - (0.06157*algo_ap) + 49.54);

	booster_fuel_res_est = 24000 + (booster_fuel_res_est_excess * booster_apo_factor);// + azm_fuel_fact;

	double booster_apo_factor2 = (ap_tgt - 300000)*0.0333; ////// further core apo adjustment!!
	if(booster_apo_factor2 < 0)booster_apo_factor2 = 0;

	booster_fuel_res_est = booster_fuel_res_est - booster_apo_factor2;

	//booster_fuel_res_est = booster_fuel_res_est - xfeed_factor;
	if (edlo_sw_boosters == 1)booster_fuel_res_est = 22000;
	if (booster_fuel_res_est > 60000)booster_fuel_res_est = 60000;
	//else if (booster_fuel_res_est < 300)booster_fuel_res_est = 300;
	}

	if (no_legs == 1) 
	{
		fuel_res_est = 0;
	}
	else if (no_legs == 2) 
	{
		fuel_res_est = 0;
		booster_fuel_res_est = 0;
	}

	CalcAsdsPos();
}

void f9h_stg2::DoLOT(void)
{
	oapiDelAnnotation (hNote);
	HWND hDlg;
	int lot_secs = (3600*lot_hh) + (60*lot_mm) + (lot_ss) - 1;
	double current_mjd = oapiGetSimMJD();
	double ct_secs_frac = current_mjd - (int)(current_mjd);
	int ct_secs = (int)(86400*ct_secs_frac);

	//if (lot_status == 1)
	//{
	if (lot_secs < ct_secs)lot_secs = lot_secs + 86400;
	//}

	int t_minus = (int) (lot_secs - ct_secs);
	int t_minus_hh = (int)(t_minus/3600);
	int t_minus_mm = (int)((t_minus - (t_minus_hh * 3600))/60);
	int t_minus_ss = (int)(1 + t_minus - (t_minus_hh * 3600) - (t_minus_mm * 60));

char cbuf_hh[2];
char cbuf_mm[2];
char cbuf_ss[2];
char cbuf[256];

if (abs(t_minus_hh) < 10)sprintf(cbuf_hh, "0%d", t_minus_hh);
else sprintf(cbuf_hh, "%d", t_minus_hh);

if (abs(t_minus_mm) < 10)sprintf(cbuf_mm, "0%d", t_minus_mm);
else sprintf(cbuf_mm, "%d", t_minus_mm);

if (abs(t_minus_ss) < 10)sprintf(cbuf_ss, "0%d", t_minus_ss);
else sprintf(cbuf_ss, "%d", t_minus_ss);

sprintf(cbuf,"T - %s:%s:%s", cbuf_hh,cbuf_mm,cbuf_ss);

VECTOR3 col = _V(1,0.9,0);
hNote = oapiCreateAnnotation(true, 3, col);
oapiAnnotationSetPos(hNote,0.36,0.89,0.8,0.99);
oapiAnnotationSetText(hNote, cbuf);

	if (lot_status == 1)
	{
	if ((lot_secs - ct_secs) < 0) return;
	else if ((lot_secs - ct_secs) < 10) 
		{
		SendBufferedKey(OAPI_KEY_V);
		lot_status = 2;
		hDlg = oapiFindDialog (g_Param.hDLL, IDD_CTRL);
		if (hDlg) InitDialog(hDlg);
		}
	}
	if (lot_status == 2)
	{
	lot_timer = lot_timer + step_time;
	if (lot_timer > 10)
		{
		sprintf(cbuf,"T - 00:00:00");
		oapiAnnotationSetText(hNote, cbuf);
		}
	if (lot_timer > 11) 
		{
		lot_status = 3;
		lot_timer = 0;
		oapiDelAnnotation (hNote);
		hDlg = oapiFindDialog (g_Param.hDLL, IDD_CTRL);
		if (hDlg) InitDialog(hDlg);
		return;
		}
	}

}

void f9h_stg2::CalcAsdsPos(void)
{

	// stg1 ap at meco in km
	double algo_ap = ap_tgt * 0.001;
	double algo_ap2 = algo_ap * algo_ap;

	double algo_bfr = booster_fuel_reserve*0.001;
	double algo_bfr2 = algo_bfr*algo_bfr;

	//double core_range = 1000*((-2.417554316E-3*algo_ap2) + (1.98369872*algo_ap) + 1066.890325);
	//double core_range = 1000*((-1.184188526E-4*algo_ap2) + (1.96128451*algo_ap) + 1057.401788);
	double core_range = 1000*((-8.536140372E-3*algo_ap2) - (4.332049913E-2*algo_ap*algo_bfr) - (1.466666667E-1*algo_bfr2) + (6.800276681*algo_ap) + (10.76163257*algo_bfr) + 358.753); //575.753
	double bstr_range = 1000*((-3.796484809E-4*algo_ap2) + (9.182575512E-2*algo_ap) + 557.557); //435.557

	

	OBJHANDLE hEarth = GetGravityRef();
	VECTOR3 falcon_pos = _V(0,0,0);
	GetRelativePos(hEarth,falcon_pos);
	VECTOR3 falcon_pos_nm = falcon_pos;
	normalise(falcon_pos_nm);

	double azm_angle = CalcAzimuth();
	double azm_apo_factor = 0.01*RAD*((-4.032535678E-5*algo_ap2) + (6.156698435E-2*algo_ap) + 120.4607418);
	azm_angle = azm_angle+azm_apo_factor - (RAD*0.5);
	double algo_range = core_range*0.001;
	double algo_range2 = algo_range*algo_range;
	double azm_range_factor = 0.008*RAD*((-3.7925651E-3*algo_range2) + (11.42118093*algo_range) - 8422.957023);//0.01

	//double azm_cor_factor = (1.3*(abs(cos(azm_angle)))*(algo_ap/292)*RAD*edlo_sw_core);
	double azm_cor_factor = 1.3*(abs(cos(azm_angle)))*(algo_ap/292)*RAD*edlo_sw_core;
	double sin_azm = sin(azm_angle);
	azm_cor_factor = azm_cor_factor + (((algo_ap/53.182) - 4.091)*RAD*edlo_sw_core*(pow(sin_azm, 2)) );

	//azm_angle = azm_angle + azm_range_factor;// + azm_cor_factor;
	//sprintf(oapiDebugString(), "azm_range_factor %0.4f azm_cor_factor %0.4f azm_angle %0.4f algo_range %0.1f",azm_range_factor*DEG, azm_cor_factor*DEG, azm_angle*DEG, algo_range);

	VECTOR3 vel_vector_hz = _V((sin(azm_angle)),0,(cos(azm_angle)));
	VECTOR3 vel_vector_hz_nm = vel_vector_hz;
	normalise(vel_vector_hz_nm);

	VECTOR3 vel_vector_loc = _V(0,0,0);
	HorizonInvRot(vel_vector_hz, vel_vector_loc);
	VECTOR3 vel_vector_loc_nm = vel_vector_loc;
	normalise(vel_vector_loc_nm);

	VECTOR3 vel_vector = _V(0,0,0);
	GlobalRot(vel_vector_loc, vel_vector);
	VECTOR3 vel_vector_nm = vel_vector;
	normalise(vel_vector_nm);

	VECTOR3 rot_axis = crossp(falcon_pos_nm, vel_vector_nm);
	VECTOR3 rot_axis_nm = rot_axis;
	normalise(rot_axis_nm);

	

	//sprintf(oapiDebugString(), "core_range %0.2f bstr_range %0.2f",core_range, bstr_range);

	//double core_edlo_factor = 0.01 * ((5.527524515E-4*algo_ap2) - (6.000157367E-1*algo_ap) + 188.0747101);
	//double core_edlo_factor = 0.01 * ((1.634652268E-3*algo_ap2) - (1.276203122*algo_ap) + 273.2743206);
	double core_edlo_factor = 0.003 * ((1.945452578E-3*algo_ap2) - (1.506816952*algo_ap) + 304.1134815); //100,30,20//0.01
	//sprintf(oapiDebugString(), "core_edlo_factor %0.2f",core_edlo_factor*100000);
	double lofted_factor = (ap_tgt - 250000);///250!
	if(lofted_factor < 0)lofted_factor = 0;

	double azimuth_factor = abs(cos(azm_angle))*40000;

	double edlo_pld_factor = (12000 - (pld_mass))*15; // -510000 -180000
	if(edlo_pld_factor < 0)edlo_pld_factor = 0;

	double pld_mass_ltd = pld_mass;
	if(pld_mass_ltd < 16000) pld_mass_ltd = 16000; //22
	double edlo_pld_factor2 = (46000 - pld_mass_ltd)*6;
	double edlo_pld_factor3 = (46000 - pld_mass_ltd)*2;

	double core_rot_angle = (0.5053*core_range)/rt; //rads //0.4653
	if(edlo_sw_core == 1)core_rot_angle = (core_range - (2*lofted_factor) - (core_edlo_factor*90000) - (algo_range*10) + edlo_pld_factor + edlo_pld_factor2 - azimuth_factor)/rt;
	//if(edlo_sw_core == 1)core_rot_angle = (core_range - 100000)/rt;

	double frv_pld_factor = (41000 - pld_mass)*6;
	double frv_lofted_factor = 1 + ((ap_tgt - 175000)*0.000005);
	frv_pld_factor = frv_pld_factor*frv_lofted_factor;

	double core_fuel_factor = (fuel_reserve - 60000)*8;
	if(core_fuel_factor < 0)core_fuel_factor = 0;
	if(edlo_sw_core == 1)core_fuel_factor = 0;

	double frv_range = core_range + 150000 + frv_pld_factor - core_fuel_factor;
	double frv_rot_angle = (frv_range - (2*lofted_factor) - (core_edlo_factor*90000) - (algo_range*10) + edlo_pld_factor - azimuth_factor)/rt;

	double bstr_rot_angle = (bstr_range*0.55)/rt; // 0.50
	if(edlo_sw_boosters == 1)bstr_rot_angle = (bstr_range - (0.5*lofted_factor) - (core_edlo_factor*150000) + edlo_pld_factor3)/rt; //40000

	//sprintf(oapiDebugString(), "core_rot_angle %0.3f bstr_rot_angle %0.3f",core_rot_angle*DEG, bstr_rot_angle*DEG);

	double rot_angle = core_rot_angle;
	VECTOR3 rot_vec = rot_axis_nm;

	/*double ct = cos(rot_angle);
	double st = sin(rot_angle);

	double ux = rot_vec.x;
	double uy = rot_vec.y;
	double uz = rot_vec.z;
	double ux2 = ux*ux;
	double uy2 = uy*uy;
	double uz2 = uz*uz;

	MATRIX3 RG;*/
	MATRIX3 RG = rotm(rot_vec,rot_angle);

	/*RG.m11 = ux2+(1-ux2)*ct;
	RG.m12 = ux*uy*(1-ct)-uz*st;
	RG.m13 = ux*uz*(1-ct)+uy*st;
	RG.m21 = ux*uy*(1-ct)+uz*st;
	RG.m22 = uy2+(1-uy2)*ct;
	RG.m23 = uy*uz*(1-ct)-ux*st;
	RG.m31 = ux*uz*(1-ct)-uy*st;
	RG.m32 = uy*uz*(1-ct)+ux*st;
	RG.m33 = uz2+(1-uz2)*ct;*/

	VECTOR3 core_asds_vect = mul(RG, falcon_pos);

	double asds_rad = 0;

	VECTOR3 earth_pos = _V(0,0,0);
	oapiGetGlobalPos(hEarth,&earth_pos);
	core_asds_vect = core_asds_vect+earth_pos;
	oapiGlobalToEqu(hEarth,core_asds_vect, &asds_core_lng, &asds_core_lat, &asds_rad);

	/// frv
	rot_angle = frv_rot_angle;
	RG = rotm(rot_vec,rot_angle);
	VECTOR3 core_frv_vect = mul(RG, falcon_pos);
	core_frv_vect = core_frv_vect+earth_pos;
	oapiGlobalToEqu(hEarth,core_frv_vect, &frv_core_lng, &frv_core_lat, &asds_rad);

	/// bstr
	rot_angle = bstr_rot_angle;
	RG = rotm(rot_vec,rot_angle);
	/*ct = cos(rot_angle);
	st = sin(rot_angle);

	ux = rot_vec.x;
	uy = rot_vec.y;
	uz = rot_vec.z;
	ux2 = ux*ux;
	uy2 = uy*uy;
	uz2 = uz*uz;


	RG.m11 = ux2+(1-ux2)*ct;
	RG.m12 = ux*uy*(1-ct)-uz*st;
	RG.m13 = ux*uz*(1-ct)+uy*st;
	RG.m21 = ux*uy*(1-ct)+uz*st;
	RG.m22 = uy2+(1-uy2)*ct;
	RG.m23 = uy*uz*(1-ct)-ux*st;
	RG.m31 = ux*uz*(1-ct)-uy*st;
	RG.m32 = uy*uz*(1-ct)+ux*st;
	RG.m33 = uz2+(1-uz2)*ct;*/

	VECTOR3 bstr_asds_vect = mul(RG, falcon_pos);

	bstr_asds_vect = bstr_asds_vect+earth_pos;
	oapiGlobalToEqu(hEarth,bstr_asds_vect, &asds_bstr_lng, &asds_bstr_lat, &asds_rad);

	if(no_legs == 1)
	{
		asds_core_lng = 0;
		asds_core_lat = 0;
	}
	else if(no_legs == 2)
	{
		asds_core_lng = 0;
		asds_core_lat = 0;
		asds_bstr_lng = 0;
		asds_bstr_lat = 0;
	}

	if(intfair_status == 1)
	{
		frv_core_lng = 0;
		frv_core_lat = 0;
	}
	else if(recoverable_fair == 0)
	{
		frv_core_lng = 0;
		frv_core_lat = 0;
	}
	else if(large_fair == 1)
	{
		frv_core_lng = 0;
		frv_core_lat = 0;
	}
	
	//sprintf(oapiDebugString(), "bstr_asds_vect %0.4f %0.4f %0.4f core_asds_vect %0.4f %0.4f %0.4f",bstr_asds_vect.x,bstr_asds_vect.y,bstr_asds_vect.z,core_asds_vect.x,core_asds_vect.y,core_asds_vect.z);
	//sprintf(oapiDebugString(), "asdscore lng %0.3f lat %0.3f asds bstr lng %0.3f lat %0.3f core_range %0.3f azm_rng_factor %0.3f", (asds_core_lng*DEG), (asds_core_lat*DEG), (asds_bstr_lng*DEG), (asds_bstr_lat*DEG), core_range*0.001,azm_range_factor*DEG);
}


void f9h_stg2::FairTargeting(void)
{
	double alt = GetAltitude(ALTMODE_MEANRAD);

	OBJHANDLE hTarget = oapiGetObjectByName("FRV");
	if(hTarget == NULL)return;

	VECTOR3 tgt_glob_pos = _V(0,0,0);
	oapiGetGlobalPos(hTarget, &tgt_glob_pos);

	VECTOR3 tgt_loc_pos = _V(0,0,0);
	Global2Local(tgt_glob_pos,tgt_loc_pos);

	VECTOR3 tgt_hrz_pos = _V(0,0,0);
	HorizonRot(tgt_loc_pos,tgt_hrz_pos);

	VECTOR3 haspdv = _V(0,0,0);
	GetGroundspeedVector(FRAME_HORIZON,haspdv);
	double gsvx = sqrt((haspdv.x*haspdv.x)+(haspdv.z*haspdv.z));
	double vmass = GetMass();

	OBJHANDLE hearth=GetSurfaceRef();

	VECTOR3 relvelhz = _V(0,0,0);
	GetRelativeVel(hearth, relvelhz);
	double rvh = length(relvelhz);
	rvh = sqrt((rvh*rvh) - (haspdv.y*haspdv.y));
	double acc_cfg = rvh*rvh/(alt+6371010);

	double rds = 6371010+alt;
    double GM = GGRAV*oapiGetMass(hearth);
	double gravt = GM/(rds*rds);
	gravt = gravt - acc_cfg;
	double g = -gravt;
	double aa = g/2;
	double aaa = -g;
	double bb = haspdv.y;
	double cc = alt;
	double timetoimp = (-bb-(sqrt((bb*bb)-(4*aa*cc))))/(2*aa);
	double timetoimp2 = (-bb-(sqrt((bb*bb)-(4*aa*(cc-20000)))))/(2*aa);
	if (alt < 20000)timetoimp2 = 0;

	 double range1 = gsvx*timetoimp2*1.04;  // above 30km
	 
	 double k = 75*9.81/8; // 2
	 double expo = -k*(timetoimp-timetoimp2)/vmass;
	 double range2 = vmass*gsvx/k*(1-(pow(M_E, expo))); //below 25km
	 double range = (range1+range2);
	 double tgt_range = length(_V(tgt_hrz_pos.x,0,tgt_hrz_pos.z));

	 if(range > tgt_range)JettisonFair();

	
}

void f9h_stg2::clbkVisualCreated (VISHANDLE vis, int refcount)
{
dhMesh0 = GetDevMesh(vis, 0); // stg2
SetMeshMat();
}

void f9h_stg2::clbkVisualDestroyed (VISHANDLE vis, int refcount)
{
dhMesh0 = NULL; // stg2
}

void f9h_stg2::SetMeshMat()
{
double main_lvl = GetThrusterGroupLevel(THGROUP_MAIN);
double drf = step_time*0.1;
if (red_fact < main_lvl)red_fact = red_fact+drf;
else if (red_fact > main_lvl)red_fact = red_fact-drf;

if (red_fact > 1)red_fact = 1;
else if (red_fact < 0)red_fact = 0;
double yel_fact = red_fact*0.2;

MATERIAL *mat2 = oapiMeshMaterial (hMStg2, 3);

	
	mat2->emissive.r = red_fact;
	mat2->emissive.g = yel_fact;
	mat2->emissive.b = 0;
	mat2->emissive.a = 1;
	if(dhMesh0 != NULL)oapiSetMaterial(dhMesh0,3, mat2);
	
}